@@ -36,10 +36,10 @@
  * [git://git.nic.cz/knot-dns.git](git://git.nic.cz/knot-dns.git)
  *
  * \par Git repository browser
- * https://gitlab.labs.nic.cz/labs/knot/tree/libdnssec
+ * https://gitlab.labs.nic.cz/knot/knot-dns/tree/libdnssec
  *
  * \par Issue tracker
- * https://gitlab.labs.nic.cz/labs/knot/issues
+ * https://gitlab.labs.nic.cz/knot/knot-dns/issues
  *
  * \par Mailing list
  * knot-dns-users@lists.nic.cz
@@ -105,14 +105,10 @@
 #include <dnssec/binary.h>
 #include <dnssec/crypto.h>
 #include <dnssec/error.h>
-#include <dnssec/event.h>
-#include <dnssec/kasp.h>
 #include <dnssec/key.h>
 #include <dnssec/keyid.h>
-#include <dnssec/keystate.h>
 #include <dnssec/keystore.h>
 #include <dnssec/keytag.h>
-#include <dnssec/keyusage.h>
 #include <dnssec/list.h>
 #include <dnssec/nsec.h>
 #include <dnssec/random.h>
