@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,11 +16,13 @@
 
 #include <assert.h>
 
+#include "knot/common/log.h"
 #include "knot/conf/confdb.h"
 #include "knot/conf/confio.h"
+#include "knot/conf/module.h"
 #include "knot/conf/tools.h"
 
-#define FCN(io)	(io->fcn != NULL) ? io->fcn(io) : KNOT_EOK
+#define FCN(io)	(io->fcn != NULL) ? io->fcn(io) : KNOT_EOK;
 
 static void io_reset_val(
 	conf_io_t *io,
@@ -164,16 +166,16 @@ int conf_io_list(
 	if (key0 == NULL) {
 		io_reset_val(io, NULL, NULL, NULL, 0, false, NULL);
 
-		return list_section(conf()->scheme, &io->key0, io);
+		return list_section(conf()->schema, &io->key0, io);
 	}
 
-	yp_check_ctx_t *ctx = yp_scheme_check_init(conf()->scheme);
+	yp_check_ctx_t *ctx = yp_schema_check_init(&conf()->schema);
 	if (ctx == NULL) {
 		return KNOT_ENOMEM;
 	}
 
 	// Check the input.
-	int ret = yp_scheme_check_str(ctx, key0, NULL, NULL, NULL);
+	int ret = yp_schema_check_str(ctx, key0, NULL, NULL, NULL);
 	if (ret != KNOT_EOK) {
 		goto list_error;
 	}
@@ -190,7 +192,7 @@ int conf_io_list(
 
 	ret = list_section(node->item->sub_items, &io->key1, io);
 list_error:
-	yp_scheme_check_deinit(ctx);
+	yp_schema_check_deinit(ctx);
 
 	return ret;
 }
@@ -361,7 +363,9 @@ static int diff_iter_section(
 	case KNOT_EOK:
 		break;
 	case KNOT_ENOENT:
-		return KNOT_EOK;
+		// Continue to the second step.
+		ret = KNOT_EOF;
+		break;
 	default:
 		return ret;
 	}
@@ -476,7 +480,7 @@ int conf_io_diff(
 
 	// Compare all sections by default.
 	if (key0 == NULL) {
-		for (yp_item_t *i = conf()->scheme; i->name != NULL; i++) {
+		for (yp_item_t *i = conf()->schema; i->name != NULL; i++) {
 			// Skip non-group item.
 			if (i->type != YP_TGRP) {
 				continue;
@@ -495,13 +499,13 @@ int conf_io_diff(
 		return KNOT_EOK;
 	}
 
-	yp_check_ctx_t *ctx = yp_scheme_check_init(conf()->scheme);
+	yp_check_ctx_t *ctx = yp_schema_check_init(&conf()->schema);
 	if (ctx == NULL) {
 		return KNOT_ENOMEM;
 	}
 
 	// Check the input.
-	int ret = yp_scheme_check_str(ctx, key0, key1, id, NULL);
+	int ret = yp_schema_check_str(ctx, key0, key1, id, NULL);
 	if (ret != KNOT_EOK) {
 		goto diff_error;
 	}
@@ -556,7 +560,7 @@ int conf_io_diff(
 	// Compare the section with a possible identifier.
 	ret = diff_section(io);
 diff_error:
-	yp_scheme_check_deinit(ctx);
+	yp_schema_check_deinit(ctx);
 
 	return ret;
 }
@@ -676,7 +680,7 @@ int conf_io_get(
 
 	// List all sections by default.
 	if (key0 == NULL) {
-		for (yp_item_t *i = conf()->scheme; i->name != NULL; i++) {
+		for (yp_item_t *i = conf()->schema; i->name != NULL; i++) {
 			// Skip non-group item.
 			if (i->type != YP_TGRP) {
 				continue;
@@ -694,13 +698,13 @@ int conf_io_get(
 		return KNOT_EOK;
 	}
 
-	yp_check_ctx_t *ctx = yp_scheme_check_init(conf()->scheme);
+	yp_check_ctx_t *ctx = yp_schema_check_init(&conf()->schema);
 	if (ctx == NULL) {
 		return KNOT_ENOMEM;
 	}
 
 	// Check the input.
-	int ret = yp_scheme_check_str(ctx, key0, key1, id, NULL);
+	int ret = yp_schema_check_str(ctx, key0, key1, id, NULL);
 	if (ret != KNOT_EOK) {
 		goto get_error;
 	}
@@ -771,7 +775,7 @@ int conf_io_get(
 	// List the section with a possible identifier.
 	ret = get_section(txn, io);
 get_error:
-	yp_scheme_check_deinit(ctx);
+	yp_schema_check_deinit(ctx);
 
 	return ret;
 }
@@ -876,20 +880,32 @@ static int set_item(
 	}
 
 	// Postpone group callbacks to config check.
-	if (io->key0->type == YP_TGRP) {
+	if (io->key0->type == YP_TGRP && io->id_len == 0) {
 		return KNOT_EOK;
 	}
 
-	conf_check_t args = {
+	knotd_conf_check_extra_t extra = {
 		.conf = conf(),
-		.txn = conf()->io.txn,
-		.item = (io->key1 != NULL) ? io->key1 : io->key0,
+		.txn = conf()->io.txn
+	};
+	knotd_conf_check_args_t args = {
+		.item = (io->key1 != NULL) ? io->key1 :
+		         ((io->id_len == 0) ? io->key0 : io->key0->var.g.id),
+		.id = io->id,
+		.id_len = io->id_len,
 		.data = io->data.bin,
-		.data_len = io->data.bin_len
+		.data_len = io->data.bin_len,
+		.extra = &extra
 	};
 
-	// Call the item callbacks (include).
-	return conf_exec_callbacks(&args);
+	// Call the item callbacks (include, item check, mod-id check).
+	ret = conf_exec_callbacks(&args);
+	if (ret != KNOT_EOK) {
+		CONF_LOG(LOG_DEBUG, "item '%s' (%s)", args.item->name + 1,
+		         args.err_str != NULL ? args.err_str : knot_strerror(ret));
+	}
+
+	return ret;
 }
 
 int conf_io_set(
@@ -909,13 +925,13 @@ int conf_io_set(
 		return KNOT_EINVAL;
 	}
 
-	yp_check_ctx_t *ctx = yp_scheme_check_init(conf()->scheme);
+	yp_check_ctx_t *ctx = yp_schema_check_init(&conf()->schema);
 	if (ctx == NULL) {
 		return KNOT_ENOMEM;
 	}
 
 	// Check the input.
-	int ret = yp_scheme_check_str(ctx, key0, key1, id, data);
+	int ret = yp_schema_check_str(ctx, key0, key1, id, data);
 	if (ret != KNOT_EOK) {
 		goto set_error;
 	}
@@ -930,25 +946,34 @@ int conf_io_set(
 
 	// Key1 is not a group identifier.
 	if (parent != NULL) {
+		if (node->data_len == 0) {
+			ret = KNOT_YP_ENODATA;
+			goto set_error;
+		}
 		upd_type = CONF_IO_TCHANGE;
 		upd_flags |= parent->item->flags;
 		io_reset_bin(&io, parent->item, node->item, parent->id,
 		             parent->id_len, node->data, node->data_len);
-	// No key1 but a group identifier.
-	} else if (node->id_len != 0) {
-		assert(node->item->type == YP_TGRP &&
-		       (node->item->flags & YP_FMULTI) != 0);
-		assert(node->data_len == 0);
-
+	// A group identifier or whole group.
+	} else if (node->item->type == YP_TGRP) {
 		upd_type = CONF_IO_TSET;
-		upd_flags |= node->item->var.g.id->flags;
-		io_reset_bin(&io, node->item, node->item->var.g.id, node->id,
-		             node->id_len, NULL, 0);
-	// Ensure some data for non-group items (include).
-	} else if (node->item->type == YP_TGRP || node->data_len != 0) {
+		if ((node->item->flags & YP_FMULTI) != 0) {
+			if (node->id_len == 0) {
+				ret = KNOT_YP_ENOID;
+				goto set_error;
+			}
+			upd_flags |= node->item->var.g.id->flags;
+		} else {
+			ret = KNOT_ENOTSUP;
+			goto set_error;
+		}
+		assert(node->data_len == 0);
 		io_reset_bin(&io, node->item, NULL, node->id, node->id_len,
-		             node->data, node->data_len);
-	// Non-group without data.
+		             NULL, 0);
+	// A non-group item with data (include).
+	} else if (node->data_len > 0) {
+		io_reset_bin(&io, node->item, NULL, NULL, 0, node->data,
+		             node->data_len);
 	} else {
 		ret = KNOT_YP_ENODATA;
 		goto set_error;
@@ -1004,7 +1029,7 @@ int conf_io_set(
 		upd_changes(&io, upd_type, upd_flags, false);
 	}
 set_error:
-	yp_scheme_check_deinit(ctx);
+	yp_schema_check_deinit(ctx);
 
 	return ret;
 }
@@ -1088,7 +1113,7 @@ int conf_io_unset(
 
 	// Unset all sections by default.
 	if (key0 == NULL) {
-		for (yp_item_t *i = conf()->scheme; i->name != NULL; i++) {
+		for (yp_item_t *i = conf()->schema; i->name != NULL; i++) {
 			// Skip non-group item.
 			if (i->type != YP_TGRP) {
 				continue;
@@ -1107,13 +1132,13 @@ int conf_io_unset(
 		return KNOT_EOK;
 	}
 
-	yp_check_ctx_t *ctx = yp_scheme_check_init(conf()->scheme);
+	yp_check_ctx_t *ctx = yp_schema_check_init(&conf()->schema);
 	if (ctx == NULL) {
 		return KNOT_ENOMEM;
 	}
 
 	// Check the input.
-	int ret = yp_scheme_check_str(ctx, key0, key1, id, data);
+	int ret = yp_schema_check_str(ctx, key0, key1, id, data);
 	if (ret != KNOT_EOK) {
 		goto unset_error;
 	}
@@ -1132,21 +1157,17 @@ int conf_io_unset(
 		upd_flags |= parent->item->flags;
 		io_reset_bin(&io, parent->item, node->item, parent->id,
 		             parent->id_len, node->data, node->data_len);
-	// No key1 but a group identifier or whole group.
-	} else {
-		if (node->id_len != 0) {
-			assert(node->item->type == YP_TGRP &&
-			       (node->item->flags & YP_FMULTI) != 0);
-
-			upd_type = CONF_IO_TUNSET;
+	// A group identifier or whole group.
+	} else if (node->item->type == YP_TGRP) {
+		upd_type = CONF_IO_TUNSET;
+		if ((node->item->flags & YP_FMULTI) != 0) {
 			upd_flags |= node->item->var.g.id->flags;
 		}
+		assert(node->data_len == 0);
 		io_reset_bin(&io, node->item, NULL, node->id, node->id_len,
-		             node->data, node->data_len);
-	}
-
-	// Check for a non-group item.
-	if (io.key0->type != YP_TGRP) {
+		             NULL, 0);
+	// A non-group item (include).
+	} else {
 		ret = KNOT_ENOTSUP;
 		goto unset_error;
 	}
@@ -1258,7 +1279,7 @@ int conf_io_unset(
 		upd_changes(&io, upd_type, upd_flags, false);
 	}
 unset_error:
-	yp_scheme_check_deinit(ctx);
+	yp_schema_check_deinit(ctx);
 
 	return ret;
 }
@@ -1269,11 +1290,15 @@ static int check_section(
 	size_t id_len,
 	conf_io_t *io)
 {
-	conf_check_t args = {
+	knotd_conf_check_extra_t extra = {
 		.conf = conf(),
 		.txn = conf()->io.txn,
+		.check = true
+	};
+	knotd_conf_check_args_t args = {
 		.id = id,
-		.id_len = id_len
+		.id_len = id_len,
+		.extra = &extra
 	};
 
 	bool non_empty = false;
@@ -1437,8 +1462,8 @@ int conf_io_check(
 
 	int ret;
 
-	// Iterate over the scheme.
-	for (yp_item_t *item = conf()->scheme; item->name != NULL; item++) {
+	// Iterate over the schema.
+	for (yp_item_t *item = conf()->schema; item->name != NULL; item++) {
 		// Skip non-group items (include).
 		if (item->type != YP_TGRP) {
 			continue;
@@ -1475,6 +1500,7 @@ int conf_io_check(
 
 	ret = KNOT_EOK;
 check_error:
+	conf_mod_load_purge(conf(), true);
 
 	return ret;
 }
