@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,9 +32,9 @@
 #include "dnssec/key.h"
 #include "knot/conf/tools.h"
 #include "knot/conf/conf.h"
-#include "knot/conf/scheme.h"
+#include "knot/conf/module.h"
+#include "knot/conf/schema.h"
 #include "knot/common/log.h"
-#include "knot/nameserver/query_module.h"
 #include "libknot/errcode.h"
 #include "libknot/yparser/yptrafo.h"
 #include "contrib/wire_ctx.h"
@@ -50,14 +50,14 @@ static bool is_default_id(
 }
 
 int conf_exec_callbacks(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
 	if (args == NULL) {
 		return KNOT_EINVAL;
 	}
 
 	for (size_t i = 0; i < YP_MAX_MISC_COUNT; i++) {
-		int (*fcn)(conf_check_t *) = args->item->misc[i];
+		int (*fcn)(knotd_conf_check_args_t *) = args->item->misc[i];
 		if (fcn == NULL) {
 			break;
 		}
@@ -185,130 +185,14 @@ int edns_opt_to_txt(
 	YP_CHECK_RET;
 }
 
-int addr_range_to_bin(
-	YP_TXT_BIN_PARAMS)
-{
-	YP_CHECK_PARAMS_BIN;
-
-	// Format: 0 - single address, 1 - address prefix, 2 - address range.
-	uint8_t format = 0;
-
-	// Check for the "addr/mask" format.
-	const uint8_t *pos = (uint8_t *)strchr((char *)in->position, '/');
-	if (pos >= stop) {
-		pos = NULL;
-	}
-
-	if (pos != NULL) {
-		format = 1;
-	} else {
-		// Check for the "addr1-addr2" format.
-		pos = (uint8_t *)strchr((char *)in->position, '-');
-		if (pos >= stop) {
-			pos = NULL;
-		}
-		if (pos != NULL) {
-			format = 2;
-		}
-	}
-
-	// Store address1 type position.
-	uint8_t *type1 = out->position;
-
-	// Write the first address.
-	int ret = yp_addr_noport_to_bin(in, out, pos, false);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	wire_ctx_write_u8(out, format);
-
-	switch (format) {
-	case 1:
-		// Skip the separator.
-		wire_ctx_skip(in, sizeof(uint8_t));
-
-		// Write the prefix length.
-		ret = yp_int_to_bin(in, out, stop, 0, (*type1 == 4) ? 32 : 128,
-		                    YP_SNONE);
-		if (ret != KNOT_EOK) {
-			return ret;
-		}
-		break;
-	case 2:
-		// Skip the separator.
-		wire_ctx_skip(in, sizeof(uint8_t));
-
-		// Store address2 type position.
-		uint8_t *type2 = out->position;
-
-		// Write the second address.
-		ret = yp_addr_noport_to_bin(in, out, stop, false);
-		if (ret != KNOT_EOK) {
-			return ret;
-		}
-
-		// Check for address mismatch.
-		if (*type1 != *type2) {
-			return KNOT_EINVAL;
-		}
-		break;
-	default:
-		break;
-	}
-
-	YP_CHECK_RET;
-}
-
-int addr_range_to_txt(
-	YP_BIN_TXT_PARAMS)
-{
-	YP_CHECK_PARAMS_TXT;
-
-	// Write the first address.
-	int ret = yp_addr_noport_to_txt(in, out);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	uint8_t format = wire_ctx_read_u8(in);
-
-	switch (format) {
-	case 1:
-		// Write the separator.
-		wire_ctx_write_u8(out, '/');
-
-		// Write the prefix length.
-		ret = yp_int_to_txt(in, out, YP_SNONE);
-		if (ret != KNOT_EOK) {
-			return ret;
-		}
-		break;
-	case 2:
-		// Write the separator.
-		wire_ctx_write_u8(out, '-');
-
-		// Write the second address.
-		ret = yp_addr_noport_to_txt(in, out);
-		if (ret != KNOT_EOK) {
-			return ret;
-		}
-		break;
-	default:
-		break;
-	}
-
-	YP_CHECK_RET;
-}
-
 int check_ref(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
 	const yp_item_t *ref = args->item->var.r.ref;
 
 	// Try to find a referenced block with the id.
-	if (!conf_rawid_exists_txn(args->conf, args->txn, ref->name, args->data,
-	                           args->data_len)) {
+	if (!conf_rawid_exists_txn(args->extra->conf, args->extra->txn, ref->name,
+	                           args->data, args->data_len)) {
 		args->err_str = "invalid reference";
 		return KNOT_ENOENT;
 	}
@@ -317,7 +201,7 @@ int check_ref(
 }
 
 int check_ref_dflt(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
 	if (check_ref(args) != KNOT_EOK && !is_default_id(args->data, args->data_len)) {
 		args->err_str = "invalid reference";
@@ -328,24 +212,33 @@ int check_ref_dflt(
 }
 
 int check_modref(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
 	const yp_name_t *mod_name = (const yp_name_t *)args->data;
 	const uint8_t *id = args->data + 1 + args->data[0];
 	size_t id_len = args->data_len - 1 - args->data[0];
 
+	// Check if the module is ever available.
+	const module_t *mod = conf_mod_find(args->extra->conf, mod_name + 1,
+	                                    mod_name[0], args->extra->check);
+	if (mod == NULL) {
+		args->err_str = "unknown module";
+		return KNOT_EINVAL;
+	}
+
 	// Check if the module requires some configuration.
 	if (id_len == 0) {
-		static_module_t *mod = find_module(mod_name);
-		if (mod == NULL) {
-			return KNOT_EINVAL;
+		if (mod->api->flags & KNOTD_MOD_FLAG_OPT_CONF) {
+			return KNOT_EOK;
+		} else {
+			args->err_str = "missing module configuration";
+			return KNOT_YP_ENOID;
 		}
-
-		return mod->opt_conf ? KNOT_EOK : KNOT_YP_ENOID;
 	}
 
 	// Try to find a module with the id.
-	if (!conf_rawid_exists_txn(args->conf, args->txn, mod_name, id, id_len)) {
+	if (!conf_rawid_exists_txn(args->extra->conf, args->extra->txn, mod_name,
+	                           id, id_len)) {
 		args->err_str = "invalid module reference";
 		return KNOT_ENOENT;
 	}
@@ -353,28 +246,41 @@ int check_modref(
 	return KNOT_EOK;
 }
 
+int check_module_id(
+	knotd_conf_check_args_t *args)
+{
+	const size_t len = strlen(KNOTD_MOD_NAME_PREFIX);
+
+	if (strncmp((const char *)args->id, KNOTD_MOD_NAME_PREFIX, len) != 0) {
+		args->err_str = "required 'mod-' prefix";
+		return KNOT_EINVAL;
+	}
+
+	return KNOT_EOK;
+}
+
 int check_server(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
 	bool present = false;
 
 	conf_val_t val;
-	val = conf_get_txn(args->conf, args->txn, C_SRV, C_RATE_LIMIT);
+	val = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV, C_RATE_LIMIT);
 	if (val.code == KNOT_EOK) {
 		present = true;
 	}
 
-	val = conf_get_txn(args->conf, args->txn, C_SRV, C_RATE_LIMIT_SLIP);
+	val = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV, C_RATE_LIMIT_SLIP);
 	if (val.code == KNOT_EOK) {
 		present = true;
 	}
 
-	val = conf_get_txn(args->conf, args->txn, C_SRV, C_RATE_LIMIT_TBL_SIZE);
+	val = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV, C_RATE_LIMIT_TBL_SIZE);
 	if (val.code == KNOT_EOK) {
 		present = true;
 	}
 
-	val = conf_get_txn(args->conf, args->txn, C_SRV, C_RATE_LIMIT_WHITELIST);
+	val = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV, C_RATE_LIMIT_WHITELIST);
 	if (val.code == KNOT_EOK) {
 		present = true;
 	}
@@ -388,11 +294,11 @@ int check_server(
 }
 
 int check_keystore(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
-	conf_val_t backend = conf_rawid_get_txn(args->conf, args->txn, C_KEYSTORE,
+	conf_val_t backend = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_KEYSTORE,
 	                                        C_BACKEND, args->id, args->id_len);
-	conf_val_t config = conf_rawid_get_txn(args->conf, args->txn, C_KEYSTORE,
+	conf_val_t config = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_KEYSTORE,
 	                                       C_CONFIG, args->id, args->id_len);
 
 	if (conf_opt(&backend) == KEYSTORE_BACKEND_PKCS11 && conf_str(&config) == NULL) {
@@ -404,19 +310,28 @@ int check_keystore(
 }
 
 int check_policy(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
-	conf_val_t alg = conf_rawid_get_txn(args->conf, args->txn, C_POLICY,
+	conf_val_t alg = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
 	                                    C_ALG, args->id, args->id_len);
-	conf_val_t ksk = conf_rawid_get_txn(args->conf, args->txn, C_POLICY,
+	conf_val_t ksk = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
 	                                    C_KSK_SIZE, args->id, args->id_len);
-	conf_val_t zsk = conf_rawid_get_txn(args->conf, args->txn, C_POLICY,
+	conf_val_t zsk = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
 	                                    C_ZSK_SIZE, args->id, args->id_len);
-	conf_val_t lifetime = conf_rawid_get_txn(args->conf, args->txn, C_POLICY,
+	conf_val_t lifetime = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
 	                                    C_RRSIG_LIFETIME, args->id, args->id_len);
-	conf_val_t refresh = conf_rawid_get_txn(args->conf, args->txn, C_POLICY,
+	conf_val_t refresh = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
 	                                    C_RRSIG_REFRESH, args->id, args->id_len);
 
+	conf_val_t prop_del = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
+						 C_PROPAG_DELAY, args->id, args->id_len);
+	conf_val_t zsk_life = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
+						 C_ZSK_LIFETIME, args->id, args->id_len);
+	conf_val_t ksk_life = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
+						 C_KSK_LIFETIME, args->id, args->id_len);
+	conf_val_t dnskey_ttl = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
+						   C_DNSKEY_TTL, args->id, args->id_len);
+
 	int64_t ksk_size = conf_int(&ksk);
 	if (ksk_size != YP_NIL && !dnssec_algorithm_key_size_check(conf_opt(&alg), ksk_size)) {
 		args->err_str = "KSK key size not compatible with the algorithm";
@@ -436,20 +351,34 @@ int check_policy(
 		return KNOT_EINVAL;
 	}
 
+	int64_t prop_del_val = conf_int(&prop_del);
+	int64_t zsk_life_val = conf_int(&zsk_life);
+	int64_t ksk_life_val = conf_int(&ksk_life);
+	int64_t dnskey_ttl_val = conf_int(&dnskey_ttl);
+
+	if (zsk_life_val < 2 * prop_del_val + dnskey_ttl_val) {
+		args->err_str = "ZSK lifetime too low according to propagation delay and DNSKEY TTL";
+		return KNOT_EINVAL;
+	}
+	if (ksk_life_val != 0 && ksk_life_val < 2 * prop_del_val + 2 * dnskey_ttl_val) {
+		args->err_str = "KSK lifetime too low according to propagation delay and DNSKEY TTL";
+		return KNOT_EINVAL;
+	}
+
 	return KNOT_EOK;
 }
 
 int check_key(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
-	conf_val_t alg = conf_rawid_get_txn(args->conf, args->txn, C_KEY,
+	conf_val_t alg = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_KEY,
 	                                    C_ALG, args->id, args->id_len);
 	if (conf_val_count(&alg) == 0) {
 		args->err_str = "no key algorithm defined";
 		return KNOT_EINVAL;
 	}
 
-	conf_val_t secret = conf_rawid_get_txn(args->conf, args->txn, C_KEY,
+	conf_val_t secret = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_KEY,
 	                                       C_SECRET, args->id, args->id_len);
 	if (conf_val_count(&secret) == 0) {
 		args->err_str = "no key secret defined";
@@ -460,11 +389,11 @@ int check_key(
 }
 
 int check_acl(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
-	conf_val_t action = conf_rawid_get_txn(args->conf, args->txn, C_ACL,
+	conf_val_t action = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_ACL,
 	                                       C_ACTION, args->id, args->id_len);
-	conf_val_t deny = conf_rawid_get_txn(args->conf, args->txn, C_ACL,
+	conf_val_t deny = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_ACL,
 	                                     C_DENY, args->id, args->id_len);
 	if (conf_val_count(&action) == 0 && conf_val_count(&deny) == 0) {
 		args->err_str = "no ACL action defined";
@@ -475,9 +404,9 @@ int check_acl(
 }
 
 int check_remote(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
-	conf_val_t addr = conf_rawid_get_txn(args->conf, args->txn, C_RMT,
+	conf_val_t addr = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_RMT,
 	                                     C_ADDR, args->id, args->id_len);
 	if (conf_val_count(&addr) == 0) {
 		args->err_str = "no remote address defined";
@@ -488,7 +417,7 @@ int check_remote(
 }
 
 int check_template(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
 	// Stop if the default template.
 	if (is_default_id(args->id, args->id_len)) {
@@ -497,27 +426,31 @@ int check_template(
 
 	conf_val_t val;
 	#define CHECK_DFLT(item, name) \
-		val = conf_rawid_get_txn(args->conf, args->txn, C_TPL, item, \
-		                         args->id, args->id_len); \
+		val = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_TPL, \
+		                         item, args->id, args->id_len); \
 		if (val.code == KNOT_EOK) { \
 			args->err_str = name " in non-default template"; \
 			return KNOT_EINVAL; \
 		}
 
-	CHECK_DFLT(C_TIMER_DB, "timer database");
 	CHECK_DFLT(C_GLOBAL_MODULE, "global module");
+	CHECK_DFLT(C_TIMER_DB, "timer database path");
+	CHECK_DFLT(C_MAX_TIMER_DB_SIZE, "timer database maximum size");
 	CHECK_DFLT(C_JOURNAL_DB, "journal database path");
+	CHECK_DFLT(C_JOURNAL_DB_MODE, "journal database mode");
 	CHECK_DFLT(C_MAX_JOURNAL_DB_SIZE, "journal database maximum size");
+	CHECK_DFLT(C_KASP_DB, "KASP database path");
+	CHECK_DFLT(C_MAX_KASP_DB_SIZE, "KASP database maximum size");
 
 	return KNOT_EOK;
 }
 
 int check_zone(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
-	conf_val_t master = conf_zone_get_txn(args->conf, args->txn,
+	conf_val_t master = conf_zone_get_txn(args->extra->conf, args->extra->txn,
 	                                      C_MASTER, args->id);
-	conf_val_t dnssec = conf_zone_get_txn(args->conf, args->txn,
+	conf_val_t dnssec = conf_zone_get_txn(args->extra->conf, args->extra->txn,
 	                                      C_DNSSEC_SIGNING, args->id);
 
 	// DNSSEC signing is not possible with slave zone.
@@ -526,9 +459,9 @@ int check_zone(
 		return KNOT_EINVAL;
 	}
 
-	conf_val_t signing = conf_zone_get_txn(args->conf, args->txn,
+	conf_val_t signing = conf_zone_get_txn(args->extra->conf, args->extra->txn,
 	                                       C_DNSSEC_SIGNING, args->id);
-	conf_val_t policy = conf_zone_get_txn(args->conf, args->txn,
+	conf_val_t policy = conf_zone_get_txn(args->extra->conf, args->extra->txn,
 	                                       C_DNSSEC_POLICY, args->id);
 	if (conf_bool(&signing) && policy.code != KNOT_EOK) {
 		CONF_LOG(LOG_NOTICE, "DNSSEC policy settings in KASP database "
@@ -550,8 +483,12 @@ static int glob_error(
 }
 
 int include_file(
-	conf_check_t *args)
+	knotd_conf_check_args_t *args)
 {
+	if (args->data_len == 0) {
+		return KNOT_YP_ENODATA;
+	}
+
 	// This function should not be called in more threads.
 	static int depth = 0;
 	glob_t glob_buf = { 0 };
@@ -574,8 +511,8 @@ int include_file(
 		ret = snprintf(path, PATH_MAX, "%.*s",
 		               (int)args->data_len, args->data);
 	} else {
-		const char *file_name = args->file_name != NULL ?
-		                        args->file_name : "./";
+		const char *file_name = args->extra->file_name != NULL ?
+		                        args->extra->file_name : "./";
 		char *full_current_name = realpath(file_name, NULL);
 		if (full_current_name == NULL) {
 			ret = KNOT_ENOMEM;
@@ -619,7 +556,8 @@ int include_file(
 		}
 
 		// Include regular file.
-		ret = conf_parse(args->conf, args->txn, glob_buf.gl_pathv[i], true);
+		ret = conf_parse(args->extra->conf, args->extra->txn,
+		                 glob_buf.gl_pathv[i], true);
 		if (ret != KNOT_EOK) {
 			goto include_error;
 		}
@@ -633,3 +571,30 @@ include_error:
 
 	return ret;
 }
+
+int load_module(
+	knotd_conf_check_args_t *args)
+{
+	conf_val_t val = conf_rawid_get_txn(args->extra->conf, args->extra->txn,
+	                                    C_MODULE, C_FILE, args->id, args->id_len);
+	const char *file_name = conf_str(&val);
+
+	char *mod_name = strndup((const char *)args->id, args->id_len);
+	if (mod_name == NULL) {
+		return KNOT_ENOMEM;
+	}
+
+	int ret = conf_mod_load_extra(args->extra->conf, mod_name, file_name,
+	                              args->extra->check);
+	free(mod_name);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	// Update currently iterating item.
+	const yp_item_t *section = yp_schema_find(C_MODULE, NULL, args->extra->conf->schema);
+	assert(section);
+	args->item = section->var.g.id;
+
+	return ret;
+}
