@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,15 +16,18 @@
 
 #include <string.h>
 #include <unistd.h>
+#include <sys/time.h>
 
 #include "knot/common/log.h"
 #include "knot/common/stats.h"
 #include "knot/conf/confio.h"
 #include "knot/ctl/commands.h"
+#include "knot/dnssec/key-events.h"
 #include "knot/events/handlers.h"
 #include "knot/nameserver/query_module.h"
 #include "knot/updates/zone-update.h"
 #include "knot/zone/timers.h"
+#include "knot/zone/zonefile.h"
 #include "libknot/libknot.h"
 #include "libknot/yparser/yptrafo.h"
 #include "contrib/macros.h"
@@ -33,6 +36,9 @@
 #include "zscanner/scanner.h"
 #include "contrib/strtonum.h"
 
+#define MATCH_FILTER(args, code) ((args)->data[KNOT_CTL_IDX_FILTER] == NULL || \
+                                  strchr((args)->data[KNOT_CTL_IDX_FILTER], (code)) != NULL)
+
 void ctl_log_data(knot_ctl_data_t *data)
 {
 	if (data == NULL) {
@@ -139,7 +145,6 @@ static int zones_apply(ctl_args_t *args, int (*fcn)(zone_t *, ctl_args_t *))
 
 static int zone_status(zone_t *zone, ctl_args_t *args)
 {
-	// Zone name.
 	char name[KNOT_DNAME_TXT_MAXLEN + 1];
 	if (knot_dname_to_str(name, zone->name, sizeof(name)) == NULL) {
 		return KNOT_EINVAL;
@@ -149,105 +154,125 @@ static int zone_status(zone_t *zone, ctl_args_t *args)
 		[KNOT_CTL_IDX_ZONE] = name
 	};
 
-	// Zone type.
-	data[KNOT_CTL_IDX_TYPE] = "type";
-
-	if (zone_is_slave(conf(), zone)) {
-		data[KNOT_CTL_IDX_DATA] = "slave";
-	} else {
-		data[KNOT_CTL_IDX_DATA] = "master";
-	}
-
-	int ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_DATA, &data);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	// Zone serial.
-	data[KNOT_CTL_IDX_TYPE] = "serial";
-
+	int ret;
 	char buff[128];
-	if (zone->contents != NULL) {
-		knot_rdataset_t *soa = node_rdataset(zone->contents->apex,
-		                                     KNOT_RRTYPE_SOA);
-		ret = snprintf(buff, sizeof(buff), "%u", knot_soa_serial(soa));
-	} else {
-		ret = snprintf(buff, sizeof(buff), "none");
-	}
-	if (ret < 0 || ret >= sizeof(buff)) {
-		return KNOT_ESPACE;
-	}
+	knot_ctl_type_t type = KNOT_CTL_TYPE_DATA;
 
-	data[KNOT_CTL_IDX_DATA] = buff;
+	if (MATCH_FILTER(args, CTL_FILTER_STATUS_ROLE)) {
+		data[KNOT_CTL_IDX_TYPE] = "role";
 
-	ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_EXTRA, &data);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	// Next event.
-	data[KNOT_CTL_IDX_TYPE] = "next-event";
-
-	zone_event_type_t next_type = ZONE_EVENT_INVALID;
-	time_t next_time = zone_events_get_next(zone, &next_type);
-	if (next_type != ZONE_EVENT_INVALID) {
-		const char *next_name = zone_events_get_name(next_type);
-
-		next_time = next_time - time(NULL);
-		if (next_time < 0) {
-			ret = snprintf(buff, sizeof(buff), "%s pending",
-			               next_name);
+		if (zone_is_slave(conf(), zone)) {
+			data[KNOT_CTL_IDX_DATA] = "slave";
 		} else {
-			ret = snprintf(buff, sizeof(buff), "%s in %lldh%lldm%llds",
-			               next_name,
-			               (long long)(next_time / 3600),
-			               (long long)(next_time % 3600) / 60,
-			               (long long)(next_time % 60));
+			data[KNOT_CTL_IDX_DATA] = "master";
+		}
+
+		ret = knot_ctl_send(args->ctl, type, &data);
+		if (ret != KNOT_EOK) {
+			return ret;
+		} else {
+			type = KNOT_CTL_TYPE_EXTRA;
+		}
+	}
+
+	if (MATCH_FILTER(args, CTL_FILTER_STATUS_SERIAL)) {
+		data[KNOT_CTL_IDX_TYPE] = "serial";
+
+		if (zone->contents != NULL) {
+			knot_rdataset_t *soa = node_rdataset(zone->contents->apex,
+			                                     KNOT_RRTYPE_SOA);
+			ret = snprintf(buff, sizeof(buff), "%u", knot_soa_serial(soa));
+		} else {
+			ret = snprintf(buff, sizeof(buff), "none");
 		}
 		if (ret < 0 || ret >= sizeof(buff)) {
 			return KNOT_ESPACE;
 		}
 
 		data[KNOT_CTL_IDX_DATA] = buff;
-	} else {
-		data[KNOT_CTL_IDX_DATA] = "idle";
-	}
 
-	ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_EXTRA, &data);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	// DNSSEC re-signing time.
-	data[KNOT_CTL_IDX_TYPE] = "auto-dnssec";
-
-	conf_val_t val = conf_zone_get(conf(), C_DNSSEC_SIGNING, zone->name);
-	if (conf_bool(&val)) {
-		struct tm time_gm = { 0 };
-		time_t refresh_at = zone_events_get_time(zone, ZONE_EVENT_DNSSEC);
-		gmtime_r(&refresh_at, &time_gm);
-
-		size_t written = strftime(buff, sizeof(buff), KNOT_LOG_TIME_FORMAT,
-		                          &time_gm);
-		if (written == 0) {
-			return KNOT_ESPACE;
+		ret = knot_ctl_send(args->ctl, type, &data);
+		if (ret != KNOT_EOK) {
+			return ret;
+		} else {
+			type = KNOT_CTL_TYPE_EXTRA;
 		}
-
-		data[KNOT_CTL_IDX_DATA] = buff;
-	} else {
-		data[KNOT_CTL_IDX_DATA] = "disabled";
 	}
 
-	ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_EXTRA, &data);
-	if (ret != KNOT_EOK) {
-		return ret;
+	if (MATCH_FILTER(args, CTL_FILTER_STATUS_TRANSACTION)) {
+		data[KNOT_CTL_IDX_TYPE] = "transaction";
+		data[KNOT_CTL_IDX_DATA] = (zone->control_update != NULL) ? "open" : "none";
+		ret = knot_ctl_send(args->ctl, type, &data);
+		if (ret != KNOT_EOK) {
+			return ret;
+		} else {
+			type = KNOT_CTL_TYPE_EXTRA;
+		}
 	}
 
-	// Zone transaction.
-	data[KNOT_CTL_IDX_TYPE] = "transaction";
-	data[KNOT_CTL_IDX_DATA] = (zone->control_update != NULL) ? "open" : "none";
+	bool ufrozen = zone->events.ufrozen;
+	if (MATCH_FILTER(args, CTL_FILTER_STATUS_FREEZE)) {
+		data[KNOT_CTL_IDX_TYPE] = "freeze";
+		if (ufrozen) {
+			if (zone_events_get_time(zone, ZONE_EVENT_UTHAW) < time(NULL)) {
+				data[KNOT_CTL_IDX_DATA] = "yes";
+			} else {
+				data[KNOT_CTL_IDX_DATA] = "thawing";
+			}
+		} else {
+			if (zone_events_get_time(zone, ZONE_EVENT_UFREEZE) < time(NULL)) {
+				data[KNOT_CTL_IDX_DATA] = "no";
+			} else {
+				data[KNOT_CTL_IDX_DATA] = "freezing";
 
-	return knot_ctl_send(args->ctl, KNOT_CTL_TYPE_EXTRA, &data);
+			}
+		}
+		ret = knot_ctl_send(args->ctl, type, &data);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+	}
+
+	if (MATCH_FILTER(args, CTL_FILTER_STATUS_EVENTS)) {
+		for (zone_event_type_t i = 0; i < ZONE_EVENT_COUNT; i++) {
+			// Events not worth showing or used elsewhere.
+			if (i == ZONE_EVENT_LOAD || i == ZONE_EVENT_UFREEZE ||
+			    i == ZONE_EVENT_UTHAW) {
+				continue;
+			}
+
+			// Skip events affected by freeze.
+			if (ufrozen && ufreeze_applies(i)) {
+				continue;
+			}
+
+			data[KNOT_CTL_IDX_TYPE] = zone_events_get_name(i);
+			time_t ev_time = zone_events_get_time(zone, i);
+			if (ev_time <= 0) {
+				ret = snprintf(buff, sizeof(buff), "not scheduled");
+			} else if (ev_time <= time(NULL)) {
+				ret = snprintf(buff, sizeof(buff), "pending");
+			} else {
+				ev_time -= time(NULL);
+				ret = snprintf(buff, sizeof(buff), "in %lldh%lldm%llds",
+				               (long long)(ev_time / 3600),
+				               (long long)(ev_time % 3600) / 60,
+				               (long long)(ev_time % 60));
+			}
+			if (ret < 0 || ret >= sizeof(buff)) {
+				return KNOT_ESPACE;
+			}
+			data[KNOT_CTL_IDX_DATA] = buff;
+
+			ret = knot_ctl_send(args->ctl, type, &data);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+			type = KNOT_CTL_TYPE_EXTRA;
+		}
+	}
+
+	return KNOT_EOK;
 }
 
 static int zone_reload(zone_t *zone, ctl_args_t *args)
@@ -258,7 +283,7 @@ static int zone_reload(zone_t *zone, ctl_args_t *args)
 		return KNOT_ENOTSUP;
 	}
 
-	zone_events_schedule_now(zone, ZONE_EVENT_LOAD);
+	zone_events_schedule_user(zone, ZONE_EVENT_LOAD);
 
 	return KNOT_EOK;
 }
@@ -271,7 +296,7 @@ static int zone_refresh(zone_t *zone, ctl_args_t *args)
 		return KNOT_ENOTSUP;
 	}
 
-	zone_events_schedule_now(zone, ZONE_EVENT_REFRESH);
+	zone_events_schedule_user(zone, ZONE_EVENT_REFRESH);
 
 	return KNOT_EOK;
 }
@@ -285,20 +310,24 @@ static int zone_retransfer(zone_t *zone, ctl_args_t *args)
 	}
 
 	zone->flags |= ZONE_FORCE_AXFR;
-	zone_events_schedule_now(zone, ZONE_EVENT_REFRESH);
+	zone_events_schedule_user(zone, ZONE_EVENT_REFRESH);
 
 	return KNOT_EOK;
 }
 
 static int zone_flush(zone_t *zone, ctl_args_t *args)
 {
-	UNUSED(args);
+	if (args->data[KNOT_CTL_IDX_FILTER] != NULL &&
+	    strchr(args->data[KNOT_CTL_IDX_FILTER], CTL_FILTER_FLUSH_OUTDIR) != NULL) {
+		// ^^ this is different than macro MATCH_FILTER
+		return zone_dump_to_dir(conf(), zone, args->data[KNOT_CTL_IDX_DATA]);
+	}
 
 	if (ctl_has_flag(args->data[KNOT_CTL_IDX_FLAGS], CTL_FLAG_FORCE)) {
 		zone->flags |= ZONE_FORCE_FLUSH;
 	}
 
-	zone_events_schedule_now(zone, ZONE_EVENT_FLUSH);
+	zone_events_schedule_user(zone, ZONE_EVENT_FLUSH);
 
 	return KNOT_EOK;
 }
@@ -313,7 +342,44 @@ static int zone_sign(zone_t *zone, ctl_args_t *args)
 	}
 
 	zone->flags |= ZONE_FORCE_RESIGN;
+	zone_events_schedule_user(zone, ZONE_EVENT_DNSSEC);
+
+	return KNOT_EOK;
+}
+
+static int zone_ksk_sbm_confirm(zone_t *zone, ctl_args_t *args)
+{
+	kdnssec_ctx_t ctx = { 0 };
+
+	int ret = kdnssec_ctx_init(conf(), &ctx, zone->name, NULL);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	ret = knot_dnssec_ksk_sbm_confirm(&ctx);
+
+	kdnssec_ctx_deinit(&ctx);
+
 	zone_events_schedule_now(zone, ZONE_EVENT_DNSSEC);
+	// NOT zone_events_schedule_user(), intentionally
+
+	return ret;
+}
+
+static int zone_freeze(zone_t *zone, ctl_args_t *args)
+{
+	UNUSED(args);
+
+	zone_events_schedule_now(zone, ZONE_EVENT_UFREEZE);
+
+	return KNOT_EOK;
+}
+
+static int zone_thaw(zone_t *zone, ctl_args_t *args)
+{
+	UNUSED(args);
+
+	zone_events_schedule_now(zone, ZONE_EVENT_UTHAW);
 
 	return KNOT_EOK;
 }
@@ -916,25 +982,40 @@ static int zone_txn_unset(zone_t *zone, ctl_args_t *args)
 
 static int zone_purge(zone_t *zone, ctl_args_t *args)
 {
-	UNUSED(args);
-
 	// Abort possible editing transaction.
-	(void)zone_txn_abort(zone, args);
+	if (MATCH_FILTER(args, CTL_FILTER_PURGE_EXPIRE)) {
+		(void)zone_txn_abort(zone, args);
+	}
 
 	// Purge the zone timers.
-	memset(&zone->timers, 0, sizeof(zone->timers));
+	if (MATCH_FILTER(args, CTL_FILTER_PURGE_TIMERS)) {
+		memset(&zone->timers, 0, sizeof(zone->timers));
+	}
 
 	// Expire the zone.
-	event_expire(conf(), zone);
+	if (MATCH_FILTER(args, CTL_FILTER_PURGE_EXPIRE)) {
+		(void)event_expire(conf(), zone);
+	}
 
 	// Purge the zone file.
-	char *zonefile = conf_zonefile(conf(), zone->name);
-	(void)unlink(zonefile);
-	free(zonefile);
+	if (MATCH_FILTER(args, CTL_FILTER_PURGE_ZONEFILE)) {
+		char *zonefile = conf_zonefile(conf(), zone->name);
+		(void)unlink(zonefile);
+		free(zonefile);
+	}
 
 	// Purge the zone journal.
-	if (journal_open(zone->journal, zone->journal_db, zone->name) == KNOT_EOK) {
-		(void)scrape_journal(zone->journal);
+	if (MATCH_FILTER(args, CTL_FILTER_PURGE_JOURNAL)) {
+		if (journal_open(zone->journal, zone->journal_db, zone->name) == KNOT_EOK) {
+			(void)journal_scrape(zone->journal);
+		}
+	}
+
+	// Purge KASP DB.
+	if (MATCH_FILTER(args, CTL_FILTER_PURGE_KASPDB)) {
+		if (kasp_db_open(*kaspdb()) == KNOT_EOK) {
+			(void)kasp_db_delete_all(*kaspdb(), zone->name);
+		}
 	}
 
 	return KNOT_EOK;
@@ -948,7 +1029,7 @@ static int send_stats_ctr(mod_ctr_t *ctr, ctl_args_t *args, knot_ctl_data_t *dat
 	if (ctr->count == 1) {
 		int ret = snprintf(value, sizeof(value), "%"PRIu64, ctr->counter);
 		if (ret <= 0 || ret >= sizeof(value)) {
-			return ret;
+			return KNOT_ESPACE;
 		}
 
 		(*data)[KNOT_CTL_IDX_ID] = NULL;
@@ -980,13 +1061,13 @@ static int send_stats_ctr(mod_ctr_t *ctr, ctl_args_t *args, knot_ctl_data_t *dat
 				ret = snprintf(index, sizeof(index), "%u", i);
 			}
 			if (ret <= 0 || ret >= sizeof(index)) {
-				return ret;
+				return KNOT_ESPACE;
 			}
 
-			ret = snprintf(value, sizeof(value),  "%"PRIu64,
+			ret = snprintf(value, sizeof(value), "%"PRIu64,
 			               ctr->counters[i]);
 			if (ret <= 0 || ret >= sizeof(value)) {
-				return ret;
+				return KNOT_ESPACE;
 			}
 
 			(*data)[KNOT_CTL_IDX_ID] = index;
@@ -1019,7 +1100,7 @@ static int modules_stats(list_t *query_modules, ctl_args_t *args, knot_dname_t *
 	bool section_found = (section == NULL) ? true : false;
 	bool item_found = (item == NULL) ? true : false;
 
-	struct query_module *mod = NULL;
+	knotd_mod_t *mod = NULL;
 	WALK_LIST(mod, *query_modules) {
 		// Skip modules without statistics.
 		if (mod->stats_count == 0) {
@@ -1099,6 +1180,12 @@ static int ctl_zone(ctl_args_t *args, ctl_cmd_t cmd)
 		return zones_apply(args, zone_flush);
 	case CTL_ZONE_SIGN:
 		return zones_apply(args, zone_sign);
+	case CTL_ZONE_KSK_SBM:
+		return zones_apply(args, zone_ksk_sbm_confirm);
+	case CTL_ZONE_FREEZE:
+		return zones_apply(args, zone_freeze);
+	case CTL_ZONE_THAW:
+		return zones_apply(args, zone_thaw);
 	case CTL_ZONE_READ:
 		return zones_apply(args, zone_read);
 	case CTL_ZONE_BEGIN:
@@ -1125,13 +1212,47 @@ static int ctl_zone(ctl_args_t *args, ctl_cmd_t cmd)
 	}
 }
 
+static int server_status(ctl_args_t *args)
+{
+	const char *type = args->data[KNOT_CTL_IDX_TYPE];
+
+	if (type == NULL || strlen(type) == 0) {
+		return KNOT_EOK;
+	}
+
+	char buff[2048] = "";
+
+	int ret;
+	if (strcasecmp(type, "version") == 0) {
+		ret = snprintf(buff, sizeof(buff), "Version: %s", PACKAGE_VERSION);
+	} else if (strcasecmp(type, "workers") == 0) {
+		ret = snprintf(buff, sizeof(buff), "UDP workers: %zu, TCP workers %zu, "
+		               "background workers: %zu", conf_udp_threads(conf()),
+		               conf_tcp_threads(conf()), conf_bg_threads(conf()));
+	} else if (strcasecmp(type, "configure") == 0) {
+		ret = snprintf(buff, sizeof(buff), "%s", CONFIGURE_SUMMARY);
+	} else {
+		return KNOT_EINVAL;
+	}
+	if (ret <= 0 || ret >= sizeof(buff)) {
+		return KNOT_ESPACE;
+	}
+
+	args->data[KNOT_CTL_IDX_DATA] = buff;
+
+	return knot_ctl_send(args->ctl, KNOT_CTL_TYPE_DATA, &args->data);
+}
+
 static int ctl_server(ctl_args_t *args, ctl_cmd_t cmd)
 {
 	int ret = KNOT_EOK;
 
 	switch (cmd) {
 	case CTL_STATUS:
-		ret = KNOT_EOK;
+		ret = server_status(args);
+		if (ret != KNOT_EOK) {
+			send_error(args, knot_strerror(ret));
+		}
 		break;
 	case CTL_STOP:
 		ret = KNOT_CTL_ESTOP;
@@ -1182,6 +1303,7 @@ static int ctl_stats(ctl_args_t *args, ctl_cmd_t cmd)
 			int ret = snprintf(value, sizeof(value), "%"PRIu64,
 			                   i->val(args->server));
 			if (ret <= 0 || ret >= sizeof(value)) {
+				ret = KNOT_ESPACE;
 				send_error(args, knot_strerror(ret));
 				return ret;
 			}
@@ -1342,9 +1464,12 @@ static int ctl_conf_txn(ctl_args_t *args, ctl_cmd_t cmd)
 		// First check the database.
 		ret = conf_io_check(&io);
 		if (ret != KNOT_EOK) {
-			// Error response is already sent by the check function.
+			// A semantic error is already sent by the check function.
+			if (io.error.code != KNOT_EOK) {
+				return KNOT_EOK;
+			}
 			// No transaction abort!
-			return KNOT_EOK;
+			break;
 		}
 
 		ret = conf_io_commit(false);
@@ -1485,6 +1610,9 @@ static const desc_t cmd_table[] = {
 	[CTL_ZONE_RETRANSFER] = { "zone-retransfer", ctl_zone },
 	[CTL_ZONE_FLUSH]      = { "zone-flush",      ctl_zone },
 	[CTL_ZONE_SIGN]       = { "zone-sign",       ctl_zone },
+	[CTL_ZONE_KSK_SBM]    = { "zone-ksk-submitted", ctl_zone },
+	[CTL_ZONE_FREEZE]     = { "zone-freeze",     ctl_zone },
+	[CTL_ZONE_THAW]       = { "zone-thaw",       ctl_zone },
 
 	[CTL_ZONE_READ]       = { "zone-read",       ctl_zone },
 	[CTL_ZONE_BEGIN]      = { "zone-begin",      ctl_zone },
