@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +31,20 @@
 #define CTL_FLAG_ADD	"+"
 #define CTL_FLAG_REM	"-"
 
+#define CTL_FILTER_FLUSH_OUTDIR		'd'
+
+#define CTL_FILTER_STATUS_ROLE		'r'
+#define CTL_FILTER_STATUS_SERIAL	's'
+#define CTL_FILTER_STATUS_TRANSACTION	't'
+#define CTL_FILTER_STATUS_FREEZE	'f'
+#define CTL_FILTER_STATUS_EVENTS	'e'
+
+#define CTL_FILTER_PURGE_EXPIRE		'e'
+#define CTL_FILTER_PURGE_TIMERS		't'
+#define CTL_FILTER_PURGE_ZONEFILE	'f'
+#define CTL_FILTER_PURGE_JOURNAL	'j'
+#define CTL_FILTER_PURGE_KASPDB		'k'
+
 /*! Control commands. */
 typedef enum {
 	CTL_NONE,
@@ -46,7 +60,9 @@ typedef enum {
 	CTL_ZONE_RETRANSFER,
 	CTL_ZONE_FLUSH,
 	CTL_ZONE_SIGN,
-	CTL_ZONE_STATS,
+	CTL_ZONE_KSK_SBM,
+	CTL_ZONE_FREEZE,
+	CTL_ZONE_THAW,
 
 	CTL_ZONE_READ,
 	CTL_ZONE_BEGIN,
@@ -57,6 +73,7 @@ typedef enum {
 	CTL_ZONE_SET,
 	CTL_ZONE_UNSET,
 	CTL_ZONE_PURGE,
+	CTL_ZONE_STATS,
 
 	CTL_CONF_LIST,
 	CTL_CONF_READ,
