@@ -37,8 +37,7 @@
 #include "libknot/rrtype/soa.h"
 #include "contrib/dynarray.h"
 #include "contrib/macros.h"
-
-dynarray_define(keyptr, zone_key_t *, 1)
+#include "contrib/wire_ctx.h"
 
 typedef struct type_node {
 	node_t n;
@@ -73,11 +72,6 @@ static knot_rrset_t create_empty_rrsigs_for(const knot_rrset_t *covered)
 	return rrset_init_from(covered, KNOT_RRTYPE_RRSIG);
 }
 
-static knot_rrset_t dnskey_rrset_from_soa(const knot_rrset_t *soa)
-{
-	return rrset_init_from(soa, KNOT_RRTYPE_DNSKEY);
-}
-
 /*- private API - signing of in-zone nodes -----------------------------------*/
 
 /*!
@@ -135,7 +129,7 @@ static bool use_key(const zone_key_t *key, const knot_rrset_t *covered)
 	assert(key);
 	assert(covered);
 
-	if (!key->is_active) {
+	if (!key->is_active && !key->is_ready) {
 		return false;
 	}
 
@@ -189,7 +183,7 @@ static bool all_signatures_exist(const knot_rrset_t *covered,
  *
  * \return Dynarray of such keys.
  */
-static struct keyptr_dynarray get_matching_zone_keys(const knot_rrset_t *rrsigs,
+static keyptr_dynarray_t get_matching_zone_keys(const knot_rrset_t *rrsigs,
                                                      size_t pos, const zone_keyset_t *keys)
 {
 	assert(rrsigs && rrsigs->type == KNOT_RRTYPE_RRSIG);
@@ -208,15 +202,14 @@ static struct keyptr_dynarray get_matching_zone_keys(const knot_rrset_t *rrsigs,
  * \param expires_at  Current earliest expiration, will be updated.
  */
 static void note_earliest_expiration(const knot_rrset_t *rrsigs, size_t pos,
-                                     uint32_t *expires_at)
+                                     knot_time_t *expires_at)
 {
 	assert(rrsigs);
 	assert(expires_at);
 
-	uint32_t current = knot_rrsig_sig_expiration(&rrsigs->rrs, pos);
-	if (current < *expires_at) {
-		*expires_at = current;
-	}
+	uint32_t curr_rdata = knot_rrsig_sig_expiration(&rrsigs->rrs, pos);
+	knot_time_t current = knot_time_from_u32(curr_rdata);
+	*expires_at = knot_time_min(current, *expires_at);
 }
 
 /*!
@@ -236,7 +229,7 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
                                  const zone_keyset_t *zone_keys,
                                  const kdnssec_ctx_t *dnssec_ctx,
                                  changeset_t *changeset,
-                                 uint32_t *expires_at)
+                                 knot_time_t *expires_at)
 {
 	assert(changeset);
 
@@ -262,21 +255,22 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
 	uint16_t rrsig_rdata_count = synth_rrsig.rrs.rr_count;
 	for (uint16_t i = 0; i < rrsig_rdata_count; i++) {
 		struct keyptr_dynarray keys = get_matching_zone_keys(&synth_rrsig, i, zone_keys);
-		keyptr_dynarray_fix(&keys);
 		int endloop = 0; // 1 - continue; 2 - break
 
-		for (size_t j = 0; j < keys.size && endloop == 0; j++) {
-			if (!keys.arr[j]->is_active) {
+		dynarray_foreach(keyptr, zone_key_t *, key, keys) {
+			if (!(*key)->is_active && !(*key)->is_ready) {
 				continue;
 			}
 			result = knot_check_signature(covered, &synth_rrsig, i,
-			                              keys.arr[j]->key, keys.arr[j]->ctx, dnssec_ctx);
+			                              (*key)->key, (*key)->ctx, dnssec_ctx);
 			if (result == KNOT_EOK) {
 				// valid signature
 				note_earliest_expiration(&synth_rrsig, i, expires_at);
 				endloop = 1;
+				break;
 			} else if (result != DNSSEC_INVALID_SIGNATURE) {
 				endloop = 2;
+				break;
 			}
 		}
 		keyptr_dynarray_free(&keys);
@@ -311,11 +305,11 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
 /*!
  * \brief Add missing RRSIGs into the changeset for adding.
  *
- * \param covered    RR set with covered records.
- * \param rrsigs     RR set with RRSIGs.
- * \param zone_keys  Zone keys.
- * \param policy     DNSSEC policy.
- * \param changeset  Changeset to be updated.
+ * \param covered     RR set with covered records.
+ * \param rrsigs      RR set with RRSIGs.
+ * \param zone_keys   Zone keys.
+ * \param dnssec_ctx  DNSSEC signing context
+ * \param changeset   Changeset to be updated.
  *
  * \return Error code, KNOT_EOK if successful.
  */
@@ -438,7 +432,7 @@ static int resign_rrset(const knot_rrset_t *covered,
                         const zone_keyset_t *zone_keys,
                         const kdnssec_ctx_t *dnssec_ctx,
                         changeset_t *changeset,
-                        uint32_t *expires_at)
+                        knot_time_t *expires_at)
 {
 	assert(!knot_rrset_empty(covered));
 
@@ -499,7 +493,7 @@ static int sign_node_rrsets(const zone_node_t *node,
                             const zone_keyset_t *zone_keys,
                             const kdnssec_ctx_t *dnssec_ctx,
                             changeset_t *changeset,
-                            uint32_t *expires_at)
+                            knot_time_t *expires_at)
 {
 	assert(node);
 	assert(dnssec_ctx);
@@ -540,7 +534,7 @@ typedef struct node_sign_args {
 	const zone_keyset_t *zone_keys;
 	const kdnssec_ctx_t *dnssec_ctx;
 	changeset_t *changeset;
-	uint32_t expires_at;
+	knot_time_t expires_at;
 } node_sign_args_t;
 
 /*!
@@ -586,7 +580,7 @@ static int zone_tree_sign(zone_tree_t *tree,
                           const zone_keyset_t *zone_keys,
                           const kdnssec_ctx_t *dnssec_ctx,
                           changeset_t *changeset,
-                          uint32_t *expires_at)
+                          knot_time_t *expires_at)
 {
 	assert(zone_keys);
 	assert(dnssec_ctx);
@@ -596,7 +590,7 @@ static int zone_tree_sign(zone_tree_t *tree,
 		.zone_keys = zone_keys,
 		.dnssec_ctx = dnssec_ctx,
 		.changeset = changeset,
-		.expires_at = dnssec_ctx->now + dnssec_ctx->policy->rrsig_lifetime
+		.expires_at = knot_time_add(dnssec_ctx->now, dnssec_ctx->policy->rrsig_lifetime),
 	};
 
 	int result = zone_tree_apply(tree, sign_node, &args);
@@ -628,7 +622,7 @@ typedef struct {
  *
  * \return DNSKEY RDATA match with DNSSEC key.
  */
-static bool dnskey_rdata_match(const zone_key_t *key,
+static bool dnskey_rdata_match(zone_key_t *key,
                                const dnssec_binary_t *rdata)
 {
 	assert(key);
@@ -640,30 +634,64 @@ static bool dnskey_rdata_match(const zone_key_t *key,
 	return dnssec_binary_cmp(&dnskey_rdata, rdata) == 0;
 }
 
+static bool cds_rdata_match(zone_key_t *key,
+                            const dnssec_binary_t *rdata)
+{
+        assert(key);
+        assert(rdata);
+        dnssec_binary_t cds_rdata = { 0 };
+        int ret = zone_key_calculate_ds(key, &cds_rdata);
+        int res = dnssec_binary_cmp(&cds_rdata, rdata);
+        return (ret == KNOT_EOK && res == 0);
+}
+
+bool knot_match_key_ds(zone_key_t *key, const knot_rdata_t *rdata)
+{
+        dnssec_binary_t rdata_bin = {
+                .data = knot_rdata_data(rdata),
+		.size = knot_rdata_rdlen(rdata)
+	};
+        return cds_rdata_match(key, &rdata_bin);
+}
+
 /*!
- * \brief Check if DNSKEY is present in public zone key set.
+ * \brief Check if DNSKEY/DS is present in public zone key set.
  */
-static bool dnskey_in_keyset(const zone_keyset_t *keyset,
-                             const knot_rdata_t *dnskey)
+static bool is_from_keyset(zone_keyset_t *keyset,
+                           const knot_rdata_t *record,
+                           bool is_ds, // otherwise, it's DNSKEY
+                           bool is_cds_cdnskey, // in this case we match only ready keys
+                           zone_key_t **matching_key) // out, optional
 {
 	assert(keyset);
-	assert(dnskey);
+	assert(record);
 
 	dnssec_binary_t rdata = {
-		.data = knot_rdata_data(dnskey),
-		.size = knot_rdata_rdlen(dnskey)
+		.data = knot_rdata_data(record),
+		.size = knot_rdata_rdlen(record)
 	};
 
 	uint16_t tag = 0;
-	dnssec_keytag(&rdata, &tag);
+	bool (*match_fce)(zone_key_t *, const dnssec_binary_t *);
+	if (is_ds) {
+		wire_ctx_t wrdata = wire_ctx_init(rdata.data, rdata.size);
+		tag = wire_ctx_read_u16(&wrdata); // key tag in DS is just at beginning of wire
+		match_fce = cds_rdata_match;
+	} else {
+		dnssec_keytag(&rdata, &tag);
+		match_fce = dnskey_rdata_match;
+	}
 	bool found = false;
 
 	struct keyptr_dynarray keys = get_zone_keys(keyset, tag);
-	keyptr_dynarray_fix(&keys);
 
 	for (size_t i = 0; i < keys.size; i++) {
-		if (keys.arr[i]->is_public && dnskey_rdata_match(keys.arr[i], &rdata)) {
+		bool usekey = (is_cds_cdnskey ? (keys.arr(&keys)[i]->is_ready && !keys.arr(&keys)[i]->is_active) : keys.arr(&keys)[i]->is_public);
+		if (usekey && match_fce(keys.arr(&keys)[i], &rdata)) {
 			found = true;
+			if (matching_key != NULL) {
+				*matching_key = keys.arr(&keys)[i];
+			}
 			break;
 		}
 	}
@@ -673,31 +701,44 @@ static bool dnskey_in_keyset(const zone_keyset_t *keyset,
 }
 
 /*!
- * \brief Check if DNSKEY is present in the zone.
+ * \brief Check if DNSKEY/DS/CDNSKEY/CDS is present in the zone.
  *
- * \param dnskeys  DNSKEYS RR set in zone apex.
+ * \param records  RR set in zone apex.
  * \param key      Key to be searched for.
  * \param ttl      Key TTL.
  *
  * \return DNSKEY exists in the zone.
  */
-static bool dnskey_in_zone(const knot_rrset_t *dnskeys,
-                           const zone_key_t *key,
-                           uint32_t ttl)
+static bool is_in_zone(const knot_rrset_t *records,
+                       zone_key_t *key,
+                       uint32_t ttl)
 {
-	assert(dnskeys);
+	assert(records);
 	assert(key);
 
-	uint16_t dnskeys_rdata_count = dnskeys->rrs.rr_count;
-	for (uint16_t i = 0; i < dnskeys_rdata_count; i++) {
-		const knot_rdata_t *dnskey = knot_rdataset_at(&dnskeys->rrs, i);
-		uint32_t dnskey_ttl = knot_rdata_ttl(dnskey);
+	bool (*match_fce)(zone_key_t *, const dnssec_binary_t *);
+	switch (records->type) {
+	case KNOT_RRTYPE_DNSKEY:
+	case KNOT_RRTYPE_CDNSKEY:
+		match_fce = dnskey_rdata_match;
+		break;
+	case KNOT_RRTYPE_CDS:
+		match_fce = cds_rdata_match;
+		break;
+	default:
+		return false;
+	}
+
+	uint16_t rdata_count = records->rrs.rr_count;
+	for (uint16_t i = 0; i < rdata_count; i++) {
+		const knot_rdata_t *r = knot_rdataset_at(&records->rrs, i);
+		uint32_t r_ttl = knot_rdata_ttl(r);
 		const dnssec_binary_t rdata = {
-			.data = knot_rdata_data(dnskey),
-			.size = knot_rdata_rdlen(dnskey)
+			.data = knot_rdata_data(r),
+			.size = knot_rdata_rdlen(r)
 		};
 
-		if (dnskey_ttl == ttl && dnskey_rdata_match(key, &rdata)) {
+		if (r_ttl == ttl && match_fce(key, &rdata)) {
 			return true;
 		}
 	}
@@ -706,7 +747,7 @@ static bool dnskey_in_zone(const knot_rrset_t *dnskeys,
 }
 
 static int rrset_add_zone_key(knot_rrset_t *rrset,
-                              const zone_key_t *zone_key,
+                              zone_key_t *zone_key,
                               uint32_t ttl)
 {
 	assert(rrset);
@@ -719,47 +760,73 @@ static int rrset_add_zone_key(knot_rrset_t *rrset,
 	                            dnskey_rdata.size, ttl, NULL);
 }
 
+static int rrset_add_zone_ds(knot_rrset_t *rrset,
+                             zone_key_t *zone_key,
+                             uint32_t ttl)
+{
+	assert(rrset);
+	assert(zone_key);
+
+	dnssec_binary_t cds_rdata = { 0 };
+	zone_key_calculate_ds(zone_key, &cds_rdata);
+
+	return knot_rrset_add_rdata(rrset, cds_rdata.data,
+	                            cds_rdata.size, ttl, NULL);
+}
+
 /*!
  * \brief Remove extra DNSKEYs from the zone by updating the changeset.
  *
  * Extra DNSKEY is a key, which is not present in zone public key set.
  *
- * \param soa        RR set with SOA (to get TTL value from).
- * \param dnskeys    RR set with DNSKEYs.
- * \param keyset     Zone keys.
- * \param changeset  Changeset to be updated.
+ * \param records     RR set with DNSKEYs/CDNSKEYs/CDSs.
+ * \param keyset      Zone keys.
+ * \param changeset   Changeset to be updated.
+ * \param dnskey_ttl  DNSKEY TTL value.
  *
  * \return Error code, KNOT_EOK if successful.
  */
-static int remove_invalid_dnskeys(const knot_rrset_t *soa,
-                                  const knot_rrset_t *dnskeys,
-                                  const zone_keyset_t *keyset,
-                                  changeset_t *changeset)
+static int remove_invalid_records(const knot_rrset_t *records,
+                                  zone_keyset_t *keyset, // not const just due to pre-computing DS
+                                  changeset_t *changeset,
+                                  uint32_t dnskey_ttl)
 {
-	if (knot_rrset_empty(dnskeys)) {
+	if (knot_rrset_empty(records)) {
 		return KNOT_EOK;
 	}
 
-	assert(soa->type == KNOT_RRTYPE_SOA);
-	assert(dnskeys->type == KNOT_RRTYPE_DNSKEY);
 	assert(keyset);
 	assert(changeset);
 
-	knot_rrset_t to_remove;
-	knot_rrset_init(&to_remove, dnskeys->owner, dnskeys->type, dnskeys->rclass);
+	bool is_ds = false;
+	bool is_c = false;
 
-	const knot_rdata_t *soa_data = knot_rdataset_at(&soa->rrs, 0);
-	uint32_t soa_ttl = knot_rdata_ttl(soa_data);
+	switch (records->type) {
+	case KNOT_RRTYPE_DNSKEY:
+		break;
+	case KNOT_RRTYPE_CDNSKEY:
+		is_c = true;
+		break;
+	case KNOT_RRTYPE_CDS:
+		is_ds = true;
+		is_c = true;
+		break;
+	default:
+		return KNOT_EINVAL;
+	}
+
+	knot_rrset_t to_remove;
+	knot_rrset_init(&to_remove, records->owner, records->type, records->rclass);
 
 	int result = KNOT_EOK;
-	for (uint16_t i = 0; i < dnskeys->rrs.rr_count; i++) {
-		const knot_rdata_t *dnskey = knot_rdataset_at(&dnskeys->rrs, i);
-		uint32_t dnskey_ttl = knot_rdata_ttl(dnskey);
-		if (dnskey_ttl == soa_ttl && dnskey_in_keyset(keyset, dnskey)) {
+	for (uint16_t i = 0; i < records->rrs.rr_count; i++) {
+		const knot_rdata_t *r = knot_rdataset_at(&records->rrs, i);
+		uint32_t r_ttl = knot_rdata_ttl(r);
+		if (r_ttl == dnskey_ttl && is_from_keyset(keyset, r, is_ds, is_c, NULL)) {
 			continue;
 		}
 
-		result = knot_rdataset_add(&to_remove.rrs, dnskey, NULL);
+		result = knot_rdataset_add(&to_remove.rrs, r, NULL);
 		if (result != KNOT_EOK) {
 			break;
 		}
@@ -774,114 +841,145 @@ static int remove_invalid_dnskeys(const knot_rrset_t *soa,
 	return result;
 }
 
+static bool publish_cds(const zone_key_t *key)
+{
+        return (key->is_ready && !key->is_active);
+}
+
 /*!
  * \brief Add missing DNSKEYs into the zone by updating the changeset.
  *
- * \param soa        RR set with SOA (to get TTL value from).
- * \param dnskeys    RR set with DNSKEYs.
- * \param keyset     Zone keys.
- * \param changeset  Changeset to be updated.
+ * \param soa         RR set with SOA (to get TTL value from).
+ * \param dnskeys     RR set with DNSKEYs.
+ * \param cdnskeys    RR set with CDNSKEYs.
+ * \param cdss        RR set with CDSs.
+ * \param keyset      Zone keys.
+ * \param changeset   Changeset to be updated.
+ * \param dnskey_ttl  DNSKEY TTL value.
  *
  * \return Error code, KNOT_EOK if successful.
  */
-static int add_missing_dnskeys(const knot_rrset_t *soa,
+static int add_missing_records(const knot_rrset_t *soa,
                                const knot_rrset_t *dnskeys,
-                               const zone_keyset_t *keyset,
-                               changeset_t *changeset)
+                               const knot_rrset_t *cdnskeys,
+                               const knot_rrset_t *cdss,
+                               zone_keyset_t *keyset,
+                               changeset_t *changeset,
+                               uint32_t dnskey_ttl)
 {
 	assert(soa && soa->type == KNOT_RRTYPE_SOA);
 	assert(knot_rrset_empty(dnskeys) || dnskeys->type == KNOT_RRTYPE_DNSKEY);
+	assert(knot_rrset_empty(cdnskeys) || cdnskeys->type == KNOT_RRTYPE_CDNSKEY);
+	assert(knot_rrset_empty(cdss) || cdss->type == KNOT_RRTYPE_CDS);
 	assert(keyset);
 	assert(changeset);
 
-	const knot_rdata_t *soa_rr = knot_rdataset_at(&soa->rrs, 0);
-	uint32_t soa_ttl = knot_rdata_ttl(soa_rr);
-
-	knot_rrset_t to_add;
-	knot_rrset_init_empty(&to_add);
+	knot_rrset_t to_add_dnskey, to_add_cdnskey, to_add_cds;
+	knot_rrset_init_empty(&to_add_dnskey);
+	knot_rrset_init_empty(&to_add_cdnskey);
+	knot_rrset_init_empty(&to_add_cds);
 
 	int result = KNOT_EOK;
 	for (int i = 0; i < keyset->count; i++) {
-		const zone_key_t *key = &keyset->keys[i];
+		zone_key_t *key = &keyset->keys[i];
 		if (!key->is_public) {
 			continue;
 		}
 
-		if (dnskey_in_zone(dnskeys, key, soa_ttl)) {
-			continue;
+#define check_add(keys, toadd, rrtype, addfce, ttl) \
+		if (!is_in_zone((keys), key, ttl)) { \
+			if (knot_rrset_empty(&(toadd))) { \
+				(toadd) = rrset_init_from(soa, rrtype); \
+			} \
+			result = addfce(&(toadd), key, ttl); \
+			if (result != KNOT_EOK) { \
+				break; \
+			} \
 		}
 
-		if (knot_rrset_empty(&to_add)) {
-			to_add = dnskey_rrset_from_soa(soa);
-		}
-
-		result = rrset_add_zone_key(&to_add, key, soa_ttl);
-		if (result != KNOT_EOK) {
-			break;
+		check_add(dnskeys, to_add_dnskey, KNOT_RRTYPE_DNSKEY, rrset_add_zone_key, dnskey_ttl)
+		if (publish_cds(key)) {
+			check_add(cdnskeys, to_add_cdnskey, KNOT_RRTYPE_CDNSKEY, rrset_add_zone_key, 0)
+			check_add(cdss, to_add_cds, KNOT_RRTYPE_CDS, rrset_add_zone_ds, 0)
 		}
+#undef check_add
 	}
 
-	if (!knot_rrset_empty(&to_add) && result == KNOT_EOK) {
-		result = changeset_add_addition(changeset, &to_add, 0);
-	}
+#define flush_clear(toadd) \
+	if (!knot_rrset_empty(&(toadd)) && result == KNOT_EOK) { \
+		result = changeset_add_addition(changeset, &(toadd), 0); \
+	} \
+	knot_rdataset_clear(&(toadd).rrs, NULL);
 
-	knot_rdataset_clear(&to_add.rrs, NULL);
+	flush_clear(to_add_dnskey)
+	flush_clear(to_add_cdnskey)
+	flush_clear(to_add_cds)
+#undef flush_clear
 
 	return result;
 }
 
 /*!
- * \brief Refresh DNSKEY RRSIGs in the zone by updating the changeset.
+ * \brief Refresh DNSKEY/CDS/CDNSKEY RRSIGs in the zone by updating the changeset.
  *
- * \param dnskeys     RR set with DNSKEYs.
+ * \param rrsigs      Current RRSIG rrset.
  * \param soa         RR set with SOA.
  * \param zone_keys   Zone keys.
- * \param policy      DNSSEC policy.
+ * \param dnssec_ctx  Zone signing context.
+ * \param rrtype      Type of the key-related zone records in question.
  * \param changeset   Changeset to be updated.
  * \param expires_at  Earliest RRSIG expiration.
  *
  * \return Error code, KNOT_EOK if successful.
  */
-static int update_dnskey_rrsigs(const knot_rrset_t *dnskeys,
-                                const knot_rrset_t *rrsigs,
+static int update_record_rrsigs(const knot_rrset_t *rrsigs,
                                 const knot_rrset_t *soa,
-                                const zone_keyset_t *zone_keys,
+                                zone_keyset_t *zone_keys,
                                 const kdnssec_ctx_t *dnssec_ctx,
+                                uint16_t rrtype,
                                 changeset_t *changeset,
-                                uint32_t *expires_at)
+                                knot_time_t *expires_at)
 {
 	assert(zone_keys);
 	assert(changeset);
 
-	int result;
+	int ret = KNOT_EOK;
 
-	// We know how the DNSKEYs in zone should look like after applying
+	// We know how the DNSKEYs/CDNSKEYs/CDSs in zone should look like after applying
 	// the changeset. RRSIGs can be then built easily.
 
-	knot_rrset_t new_dnskeys = dnskey_rrset_from_soa(soa);
+	knot_rrset_t new_records = rrset_init_from(soa, rrtype);
 	for (int i = 0; i < zone_keys->count; i++) {
-		const zone_key_t *key = &zone_keys->keys[i];
+		zone_key_t *key = &zone_keys->keys[i];
 		if (!key->is_public) {
 			continue;
 		}
 
-		const knot_rdata_t *soa_data = knot_rdataset_at(&soa->rrs, 0);
-		result = rrset_add_zone_key(&new_dnskeys, key,
-		                            knot_rdata_ttl(soa_data));
-		if (result != KNOT_EOK) {
+		if (rrtype != KNOT_RRTYPE_DNSKEY && (!key->is_ready || key->is_active)) {
+			continue;
+		}
+
+		int (*addfce)(knot_rrset_t *, zone_key_t *, uint32_t) =
+		     (rrtype == KNOT_RRTYPE_CDS ? rrset_add_zone_ds : rrset_add_zone_key);
+
+		ret = addfce(&new_records, key,
+				(rrtype == KNOT_RRTYPE_DNSKEY ? dnssec_ctx->policy->dnskey_ttl : 0));
+		if (ret != KNOT_EOK) {
 			goto fail;
 		}
 	}
 
-	if (dnssec_ctx->rrsig_drop_existing) {
-		result = force_resign_rrset(&new_dnskeys, rrsigs, zone_keys, dnssec_ctx, changeset);
-	} else {
-		result = resign_rrset(&new_dnskeys, rrsigs, zone_keys, dnssec_ctx, changeset, expires_at);
+	if (!knot_rrset_empty(&new_records)) {
+		if (dnssec_ctx->rrsig_drop_existing) {
+			ret = force_resign_rrset(&new_records, rrsigs, zone_keys, dnssec_ctx, changeset);
+		} else {
+			ret = resign_rrset(&new_records, rrsigs, zone_keys, dnssec_ctx, changeset, expires_at);
+		}
 	}
 
 fail:
-	knot_rdataset_clear(&new_dnskeys.rrs, NULL);
-	return result;
+	knot_rdataset_clear(&new_records.rrs, NULL);
+	return ret;
 }
 
 /*!
@@ -896,10 +994,10 @@ fail:
  * \return Error code, KNOT_EOK if successful.
  */
 static int update_dnskeys(const zone_contents_t *zone,
-                          const zone_keyset_t *zone_keys,
+                          zone_keyset_t *zone_keys,
                           const kdnssec_ctx_t *dnssec_ctx,
                           changeset_t *changeset,
-                          uint32_t *expires_at)
+                          knot_time_t *expires_at)
 {
 	assert(zone);
 	assert(zone->apex);
@@ -907,26 +1005,57 @@ static int update_dnskeys(const zone_contents_t *zone,
 
 	const zone_node_t *apex = zone->apex;
 	knot_rrset_t dnskeys = node_rrset(apex, KNOT_RRTYPE_DNSKEY);
+	knot_rrset_t cdnskeys = node_rrset(apex, KNOT_RRTYPE_CDNSKEY);
+	knot_rrset_t cdss = node_rrset(apex, KNOT_RRTYPE_CDS);
 	knot_rrset_t soa = node_rrset(apex, KNOT_RRTYPE_SOA);
 	knot_rrset_t rrsigs = node_rrset(apex, KNOT_RRTYPE_RRSIG);
+	uint32_t dnskey_ttl = dnssec_ctx->policy->dnskey_ttl;
 	if (knot_rrset_empty(&soa)) {
 		return KNOT_EINVAL;
 	}
 
 	int result;
 
-	result = remove_invalid_dnskeys(&soa, &dnskeys, zone_keys, changeset);
+	result = remove_invalid_records(&dnskeys, zone_keys, changeset, dnskey_ttl);
 	if (result != KNOT_EOK) {
 		return result;
 	}
 
-	result = add_missing_dnskeys(&soa, &dnskeys, zone_keys, changeset);
+	result = remove_invalid_records(&cdnskeys, zone_keys, changeset, 0);
 	if (result != KNOT_EOK) {
 		return result;
 	}
 
-	return update_dnskey_rrsigs(&dnskeys, &rrsigs, &soa, zone_keys,
-	                            dnssec_ctx, changeset, expires_at);
+	result = remove_invalid_records(&cdss, zone_keys, changeset, 0);
+	if (result != KNOT_EOK) {
+		return result;
+	}
+
+	result = add_missing_records(&soa, &dnskeys, &cdnskeys, &cdss, zone_keys,
+	                             changeset, dnskey_ttl);
+	if (result != KNOT_EOK) {
+		return result;
+	}
+
+	result = update_record_rrsigs(&rrsigs, &soa, zone_keys, dnssec_ctx,
+	                              KNOT_RRTYPE_DNSKEY, changeset, expires_at);
+	if (result != KNOT_EOK) {
+		return result;
+	}
+
+	result = update_record_rrsigs(&rrsigs, &soa, zone_keys, dnssec_ctx,
+	                              KNOT_RRTYPE_CDNSKEY, changeset, expires_at);
+	if (result != KNOT_EOK) {
+		return result;
+	}
+
+	result = update_record_rrsigs(&rrsigs, &soa, zone_keys, dnssec_ctx,
+	                              KNOT_RRTYPE_CDS, changeset, expires_at);
+	if (result != KNOT_EOK) {
+		return result;
+	}
+
+	return KNOT_EOK;
 }
 
 /*!
@@ -1136,10 +1265,10 @@ static void knot_zone_clear_sorted_changes(trie_t *t)
 /*- public API ---------------------------------------------------------------*/
 
 int knot_zone_sign(const zone_contents_t *zone,
-                   const zone_keyset_t *zone_keys,
+                   zone_keyset_t *zone_keys,
                    const kdnssec_ctx_t *dnssec_ctx,
                    changeset_t *changeset,
-                   uint32_t *expire_at)
+                   knot_time_t *expire_at)
 {
 	if (!zone || !zone_keys || !dnssec_ctx || !changeset || !expire_at) {
 		return KNOT_EINVAL;
@@ -1147,28 +1276,28 @@ int knot_zone_sign(const zone_contents_t *zone,
 
 	int result;
 
-	uint32_t dnskey_expire = UINT32_MAX;
+	knot_time_t dnskey_expire = 0;
 	result = update_dnskeys(zone, zone_keys, dnssec_ctx, changeset,
 	                        &dnskey_expire);
 	if (result != KNOT_EOK) {
 		return result;
 	}
 
-	uint32_t normal_expire = UINT32_MAX;
+	knot_time_t normal_expire = 0;
 	result = zone_tree_sign(zone->nodes, zone_keys, dnssec_ctx, changeset,
 	                        &normal_expire);
 	if (result != KNOT_EOK) {
 		return result;
 	}
 
-	uint32_t nsec3_expire = UINT32_MAX;
+	knot_time_t nsec3_expire = 0;
 	result = zone_tree_sign(zone->nsec3_nodes, zone_keys, dnssec_ctx,
 	                        changeset, &nsec3_expire);
 	if (result != KNOT_EOK) {
 		return result;
 	}
 
-	*expire_at = MIN(dnskey_expire, MIN(normal_expire, nsec3_expire));
+	*expire_at = knot_time_min(dnskey_expire, knot_time_min(normal_expire, nsec3_expire));
 
 	return KNOT_EOK;
 }
