@@ -48,9 +48,9 @@
  * \return Error code, KNOT_EOK if successful.
  */
 int knot_zone_sign(const zone_contents_t *zone,
-                   const zone_keyset_t *zone_keys,
+                   zone_keyset_t *zone_keys,
                    const kdnssec_ctx_t *dnssec_ctx,
-                   changeset_t *out_ch, uint32_t *expire_at);
+                   changeset_t *out_ch, knot_time_t *expire_at);
 
 /*!
  * \brief Update and sign SOA and store performed changes in changeset.
@@ -125,4 +125,6 @@ int knot_zone_sign_nsecs_in_changeset(const zone_keyset_t *zone_keys,
 bool knot_zone_sign_rr_should_be_signed(const zone_node_t *node,
                                         const knot_rrset_t *rrset);
 
+bool knot_match_key_ds(zone_key_t *key, const knot_rdata_t *rdata);
+
 /*! @} */
