@@ -1,4 +1,4 @@
-/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,13 +37,17 @@ typedef struct event_info {
 } event_info_t;
 
 static const event_info_t EVENT_INFO[] = {
-	{ ZONE_EVENT_LOAD,    event_load,    "load" },
-	{ ZONE_EVENT_REFRESH, event_refresh, "refresh" },
-	{ ZONE_EVENT_UPDATE,  event_update,  "update" },
-	{ ZONE_EVENT_EXPIRE,  event_expire,  "expiration" },
-	{ ZONE_EVENT_FLUSH,   event_flush,   "journal flush" },
-	{ ZONE_EVENT_NOTIFY,  event_notify,  "notify" },
-	{ ZONE_EVENT_DNSSEC,  event_dnssec,  "DNSSEC resign" },
+	{ ZONE_EVENT_LOAD,         event_load,    "load" },
+	{ ZONE_EVENT_REFRESH,      event_refresh, "refresh" },
+	{ ZONE_EVENT_UPDATE,       event_update,  "update" },
+	{ ZONE_EVENT_EXPIRE,       event_expire,  "expiration" },
+	{ ZONE_EVENT_FLUSH,        event_flush,   "journal flush" },
+	{ ZONE_EVENT_NOTIFY,       event_notify,  "notify" },
+	{ ZONE_EVENT_DNSSEC,       event_dnssec,  "DNSSEC resign" },
+	{ ZONE_EVENT_UFREEZE,      event_ufreeze, "update freeze" },
+	{ ZONE_EVENT_UTHAW,        event_uthaw,   "update thaw" },
+	{ ZONE_EVENT_NSEC3RESALT,  event_nsec3resalt,  "NSEC3 resalt" },
+	{ ZONE_EVENT_PARENT_DS_Q,  event_parent_ds_q,  "parent DS query" },
 	{ 0 }
 };
 
@@ -65,6 +69,22 @@ static bool valid_event(zone_event_type_t type)
 	return (type > ZONE_EVENT_INVALID && type < ZONE_EVENT_COUNT);
 }
 
+bool ufreeze_applies(zone_event_type_t type)
+{
+	switch (type) {
+	case ZONE_EVENT_LOAD:
+	case ZONE_EVENT_REFRESH:
+	case ZONE_EVENT_UPDATE:
+	case ZONE_EVENT_FLUSH:
+	case ZONE_EVENT_DNSSEC:
+	case ZONE_EVENT_NSEC3RESALT:
+	case ZONE_EVENT_PARENT_DS_Q:
+		return true;
+	default:
+		return false;
+	}
+}
+
 /*! \brief Return remaining time to planned event (seconds). */
 static time_t time_until(time_t planned)
 {
@@ -97,6 +117,10 @@ static time_t event_get_time(zone_events_t *events, zone_event_type_t type)
 /*!
  * \brief Find next scheduled zone event.
  *
+ * \note Afer the UTHAW event, get_next_event() is also invoked. In that situation,
+ *       all the events are suddenly allowed, and those which were planned into
+ *       the ufrozen interval, start to be performed one-by-one sorted by their times.
+ *
  * \param events  Zone events.
  *
  * \return Zone event type, or ZONE_EVENT_INVALID if no event is scheduled.
@@ -112,11 +136,9 @@ static zone_event_type_t get_next_event(zone_events_t *events)
 
 	for (int i = 0; i < ZONE_EVENT_COUNT; i++) {
 		time_t current = events->time[i];
-		if (current == 0) {
-			continue;
-		}
 
-		if (next == 0 || current < next) {
+		if ((next == 0 || current < next) && (current != 0) &&
+		    (events->forced[i] || !events->ufrozen || !ufreeze_applies(i))) {
 			next = current;
 			next_type = i;
 		}
@@ -136,33 +158,41 @@ static time_t get_next_time(zone_events_t *events)
 
 /*!
  * \brief Cancel scheduled item, schedule first enqueued item.
- *
- * The events mutex must be locked when calling this function.
  */
 static void reschedule(zone_events_t *events)
 {
 	assert(events);
-	assert(pthread_mutex_trylock(&events->mx) == EBUSY);
+
+	pthread_mutex_lock(&events->reschedule_lock);
+	pthread_mutex_lock(&events->mx);
 
 	if (!events->event || events->running || events->frozen) {
+		pthread_mutex_unlock(&events->mx);
+		pthread_mutex_unlock(&events->reschedule_lock);
 		return;
 	}
 
 	zone_event_type_t type = get_next_event(events);
 	if (!valid_event(type)) {
+		pthread_mutex_unlock(&events->mx);
+		pthread_mutex_unlock(&events->reschedule_lock);
 		return;
 	}
 
 	time_t diff = time_until(event_get_time(events, type));
 
+	pthread_mutex_unlock(&events->mx);
+
 	evsched_schedule(events->event, diff * 1000);
+
+	pthread_mutex_unlock(&events->reschedule_lock);
 }
 
 /*!
  * \brief Zone event wrapper, expected to be called from a worker thread.
  *
  * 1. Takes the next planned event.
- * 2. Resets the event's scheduled time.
+ * 2. Resets the event's scheduled time (and forced flag).
  * 3. Perform the event's callback.
  * 4. Schedule next event planned event.
  */
@@ -182,6 +212,7 @@ static void event_wrap(task_t *task)
 		return;
 	}
 	event_set_time(events, type, 0);
+	events->forced[type] = false;
 	pthread_mutex_unlock(&events->mx);
 
 	const event_info_t *info = get_event_info(type);
@@ -204,8 +235,8 @@ static void event_wrap(task_t *task)
 
 	pthread_mutex_lock(&events->mx);
 	events->running = false;
-	reschedule(events);
 	pthread_mutex_unlock(&events->mx);
+	reschedule(events);
 }
 
 /*!
@@ -236,6 +267,7 @@ int zone_events_init(zone_t *zone)
 
 	memset(&zone->events, 0, sizeof(zone->events));
 	pthread_mutex_init(&events->mx, NULL);
+	pthread_mutex_init(&events->reschedule_lock, NULL);
 	events->task.ctx = zone;
 	events->task.run = event_wrap;
 
@@ -272,6 +304,7 @@ void zone_events_deinit(zone_t *zone)
 	evsched_event_free(zone->events.event);
 
 	pthread_mutex_destroy(&zone->events.mx);
+	pthread_mutex_destroy(&zone->events.reschedule_lock);
 
 	memset(&zone->events, 0, sizeof(zone->events));
 }
@@ -301,13 +334,29 @@ void _zone_events_schedule_at(zone_t *zone, ...)
 
 	// reschedule if changed
 	time_t next = get_next_time(events);
+	pthread_mutex_unlock(&events->mx);
 	if (old_next != next) {
 		reschedule(events);
 	}
 
+	va_end(args);
+}
+
+void zone_events_schedule_user(zone_t *zone, zone_event_type_t type)
+{
+	if (!zone || !valid_event(type)) {
+		return;
+	}
+
+	zone_events_t *events = &zone->events;
+	pthread_mutex_lock(&events->mx);
+	events->forced[type] = true;
 	pthread_mutex_unlock(&events->mx);
 
-	va_end(args);
+	zone_events_schedule_now(zone, type);
+
+	// reschedule because get_next_event result changed outside of _zone_events_schedule_at
+	reschedule(events);
 }
 
 void zone_events_enqueue(zone_t *zone, zone_event_type_t type)
@@ -321,7 +370,8 @@ void zone_events_enqueue(zone_t *zone, zone_event_type_t type)
 	pthread_mutex_lock(&events->mx);
 
 	/* Bypass scheduler if no event is running. */
-	if (!events->running && !events->frozen) {
+	if (!events->running && !events->frozen &&
+	    (!events->ufrozen || !ufreeze_applies(type))) {
 		events->running = true;
 		event_set_time(events, type, ZONE_EVENT_IMMEDIATE);
 		worker_pool_assign(events->pool, &events->task);
@@ -358,9 +408,14 @@ void zone_events_start(zone_t *zone)
 		return;
 	}
 
-	pthread_mutex_lock(&zone->events.mx);
-	reschedule(&zone->events);
-	pthread_mutex_unlock(&zone->events.mx);
+	zone_events_t *events = &zone->events;
+
+	/* Unlock the events queue. */
+	pthread_mutex_lock(&events->mx);
+	events->frozen = false;
+	pthread_mutex_unlock(&events->mx);
+
+	reschedule(events);
 }
 
 time_t zone_events_get_time(const struct zone *zone, zone_event_type_t type)
