@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 #define MDKEY_GLOBAL_LAST_INSERTER_ZONE		"last_inserter_zone"
 #define MDKEY_PERZONE_OCCUPIED			"occupied"
 #define MDKEY_PERZONE_FLAGS			"flags"
+#define KEY_BOOTSTRAP_CHANGESET			"bootstrap"
 
 /*! \brief The number of unused bytes in DB key. */
 #define DB_KEY_UNUSED_ZERO (4)
@@ -46,11 +47,20 @@
  * uint32_t serial_to + uint32_t chunk_count + 24B unused */
 #define JOURNAL_HEADER_SIZE (32)
 
+// eventually move to contrib and reuse as needed
+#define local_array_max_static_size (100)
+#define local_array(type, name, size) \
+	type name ## _static__[local_array_max_static_size] = { 0 }; \
+	type *name ## _dynamic__ = ((size) > local_array_max_static_size ? calloc((size), sizeof(type)) : NULL); \
+	type *name = ((size) > local_array_max_static_size ? name ## _dynamic__ : name ## _static__);
+#define local_array_free(name) { free(name ## _dynamic__); }
+
 enum {
-	LAST_FLUSHED_VALID  = 1 << 0, /* "last flush is valid" flag. */
-	SERIAL_TO_VALID     = 1 << 1, /* "last serial_to is valid" flag. */
-	MERGED_SERIAL_VALID = 1 << 2, /* "serial_from" of merged changeset. */
-	DIRTY_SERIAL_VALID  = 1 << 3, /* "dirty_serial" is present in the DB. */
+	LAST_FLUSHED_VALID   = 1 << 0, /* "last flush is valid" flag. */
+	SERIAL_TO_VALID      = 1 << 1, /* "last serial_to is valid" flag. */
+	MERGED_SERIAL_VALID  = 1 << 2, /* "serial_from" of merged changeset. */
+	DIRTY_SERIAL_VALID   = 1 << 3, /* "dirty_serial" is present in the DB. */
+	FIRST_SERIAL_INVALID = 1 << 4, /* "first_serial" is not valid. */
 };
 
 static bool journal_flush_allowed(journal_t *j) {
@@ -112,6 +122,7 @@ typedef struct {
 	journal_t *j;
 	knot_db_txn_t *txn;
 	int ret;
+	bool opened;
 
 	bool is_rw;
 
@@ -124,19 +135,17 @@ typedef struct {
 	metadata_t shadow_md;
 } txn_t;
 
-static void md_get(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint32_t *res);
-static void md_set(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint32_t val);
+static void md_get(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint64_t *res);
+static void md_get32(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint32_t *res);
+static void md_set(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint64_t val);
+static void md_set32(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint32_t val);
 
 static void txn_init(txn_t *txn, knot_db_txn_t *db_txn, journal_t *j)
 {
+	memset(txn, 0, sizeof(*txn));
 	txn->j = j;
 	txn->txn = db_txn;
-	txn->ret = KNOT_ESEMCHECK;
-	txn->iter = NULL;
-	txn->key.len = 0;
 	txn->key.data = &txn->key_raw;
-	txn->val.len = 0;
-	txn->val.data = NULL;
 }
 
 #define local_txn_t(txn_name, journal) \
@@ -152,6 +161,8 @@ static void txn_init(txn_t *txn, knot_db_txn_t *db_txn, journal_t *j)
  *
  * Changeset:
  * | zone_name | \0 | unused zero 4B | (be32)serial_from | (be32)chunk_index |
+ *  or
+ * | zone_name | \0 | unused zero 4B | metadata_key | \0 | (be32)serial_from |
  *
  * Structure of the changeset:
  * | (be32)serial_to | (be32)#of_chunks | unused zero 24B | serialized_changeset...
@@ -190,6 +201,23 @@ static void txn_key_2u32(txn_t *txn, const knot_dname_t *zone, uint32_t key1, ui
 	       &key_be2, sizeof(uint32_t));
 }
 
+static void txn_key_str_u32(txn_t *txn, const knot_dname_t *zone, const char *key1, uint32_t key2)
+{
+	size_t zone_size = 0;
+	if (zone != NULL) zone_size = knot_dname_size(zone);
+	txn->key.len = zone_size + DB_KEY_UNUSED_ZERO + strlen(key1) + 1 + sizeof(uint32_t);
+	if (txn->key.len > 512) {
+		txn->ret = KNOT_ERROR;
+		return;
+	}
+	if (zone != NULL) memcpy(txn->key.data, zone, zone_size);
+	memset(txn->key.data + zone_size, 0, DB_KEY_UNUSED_ZERO);
+	strcpy(txn->key.data + zone_size + DB_KEY_UNUSED_ZERO, key1);
+	uint32_t key_be2 = htobe32(key2);
+	memcpy(txn->key.data + zone_size + DB_KEY_UNUSED_ZERO + strlen(key1) + 1,
+	       &key_be2, sizeof(uint32_t));
+}
+
 static int txn_cmpkey(txn_t *txn, knot_db_val_t *key2)
 {
 	if (txn->key.len != key2->len) {
@@ -201,32 +229,39 @@ static int txn_cmpkey(txn_t *txn, knot_db_val_t *key2)
 	return memcmp(txn->key.data, key2->data, key2->len);
 }
 
-static void txn_val_u32(txn_t *txn, uint32_t *res)
+static void txn_val_u64(txn_t *txn, uint64_t *res)
 {
 	if (txn->ret != KNOT_EOK) {
 		return;
 	}
-	if (txn->val.len != sizeof(uint32_t)) {
+	uint32_t beval32;
+	uint64_t beval;
+	switch (txn->val.len) {
+	case sizeof(uint32_t):
+		memcpy(&beval32, (uint32_t *)txn->val.data, sizeof(beval32));
+		*res = (uint64_t)be32toh(beval32);
+		break;
+	case sizeof(uint64_t):
+		memcpy(&beval, (uint64_t *)txn->val.data, sizeof(beval));
+		*res = be64toh(beval);
+		break;
+	default:
 		txn->ret = KNOT_EMALF;
 	}
-	uint32_t beval;
-	memcpy(&beval, (uint32_t *)txn->val.data, sizeof(beval));
-	*res = be32toh(beval);
 }
 
-#define txn_begin_md(md) md_get(txn, txn->j->zone, #md, &txn->shadow_md.md)
-#define txn_commit_md(md) md_set(txn, txn->j->zone, #md, txn->shadow_md.md)
+#define txn_begin_md(md) md_get32(txn, txn->j->zone, #md, &txn->shadow_md.md)
+#define txn_commit_md(md) md_set32(txn, txn->j->zone, #md, txn->shadow_md.md)
 
-#define txn_check(txn) if ((txn)->ret != KNOT_EOK) return
-#define txn_check_ret(txn) if ((txn)->ret != KNOT_EOK) return ((txn)->ret)
-#define txn_ret(txn) return ((txn)->ret == KNOT_ESEMCHECK ? KNOT_EOK : (txn)->ret)
-
-#define txn_finished_ok(txn) ((txn)->ret == KNOT_ESEMCHECK || (txn)->ret == KNOT_EOK)
+#define txn_check_open(txn) if (((txn)->ret = ((txn)->opened ? (txn)->ret : KNOT_EINVAL)) != KNOT_EOK) return
+#define txn_check_ret(txn) if (((txn)->ret = ((txn)->opened ? (txn)->ret : KNOT_EINVAL)) != KNOT_EOK)  return ((txn)->ret)
 
 static void txn_begin(txn_t *txn, bool write_allowed)
 {
-	if (txn->ret != KNOT_ESEMCHECK) {
+	if (txn->ret == KNOT_EOK && txn->opened) {
 		txn->ret = KNOT_EINVAL;
+	}
+	if (txn->ret != KNOT_EOK) {
 		return;
 	}
 
@@ -234,6 +269,7 @@ static void txn_begin(txn_t *txn, bool write_allowed)
 	                                         (write_allowed ? 0 : KNOT_DB_RDONLY));
 
 	txn->is_rw = write_allowed;
+	txn->opened = true;
 
 	txn_begin_md(first_serial);
 	txn_begin_md(last_serial);
@@ -247,41 +283,38 @@ static void txn_begin(txn_t *txn, bool write_allowed)
 
 static void txn_find_force(txn_t *txn)
 {
-	if (txn->ret == KNOT_EOK) {
-		txn->ret = txn->j->db->db_api->find(txn->txn, &txn->key, &txn->val, 0);
-	}
+	txn_check_open(txn);
+	txn->ret = txn->j->db->db_api->find(txn->txn, &txn->key, &txn->val, 0);
 }
 
-static int txn_find(txn_t *txn)
+static bool txn_find(txn_t *txn)
 {
-	if (txn->ret != KNOT_EOK) {
-		return 0;
+	if (txn->ret != KNOT_EOK || !txn->opened) {
+		return false;
 	}
 	txn_find_force(txn);
 	if (txn->ret == KNOT_ENOENT) {
 		txn->ret = KNOT_EOK;
-		return 0;
+		return false;
 	}
-	return (txn->ret == KNOT_EOK ? 1 : 0);
+	return (txn->ret == KNOT_EOK);
 }
 
 static void txn_insert(txn_t *txn)
 {
-	if (txn->ret == KNOT_EOK) {
-		txn->ret = txn->j->db->db_api->insert(txn->txn, &txn->key, &txn->val, 0);
-	}
+	txn_check_open(txn);
+	txn->ret = txn->j->db->db_api->insert(txn->txn, &txn->key, &txn->val, 0);
 }
 
 static void txn_del(txn_t *txn)
 {
-	if (txn->ret == KNOT_EOK) {
-		txn->ret = txn->j->db->db_api->del(txn->txn, &txn->key);
-	}
+	txn_check_open(txn);
+	txn->ret = txn->j->db->db_api->del(txn->txn, &txn->key);
 }
 
 static void txn_iter_begin(txn_t *txn)
 {
-	txn_check(txn);
+	txn_check_open(txn);
 	txn->iter = txn->j->db->db_api->iter_begin(txn->txn, KNOT_DB_FIRST);
 	if (txn->iter == NULL) {
 		txn->ret = KNOT_ENOMEM;
@@ -331,13 +364,10 @@ static void txn_iter_finish(txn_t *txn)
 
 static void txn_abort(txn_t *txn)
 {
-	if (txn->ret == KNOT_ESEMCHECK) {
-		return;
-	}
-	txn_iter_finish(txn);
-	txn->j->db->db_api->txn_abort(txn->txn);
-	if (txn->ret == KNOT_EOK) {
-		txn->ret = KNOT_ESEMCHECK;
+	if (txn->opened) {
+		txn_iter_finish(txn);
+		txn->j->db->db_api->txn_abort(txn->txn);
+		txn->opened = false;
 	}
 }
 
@@ -363,7 +393,7 @@ static void txn_commit(txn_t *txn)
 	txn->ret = txn->j->db->db_api->txn_commit(txn->txn);
 
 	if (txn->ret == KNOT_EOK) {
-		txn->ret = KNOT_ESEMCHECK;
+		txn->opened = false;
 	}
 	txn_abort(txn); // no effect if all ok
 }
@@ -371,7 +401,8 @@ static void txn_commit(txn_t *txn)
 static void txn_restart(txn_t *txn)
 {
 	txn_commit(txn);
-	if (txn->ret == KNOT_ESEMCHECK) {
+	assert(!txn->opened);
+	if (txn->ret == KNOT_EOK) {
 		txn_begin(txn, txn->is_rw);
 	}
 }
@@ -404,21 +435,32 @@ static void txn_unreuse(txn_t **txn, txn_t *reused)
  * ********************************************************************
  */
 
-static void md_get(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint32_t *res)
+static void md_get(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint64_t *res)
 {
-	txn_check(txn);
+	txn_check_open(txn);
 	txn_key_str(txn, zone, mdkey);
-	uint32_t res1 = 0;
+	uint64_t res1 = 0;
 	if (txn_find(txn)) {
-		txn_val_u32(txn, &res1);
+		txn_val_u64(txn, &res1);
 	}
 	*res = res1;
 }
 
+static void md_get32(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint32_t *res)
+{
+	uint64_t res1 = 0;
+	md_get(txn, zone, mdkey, &res1);
+	if (res1 > UINT32_MAX) {
+		txn->ret = KNOT_EMALF;
+	} else {
+		*res = (uint32_t)res1;
+	}
+}
+
 // allocates res
 static void md_get_common_last_inserter_zone(txn_t *txn, knot_dname_t **res)
 {
-	txn_check(txn);
+	txn_check_open(txn);
 	txn_key_str(txn, NULL, MDKEY_GLOBAL_LAST_INSERTER_ZONE);
 	if (txn_find(txn)) {
 		*res = knot_dname_copy(txn->val.data, NULL);
@@ -438,14 +480,34 @@ static int md_set_common_last_inserter_zone(txn_t *txn, knot_dname_t *zone)
 	return txn->ret;
 }
 
+static void md_del_last_inserter_zone(txn_t *txn, knot_dname_t *if_equals)
+{
+	txn_check_open(txn);
+	txn_key_str(txn, NULL, MDKEY_GLOBAL_LAST_INSERTER_ZONE);
+	if (txn_find(txn)) {
+		if (if_equals == NULL || knot_dname_is_equal(txn->val.data, if_equals)) {
+			txn_del(txn);
+		}
+	}
+}
+
 static void md_get_common_last_occupied(txn_t *txn, size_t *res)
 {
-	uint32_t sres = 0;
+	uint64_t sres = 0;
 	md_get(txn, NULL, MDKEY_GLOBAL_LAST_TOTAL_OCCUPIED, &sres);
 	*res = (size_t) sres;
 }
 
-static void md_set(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint32_t val)
+static void md_set(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint64_t val)
+{
+	txn_key_str(txn, zone, mdkey);
+	uint64_t val1 = htobe64(val);
+	txn->val.len = sizeof(uint64_t);
+	txn->val.data = &val1;
+	txn_insert(txn);
+}
+
+static void md_set32(txn_t *txn, const knot_dname_t *zone, const char *mdkey, uint32_t val)
 {
 	txn_key_str(txn, zone, mdkey);
 	uint32_t val1 = htobe32(val);
@@ -462,7 +524,7 @@ static bool md_flag(txn_t *txn, int flag)
 /*! \brief Marks metadata as flushed */
 static void md_flush(txn_t *txn)
 {
-	if (md_flag(txn, SERIAL_TO_VALID)) {
+	if (md_flag(txn, SERIAL_TO_VALID) && !md_flag(txn, FIRST_SERIAL_INVALID)) {
 		txn->shadow_md.last_flushed = txn->shadow_md.last_serial;
 		txn->shadow_md.flags |= LAST_FLUSHED_VALID;
 	}
@@ -517,7 +579,7 @@ static int first_digit(char * of)
 
 static void md_update_journal_count(txn_t * txn, int change_amount)
 {
-	uint32_t jcnt = 0;
+	uint64_t jcnt = 0;
 	md_get(txn, NULL, MDKEY_GLOBAL_JOURNAL_COUNT, &jcnt);
 	md_set(txn, NULL, MDKEY_GLOBAL_JOURNAL_COUNT, jcnt + change_amount);
 }
@@ -558,7 +620,7 @@ static int initial_md_check(journal_t *j, bool *dirty_present)
 		txn_abort(txn);
 	}
 
-	txn_ret(txn);
+	return txn->ret;
 }
 
 /*
@@ -586,30 +648,36 @@ typedef struct {
 	void *iter_context;	// anything to send to the callback by the caller of iterate(), untouched by iterate()
 } iteration_ctx_t;
 
+typedef int (*iteration_cb_t)(iteration_ctx_t *ctx);
+
 /*!
  * \brief Move iter to next changeset chunk.
  *
  * Try optimisticly fast move to next DB item. But the changeset can be out of order,
  * so if we don't succeed (different serial or end of DB), we lookup next serial slowly.
  */
-static void get_iter_next(txn_t *txn, uint32_t expect_serial, int expect_chunk)
+
+static void get_iter_next(iteration_ctx_t *ctx, iteration_cb_t key_cb)
 {
 	knot_db_val_t other_key = { 0 };
 
-	txn_check(txn);
-	txn_iter_next(txn);
-	txn_iter_key(txn, &other_key);
-	txn_key_2u32(txn, txn->j->zone, expect_serial, (uint32_t)expect_chunk);
-	if (txn->ret == KNOT_ENOENT ||
-	    (txn->ret == KNOT_EOK && txn_cmpkey(txn, &other_key) != 0)) {
-			txn_iter_seek(txn);
+	txn_check_open(ctx->txn);
+	txn_iter_next(ctx->txn);
+	txn_iter_key(ctx->txn, &other_key);
+	key_cb(ctx);
+	if (ctx->txn->ret == KNOT_ENOENT ||
+	    (ctx->txn->ret == KNOT_EOK && txn_cmpkey(ctx->txn, &other_key) != 0)) {
+		ctx->txn->ret = KNOT_EOK;
+		if (ctx->txn->iter != NULL) {
+			txn_iter_finish(ctx->txn);
+		}
+		txn_iter_begin(ctx->txn);
+		txn_iter_seek(ctx->txn);
 	}
 }
 
-typedef int (*iteration_cb_t)(iteration_ctx_t *ctx);
-
 static int iterate(journal_t *j, txn_t *_txn, iteration_cb_t cb, int method,
-                   void *iter_context, uint32_t first, uint32_t last)
+                   void *iter_context, uint32_t first, uint32_t last, iteration_cb_t key_cb)
 {
 	reuse_txn(txn, j, _txn, true);
 
@@ -625,7 +693,7 @@ static int iterate(journal_t *j, txn_t *_txn, iteration_cb_t cb, int method,
 
 	txn_iter_begin(txn);
 
-	txn_key_2u32(txn, j->zone, ctx.serial, ctx.chunk_index);
+	key_cb(&ctx);
 	txn_iter_seek(txn);
 
 	ctx.val = &txn->val;
@@ -671,7 +739,7 @@ static int iterate(journal_t *j, txn_t *_txn, iteration_cb_t cb, int method,
 			ctx.chunk_index++;
 		}
 
-		get_iter_next(txn, ctx.serial, ctx.chunk_index);
+		get_iter_next(&ctx, key_cb);
 	}
 
 	if (vals != NULL) {
@@ -681,7 +749,13 @@ static int iterate(journal_t *j, txn_t *_txn, iteration_cb_t cb, int method,
 
 	unreuse_txn(txn, _txn);
 
-	txn_ret(txn);
+	return txn->ret;
+}
+
+static int normal_iterkeycb(iteration_ctx_t *ctx)
+{
+	txn_key_2u32(ctx->txn, ctx->txn->j->zone, ctx->serial, ctx->chunk_index);
+	return KNOT_EOK;
 }
 
 /*
@@ -694,22 +768,33 @@ static int iterate(journal_t *j, txn_t *_txn, iteration_cb_t cb, int method,
 
 /*! \brief Deserialize changeset from chunks (in vals) */
 static int vals_to_changeset(knot_db_val_t *vals, int nvals,
-                             const knot_dname_t *zone_name, changeset_t **ch)
+                             const knot_dname_t *zone_name, changeset_t **ch, bool is_bootstrap)
 {
-	uint8_t *valps[nvals];
-	size_t vallens[nvals];
-	for (int i = 0; i < nvals; i++) {
+	local_array(uint8_t *, valps, nvals)
+	local_array(size_t, vallens, nvals)
+	if (valps == NULL || vallens == NULL) {
+		local_array_free(valps)
+		local_array_free(vallens)
+		return KNOT_ENOMEM;
+	}
+
+	for (size_t i = 0; i < nvals; i++) {
 		valps[i] = vals[i].data + JOURNAL_HEADER_SIZE;
 		vallens[i] = vals[i].len - JOURNAL_HEADER_SIZE;
 	}
 
 	changeset_t *t_ch = changeset_new(zone_name);
 	if (t_ch == NULL) {
+		local_array_free(valps)
+		local_array_free(vallens)
 		return KNOT_ENOMEM;
 	}
 
-	int ret = changeset_deserialize(t_ch, valps, vallens, nvals);
+	int ret = is_bootstrap ? changeset_deserialize_bootstrap(t_ch, valps, vallens, nvals)
+	                       : changeset_deserialize(t_ch, valps, vallens, nvals);
 
+	local_array_free(valps)
+	local_array_free(vallens)
 	if (ret != KNOT_EOK) {
 		changeset_free(t_ch);
 		return ret;
@@ -725,7 +810,7 @@ static int load_one_itercb(iteration_ctx_t *ctx)
 		return KNOT_EINVAL;
 	}
 
-	int ret = vals_to_changeset(ctx->val, ctx->chunk_count, ctx->txn->j->zone, &ch);
+	int ret = vals_to_changeset(ctx->val, ctx->chunk_count, ctx->txn->j->zone, &ch, false);
 	if (ret == KNOT_EOK) *targ = ch;
 	return ret;
 }
@@ -735,7 +820,7 @@ static int load_list_itercb(iteration_ctx_t *ctx)
 	changeset_t *ch = NULL;
 	list_t *chlist = *(list_t **) ctx->iter_context;
 
-	int ret = vals_to_changeset(ctx->val, ctx->chunk_count, ctx->txn->j->zone, &ch);
+	int ret = vals_to_changeset(ctx->val, ctx->chunk_count, ctx->txn->j->zone, &ch, false);
 
 	if (ret == KNOT_EOK) {
 		add_tail(chlist, &ch->n);
@@ -748,13 +833,13 @@ static int load_one(journal_t *j, txn_t *_txn, uint32_t serial, changeset_t **ch
 {
 	reuse_txn(txn, j, _txn, false);
 	changeset_t *rch = NULL;
-	iterate(j, txn, load_one_itercb, JOURNAL_ITERATION_CHANGESETS, &rch, serial, serial);
+	iterate(j, txn, load_one_itercb, JOURNAL_ITERATION_CHANGESETS, &rch, serial, serial, normal_iterkeycb);
 	unreuse_txn(txn, _txn);
-	if (txn_finished_ok(txn)) {
+	if (txn->ret == KNOT_EOK) {
 		if (rch == NULL) txn->ret = KNOT_ENOENT;
 		else *ch = rch;
 	}
-	txn_ret(txn);
+	return txn->ret;
 }
 
 static int load_merged_changeset(journal_t *j, txn_t *_txn, changeset_t **mch,
@@ -772,10 +857,9 @@ static int load_merged_changeset(journal_t *j, txn_t *_txn, changeset_t **mch,
 	}
 	unreuse_txn(txn, _txn);
 
-	txn_ret(txn);
+	return txn->ret;
 }
 
-/*! \brief API: load all changesets since "from" serial into dst. */
 int journal_load_changesets(journal_t *j, list_t *dst, uint32_t from)
 {
 	if (j == NULL || j->db == NULL || dst == NULL) return KNOT_EINVAL;
@@ -791,10 +875,70 @@ int journal_load_changesets(journal_t *j, list_t *dst, uint32_t from)
 	}
 
 	uint32_t ls = txn->shadow_md.last_serial;
-	iterate(j, txn, load_list_itercb, JOURNAL_ITERATION_CHANGESETS, &dst, from, ls);
+	iterate(j, txn, load_list_itercb, JOURNAL_ITERATION_CHANGESETS, &dst, from,
+	        ls, normal_iterkeycb);
 	txn_commit(txn);
 
-	txn_ret(txn);
+	return txn->ret;
+}
+
+int load_bootstrap_iterkeycb(iteration_ctx_t *ctx)
+{
+	txn_key_str_u32(ctx->txn, ctx->txn->j->zone, KEY_BOOTSTRAP_CHANGESET, ctx->chunk_index);
+	return KNOT_EOK;
+}
+
+static int load_bootstrap_itercb(iteration_ctx_t *ctx)
+{
+	changeset_t *ch = NULL, **targ = ctx->iter_context;
+	if (*targ != NULL) {
+		return KNOT_EINVAL;
+	}
+
+	int ret = vals_to_changeset(ctx->val, ctx->chunk_count, ctx->txn->j->zone, &ch, true);
+	if (ret == KNOT_EOK) *targ = ch;
+	return ret;
+}
+
+static int load_bootstrap_changeset(journal_t *j, txn_t *_txn, changeset_t **ch)
+{
+	reuse_txn(txn, j, _txn, false);
+	changeset_t *rch = NULL;
+	iterate(j, txn, load_bootstrap_itercb, JOURNAL_ITERATION_CHANGESETS, &rch,
+	        0, 0, load_bootstrap_iterkeycb);
+	unreuse_txn(txn, _txn);
+	if (txn->ret == KNOT_EOK) {
+		if (rch == NULL) txn->ret = KNOT_ENOENT;
+		else *ch = rch;
+	}
+	return txn->ret;
+}
+
+int journal_load_bootstrap(journal_t *j, list_t *dst)
+{
+	if (j == NULL || j->db == NULL || dst == NULL) return KNOT_EINVAL;
+
+	local_txn_t(txn, j);
+	txn_begin(txn, false);
+
+	changeset_t *bch = NULL;
+	load_bootstrap_changeset(j, txn, &bch);
+	if (bch == NULL) {
+		txn->ret = KNOT_ENOENT;
+		goto jlb_end;
+	}
+	add_tail(dst, &bch->n);
+	uint32_t from = knot_soa_serial(&bch->soa_to->rrs);
+
+	uint32_t ls = txn->shadow_md.last_serial;
+	iterate(j, txn, load_list_itercb, JOURNAL_ITERATION_CHANGESETS, &dst,
+	        from, ls, normal_iterkeycb);
+	if (txn->ret == KNOT_ENOENT) {
+		txn->ret = KNOT_EOK;
+	}
+jlb_end:
+	txn_commit(txn);
+	return txn->ret;
 }
 
 /*
@@ -842,7 +986,8 @@ static int del_upto_itercb(iteration_ctx_t *ctx)
  * Please ensure (dbfirst == j->metadata.first_serial) */
 static int delete_upto(journal_t *j, txn_t *txn, uint32_t dbfirst, uint32_t last)
 {
-	return iterate(j, txn, del_upto_itercb, JOURNAL_ITERATION_CHUNKS, NULL, dbfirst, last);
+	return iterate(j, txn, del_upto_itercb, JOURNAL_ITERATION_CHUNKS, NULL,
+	               dbfirst, last, normal_iterkeycb);
 }
 
 static int delete_merged_changeset(journal_t *j, txn_t *t)
@@ -856,7 +1001,7 @@ static int delete_merged_changeset(journal_t *j, txn_t *t)
 		delete_upto(j, txn, txn->shadow_md.merged_serial, txn->shadow_md.merged_serial);
 	}
 	unreuse_txn(txn, t);
-	txn_ret(txn);
+	return txn->ret;
 }
 
 static int drop_journal(journal_t *j, txn_t *_txn)
@@ -866,11 +1011,13 @@ static int drop_journal(journal_t *j, txn_t *_txn)
 	if (md_flag(txn, MERGED_SERIAL_VALID)) {
 		delete_merged_changeset(j, txn);
 	}
-	if (md_flag(txn, SERIAL_TO_VALID)) {
+	if (md_flag(txn, SERIAL_TO_VALID) && !md_flag(txn, FIRST_SERIAL_INVALID)) {
 		delete_upto(j, txn, txn->shadow_md.first_serial, txn->shadow_md.last_serial);
 	}
+	md_del_last_inserter_zone(txn, j->zone);
+	md_set(txn, j->zone, MDKEY_PERZONE_OCCUPIED, 0);
 	unreuse_txn(txn, _txn);
-	txn_ret(txn);
+	return txn->ret;
 }
 
 static int del_tofree_itercb(iteration_ctx_t *ctx)
@@ -924,11 +1071,13 @@ static int delete_tofree(journal_t *j, txn_t *_txn, size_t to_be_freed, size_t *
 	}
 	delete_status_t ds = { .freed_approx = 0, .to_be_freed = to_be_freed };
 	iterate(j, txn, del_tofree_itercb, JOURNAL_ITERATION_CHUNKS, &ds,
-	        txn->shadow_md.first_serial, txn->shadow_md.last_serial);
+	        txn->shadow_md.first_serial, txn->shadow_md.last_serial, normal_iterkeycb);
 	unreuse_txn(txn, _txn);
 
-	if (txn_finished_ok(txn)) *really_freed = ds.freed_approx;
-	txn_ret(txn);
+	if (txn->ret == KNOT_EOK) {
+		*really_freed = ds.freed_approx;
+	}
+	return txn->ret;
 }
 
 static int del_count_itercb(iteration_ctx_t *ctx)
@@ -976,11 +1125,13 @@ static int delete_count(journal_t *j, txn_t *_txn, size_t to_be_deleted, size_t
 	}
 	delete_status_t ds = { .freed_approx = 0, .to_be_freed = to_be_deleted };
 	iterate(j, txn, del_count_itercb, JOURNAL_ITERATION_CHUNKS, &ds,
-	        txn->shadow_md.first_serial, txn->shadow_md.last_serial);
+	        txn->shadow_md.first_serial, txn->shadow_md.last_serial, normal_iterkeycb);
 	unreuse_txn(txn, _txn);
 
-	if (txn_finished_ok(txn)) *really_deleted = ds.freed_approx;
-	txn_ret(txn);
+	if (txn->ret == KNOT_EOK) {
+		*really_deleted = ds.freed_approx;
+	}
+	return txn->ret;
 }
 
 static int delete_dirty_serial(journal_t *j, txn_t *_txn)
@@ -998,10 +1149,10 @@ static int delete_dirty_serial(journal_t *j, txn_t *_txn)
 		txn_key_2u32(txn, j->zone, ds, ++chunk);
 	}
 	unreuse_txn(txn, _txn);
-	if (txn_finished_ok(txn)) {
+	if (txn->ret == KNOT_EOK) {
 		txn->shadow_md.flags &= ~DIRTY_SERIAL_VALID;
 	}
-	txn_ret(txn);
+	return txn->ret;
 }
 
 /*
@@ -1016,7 +1167,7 @@ static int merge_itercb(iteration_ctx_t *ctx)
 {
 	changeset_t *ch = NULL, *mch = *(changeset_t **)ctx->iter_context;
 
-	int ret = vals_to_changeset(ctx->val, ctx->chunk_count, ctx->txn->j->zone, &ch);
+	int ret = vals_to_changeset(ctx->val, ctx->chunk_count, ctx->txn->j->zone, &ch, false);
 	if (ret == KNOT_EOK) {
 		ret = changeset_merge(mch, ch);
 		changeset_free(ch);
@@ -1039,6 +1190,7 @@ static int merge_unflushed_changesets(journal_t *j, txn_t *_txn, changeset_t **m
 	                                            txn->shadow_md.first_serial);
 	txn->ret = load_one(j, txn, from, mch);
 	if (!was_merged && was_flushed && txn->ret == KNOT_EOK) {
+		// we have to jump to ONE AFTER last_flushed
 		from = knot_soa_serial(&(*mch)->soa_to->rrs);
 		changeset_free(*mch);
 		*mch = NULL;
@@ -1049,16 +1201,18 @@ static int merge_unflushed_changesets(journal_t *j, txn_t *_txn, changeset_t **m
 	}
 	from = knot_soa_serial(&(*mch)->soa_to->rrs);
 
-	txn->ret = iterate(j, txn, merge_itercb, JOURNAL_ITERATION_CHANGESETS,
-	                   mch, from, txn->shadow_md.last_serial);
+	if (serial_compare(from, txn->shadow_md.last_serial_to) != 0) {
+		txn->ret = iterate(j, txn, merge_itercb, JOURNAL_ITERATION_CHANGESETS,
+		                   mch, from, txn->shadow_md.last_serial, normal_iterkeycb);
+	}
 
 m_u_ch_end:
 	unreuse_txn(txn, _txn);
-	if (!txn_finished_ok(txn) && *mch != NULL) {
+	if (txn->ret != KNOT_EOK && *mch != NULL) {
 		changeset_free(*mch);
 		*mch = NULL;
 	}
-	txn_ret(txn);
+	return txn->ret;
 }
 
 // uses local context, e.g.: j, txn, changesets, nchs, serialized_size_total, store_changeset_cleanup, inserting_merged
@@ -1067,6 +1221,9 @@ m_u_ch_end:
 		if (journal_merge_allowed(j)) { \
 			changeset_t *merged; \
 			merge_unflushed_changesets(j, txn, &merged); \
+			if (txn->ret != KNOT_EOK) { \
+				goto store_changeset_cleanup; \
+			} \
 			add_tail(changesets, &merged->n); \
 			nchs++; \
 			serialized_size_total += changeset_serialized_size(merged); \
@@ -1093,6 +1250,8 @@ static int store_changesets(journal_t *j, list_t *changesets)
 
 	int inserting_merged = false;
 
+	size_t occupied_last, occupied_now = knot_db_lmdb_get_usage(j->db->db);
+
 	WALK_LIST(ch, *changesets) {
 		nchs++;
 		serialized_size_total += changeset_serialized_size(ch);
@@ -1104,23 +1263,34 @@ static int store_changesets(journal_t *j, list_t *changesets)
 	// if you're tempted to add dirty_serial deletion somewhere here, you're wrong. Don't do it.
 
 	// PART 2 : recalculating the previous insert's occupy change
-	size_t occupied_last, occupied_now;
 	md_get_common_last_occupied(txn, &occupied_last);
-	occupied_now = knot_db_lmdb_get_usage(j->db->db);
-	md_set(txn, NULL, MDKEY_GLOBAL_LAST_TOTAL_OCCUPIED, occupied_now);
-	if (occupied_now != occupied_last) {
-		knot_dname_t *last_zone = NULL;
-		uint32_t lz_occupied;
-		md_get_common_last_inserter_zone(txn, &last_zone);
-		md_get(txn, last_zone, MDKEY_PERZONE_OCCUPIED, &lz_occupied);
-		lz_occupied += occupied_now - occupied_last;
-		md_set(txn, last_zone, MDKEY_PERZONE_OCCUPIED, lz_occupied);
-		free(last_zone);
+	if (occupied_now == 0) {
+		// This shall not happen. We just handle it to minimize damage in case it would. Not 100% correct.
+		uint64_t tz_occupied;
+		md_get(txn, j->zone, MDKEY_PERZONE_OCCUPIED, &tz_occupied);
+		tz_occupied += serialized_size_total;
+		occupied_last += serialized_size_total;
+		md_set(txn, NULL, MDKEY_GLOBAL_LAST_TOTAL_OCCUPIED, occupied_last);
+		md_set(txn, j->zone, MDKEY_PERZONE_OCCUPIED, tz_occupied);
+	} else {
+		md_set(txn, NULL, MDKEY_GLOBAL_LAST_TOTAL_OCCUPIED, occupied_now);
+		if (occupied_now != occupied_last) {
+			knot_dname_t *last_zone = NULL;
+			uint64_t lz_occupied;
+			md_get_common_last_inserter_zone(txn, &last_zone);
+			if (last_zone != NULL) {
+				md_get(txn, last_zone, MDKEY_PERZONE_OCCUPIED, &lz_occupied);
+				lz_occupied = (lz_occupied + occupied_now > occupied_last ?
+					       lz_occupied + occupied_now - occupied_last : 0);
+				md_set(txn, last_zone, MDKEY_PERZONE_OCCUPIED, lz_occupied);
+				free(last_zone);
+			}
+		}
+		md_set_common_last_inserter_zone(txn, j->zone);
 	}
-	md_set_common_last_inserter_zone(txn, j->zone);
 
 	// PART 3 : check if we exceeded designed occupation and delete some
-	uint32_t occupied = 0, occupied_max;
+	uint64_t occupied = 0, occupied_max;
 	md_get(txn, j->zone, MDKEY_PERZONE_OCCUPIED, &occupied);
 	occupied_max = journal_max_usage(j);
 	occupied += serialized_size_total;
@@ -1136,7 +1306,7 @@ static int store_changesets(journal_t *j, list_t *changesets)
 			delete_tofree(j, txn, tofree, &freed);
 			if (freed < free_min) {
 				txn->ret = KNOT_ESPACE;
-				log_zone_warning(j->zone, "journal: unable to make free space for insert");
+				log_zone_warning(j->zone, "journal, unable to make free space for insert");
 				goto store_changeset_cleanup;
 			}
 		}
@@ -1144,7 +1314,7 @@ static int store_changesets(journal_t *j, list_t *changesets)
 
 	// PART 3.5 : check if we exceeded history depth
 	long over_limit = (long)txn->shadow_md.changeset_count - journal_max_changesets(j) +
-			  list_size(changesets) - (inserting_merged ? 1 : 0);
+	                  list_size(changesets) - (inserting_merged ? 1 : 0);
 	if (over_limit > 0) {
 		size_t deled;
 		delete_count(j, txn, over_limit, &deled);
@@ -1158,22 +1328,29 @@ static int store_changesets(journal_t *j, list_t *changesets)
 
 	// PART 4: continuity and duplicity check
 	changeset_t * chs_head = (HEAD(*changesets));
-	uint32_t serial = knot_soa_serial(&chs_head->soa_from->rrs);
-	if (md_flag(txn, SERIAL_TO_VALID) && serial_compare(txn->shadow_md.last_serial_to, serial) != 0) {
-		log_zone_warning(j->zone, "discontinuity in chages history (%u -> %u), dropping older changesets",
+	bool is_bootstrap = (chs_head->soa_from == NULL);
+	uint32_t serial = is_bootstrap ? 0 : knot_soa_serial(&chs_head->soa_from->rrs);
+	if (!is_bootstrap && md_flag(txn, SERIAL_TO_VALID) &&
+	    serial_compare(txn->shadow_md.last_serial_to, serial) != 0) {
+		log_zone_warning(j->zone, "journal, discontinuity in changes history (%u -> %u), dropping older changesets",
 		                 txn->shadow_md.last_serial_to, serial);
-		try_flush
+		if (!md_flushed(txn) && !journal_merge_allowed(j)) {
+			txn->ret = KNOT_EBUSY; // aka try_flush, but don't merge
+			goto store_changeset_cleanup;
+		}
 		drop_journal(j, txn);
 		txn_restart(txn);
 	}
 	WALK_LIST(ch, *changesets) {
 		uint32_t serial_to = knot_soa_serial(&ch->soa_to->rrs);
-		if (inserting_merged && ch == TAIL(*changesets)) {
+		bool is_this_bootstrap = (is_bootstrap && ch == HEAD(*changesets));
+		bool is_this_merged = (inserting_merged && ch == TAIL(*changesets));
+		if (is_this_bootstrap || is_this_merged) {
 			continue;
 		}
 		txn_key_2u32(txn, j->zone, serial_to, 0);
 		if (txn_find(txn)) {
-			log_zone_warning(j->zone, "duplicite changeset serial (%u), dropping older changesets",
+			log_zone_warning(j->zone, "journal, duplicate changeset serial (%u), dropping older changesets",
 			                 serial_to);
 			try_flush
 			delete_upto(j, txn, txn->shadow_md.first_serial, serial_to);
@@ -1197,7 +1374,7 @@ static int store_changesets(journal_t *j, list_t *changesets)
 			txn->ret = KNOT_ENOMEM;
 			break;
 		}
-		for (int i = 0; i < maxchunks; i++) {
+		for (size_t i = 0; i < maxchunks; i++) {
 			chunkptrs[i] = allchunks + i*CHUNK_MAX + JOURNAL_HEADER_SIZE;
 		}
 		txn->ret = changeset_serialize(ch, chunkptrs, CHUNK_MAX - JOURNAL_HEADER_SIZE,
@@ -1206,19 +1383,26 @@ static int store_changesets(journal_t *j, list_t *changesets)
 			break;
 		}
 
-		uint32_t serial = knot_soa_serial(&ch->soa_from->rrs);
+		bool is_this_bootstrap = is_bootstrap && (ch == HEAD(*changesets));
+		bool is_this_merged = (inserting_merged && ch == TAIL(*changesets));
+		uint32_t serial = is_this_bootstrap ? 0 : knot_soa_serial(&ch->soa_from->rrs);
 		uint32_t serial_to = knot_soa_serial(&ch->soa_to->rrs);
 
-		for (int i = 0; i < chunks; i++) {
+		for (size_t i = 0; i < chunks; i++) {
 			vals[i].data = allchunks + i*CHUNK_MAX;
 			vals[i].len = JOURNAL_HEADER_SIZE + chunksizes[i];
 			make_header(vals + i, serial_to, chunks);
 		}
 
 	// PART 6: inserting vals into db
-		for (int i = 0; i < chunks; i++) {
+		for (size_t i = 0; i < chunks; i++) {
 			if (txn->ret != KNOT_EOK) break;
-			txn_key_2u32(txn, j->zone, serial, i);
+			if (is_this_bootstrap) {
+				txn_key_str_u32(txn, j->zone, KEY_BOOTSTRAP_CHANGESET, i);
+			}
+			else {
+				txn_key_2u32(txn, j->zone, serial, i);
+			}
 			txn->val = vals[i];
 			txn_insert(txn);
 			inserted_size += (vals+i)->len;
@@ -1236,14 +1420,22 @@ static int store_changesets(journal_t *j, list_t *changesets)
 		if (txn->ret != KNOT_EOK) {
 			break;
 		}
-		if (inserting_merged && ch == TAIL(*changesets)) {
+		if (is_this_merged) {
 			txn->shadow_md.flags |= MERGED_SERIAL_VALID;
 			txn->shadow_md.merged_serial = serial;
 		}
-		else {
+		else if (is_this_bootstrap) {
 			if (!md_flag(txn, SERIAL_TO_VALID)) {
+				txn->shadow_md.flags |= FIRST_SERIAL_INVALID;
+				txn->shadow_md.last_serial_to = serial_to;
+			}
+			txn->shadow_md.flags |= SERIAL_TO_VALID;
+		}
+		else {
+			if (!md_flag(txn, SERIAL_TO_VALID) || md_flag(txn, FIRST_SERIAL_INVALID)) {
 				txn->shadow_md.first_serial = serial;
 			}
+			txn->shadow_md.flags &= ~FIRST_SERIAL_INVALID;
 			txn->shadow_md.flags |= SERIAL_TO_VALID;
 			txn->shadow_md.last_serial = serial;
 			txn->shadow_md.last_serial_to = serial_to;
@@ -1266,7 +1458,7 @@ store_changeset_cleanup:
 
 	txn_commit(txn);
 
-	if (txn->ret != KNOT_ESEMCHECK) {
+	if (txn->ret != KNOT_EOK) {
 		local_txn_t(ddtxn, j);
 		txn_begin(ddtxn, true);
 		if (md_flag(ddtxn, DIRTY_SERIAL_VALID)) {
@@ -1288,7 +1480,7 @@ store_changeset_cleanup:
 		changeset_free(dbgchst);
 	}
 
-	txn_ret(txn);
+	return txn->ret;
 }
 #undef try_flush
 
@@ -1300,7 +1492,7 @@ int journal_store_changeset(journal_t *journal, changeset_t *ch)
 	if (ch_shallowcopy == NULL) {
 		return KNOT_ENOMEM;
 	}
-	memcpy(ch_shallowcopy, ch, sizeof(changeset_t)); // we need to copy the changeset_t sructure not to break ch->n
+	memcpy(ch_shallowcopy, ch, sizeof(changeset_t)); // we need to copy the changeset_t structure not to break ch->n
 
 	list_t list;
 	init_list(&list);
@@ -1354,6 +1546,9 @@ static int open_journal_db_unsafe(journal_db_t **db)
 	opts.mapsize = (*db)->fslimit;
 	opts.maxdbs = 1;
 	opts.maxreaders = JOURNAL_MAX_READERS;
+	opts.flags.env = ((*db)->mode == JOURNAL_MODE_ASYNC ?
+	                  KNOT_DB_LMDB_WRITEMAP | KNOT_DB_LMDB_MAPASYNC : 0);
+	opts.flags.env |= KNOT_DB_LMDB_NOTLS;
 
 	int ret = (*db)->db_api->init(&(*db)->db, NULL, &opts);
 	if (ret != KNOT_EOK) {
@@ -1367,7 +1562,7 @@ static int open_journal_db_unsafe(journal_db_t **db)
 	return KNOT_EOK;
 }
 
-static int open_journal_db(journal_db_t **db)
+int journal_open_db(journal_db_t **db)
 {
 	if (*db == NULL) return KNOT_EINVAL;
 	pthread_mutex_lock(&(*db)->db_mutex);
@@ -1388,7 +1583,7 @@ int journal_open(journal_t *j, journal_db_t **db, const knot_dname_t *zone_name)
 
 	// open shared journal DB if not already
 	if ((*db)->db == NULL) {
-		ret = open_journal_db(db);
+		ret = journal_open_db(db);
 	}
 	if (ret != KNOT_EOK) {
 		return ret;
@@ -1417,7 +1612,8 @@ void journal_close(journal_t *j)
 	j->zone = NULL;
 }
 
-int journal_db_init(journal_db_t **db, const char *lmdb_dir_path, size_t lmdb_fslimit)
+int journal_db_init(journal_db_t **db, const char *lmdb_dir_path, size_t lmdb_fslimit,
+                    journal_mode_t mode)
 {
 	if (*db != NULL) {
 		return KNOT_EOK;
@@ -1430,7 +1626,8 @@ int journal_db_init(journal_db_t **db, const char *lmdb_dir_path, size_t lmdb_fs
 		.db = NULL,
 		.db_api = knot_db_lmdb_api(),
 		.path = strdup(lmdb_dir_path),
-		.fslimit = ((lmdb_fslimit < JOURNAL_MIN_FSLIMIT) ? JOURNAL_MIN_FSLIMIT : lmdb_fslimit)
+		.fslimit = ((lmdb_fslimit < JOURNAL_MIN_FSLIMIT) ? JOURNAL_MIN_FSLIMIT : lmdb_fslimit),
+		.mode = mode,
 	};
 	memcpy(*db, &dbinit, sizeof(journal_db_t));
 	pthread_mutex_init(&(*db)->db_mutex, NULL);
@@ -1473,7 +1670,7 @@ int journal_flush(journal_t *journal)
 	txn_begin(txn, true);
 	md_flush(txn);
 	txn_commit(txn);
-	txn_ret(txn);
+	return txn->ret;
 }
 
 bool journal_exists(journal_db_t **db, knot_dname_t *zone_name)
@@ -1487,7 +1684,7 @@ bool journal_exists(journal_db_t **db, knot_dname_t *zone_name)
 		if (stat((*db)->path, &st) != 0 || st.st_size == 0) {
 			return false;
 		}
-		int ret = open_journal_db(db);
+		int ret = journal_open_db(db);
 		if (ret != KNOT_EOK) {
 			return false;
 		}
@@ -1497,10 +1694,10 @@ bool journal_exists(journal_db_t **db, knot_dname_t *zone_name)
 	local_txn_t(txn, &fake_journal);
 	txn_begin(txn, false);
 	txn_key_str(txn, zone_name, MDKEY_PERZONE_FLAGS);
-	int res = txn_find(txn);
+	bool res = txn_find(txn);
 	txn_abort(txn);
 
-	return (res == 1);
+	return res;
 }
 
 static knot_db_val_t *dbval_copy(const knot_db_val_t *from)
@@ -1514,7 +1711,7 @@ static knot_db_val_t *dbval_copy(const knot_db_val_t *from)
 	return to;
 } // TODO think of moving this fun into different place/lib
 
-int scrape_journal(journal_t *j)
+int journal_scrape(journal_t *j)
 {
 	if (j->db == NULL) return KNOT_EINVAL;
 	local_txn_t(txn, j);
@@ -1549,16 +1746,16 @@ int scrape_journal(journal_t *j)
 		WALK_LIST(del_one, to_del) {
 			txn->ret = j->db->db_api->del(txn->txn, (knot_db_val_t *)del_one->d);
 		}
-		md_update_journal_count(txn, -1);
+		if (!EMPTY_LIST(to_del)) {
+			md_update_journal_count(txn, -1);
+		}
+
+		md_del_last_inserter_zone(txn, j->zone);
+
 		txn->ret = j->db->db_api->txn_commit(txn->txn);
 	}
 scrape_end:
-
-	WALK_LIST(del_one, to_del) {
-		free(del_one->d);
-	}
-
-	ptrlist_free(&to_del, NULL);
+	ptrlist_deep_free(&to_del, NULL);
 
 	return txn->ret;
 }
@@ -1574,7 +1771,7 @@ void journal_metadata_info(journal_t *j, bool *is_empty, uint32_t *serial_from,
 
 	local_txn_t(txn, j);
 	txn_begin(txn, false);
-	txn_check(txn);
+	txn_check_open(txn);
 
 	*is_empty = !md_flag(txn, SERIAL_TO_VALID);
 	*serial_from = txn->shadow_md.first_serial;
@@ -1588,14 +1785,14 @@ void journal_metadata_info(journal_t *j, bool *is_empty, uint32_t *serial_from,
 
 int journal_db_list_zones(journal_db_t **db, list_t *zones)
 {
-	uint32_t expected_count;
+	uint64_t expected_count;
 
 	if (list_size(zones) > 0) {
 		return KNOT_EINVAL;
 	}
 
 	if ((*db)->db == NULL) {
-		int ret = open_journal_db(db);
+		int ret = journal_open_db(db);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
@@ -1634,8 +1831,10 @@ int journal_db_list_zones(journal_db_t **db, list_t *zones)
 	}
 	if (list_size(zones) != expected_count) {
 		txn->ret = KNOT_EMALF;
+		ptrlist_deep_free(zones, NULL);
+		init_list(zones);
 	}
-	txn_ret(txn);
+	return txn->ret;
 }
 
 /*
@@ -1650,6 +1849,7 @@ static void _jch_print(const knot_dname_t *zname, int warn_level, const char *fo
 {
 	static char buf[512];
 	strcpy(buf, "journal check: ");
+	char *zname_ch;
 
 	va_list args;
 	va_start(args, format);
@@ -1657,6 +1857,11 @@ static void _jch_print(const knot_dname_t *zname, int warn_level, const char *fo
 	va_end(args);
 
 	switch (warn_level) {
+	case JOURNAL_CHECK_STDERR:
+		zname_ch = knot_dname_to_str_alloc(zname);
+		fprintf(stderr, "[%s] %s\n", zname_ch, buf);
+		free(zname_ch);
+		break;
 	case JOURNAL_CHECK_INFO:
 		log_zone_info(zname, "%s", buf);
 		break;
@@ -1668,14 +1873,14 @@ static void _jch_print(const knot_dname_t *zname, int warn_level, const char *fo
 	}
 }
 
-#define jch_print(wl, fmt_args...) if ((wl) <= warn_level) _jch_print(j->zone, wl, fmt_args)
+#define jch_print(wl, fmt_args...) if ((wl) <= warn_level) _jch_print(j->zone, warn_level, fmt_args)
 #define jch_info(fmt_args...) jch_print(JOURNAL_CHECK_INFO, fmt_args)
 #define jch_warn(fmt_args...) jch_print((allok = 0, JOURNAL_CHECK_WARN), fmt_args)
-#define jch_txn(comment, fatal) do { if (txn->ret != KNOT_EOK && txn->ret != KNOT_ESEMCHECK) { \
+#define jch_txn(comment, fatal) do { if (txn->ret != KNOT_EOK) { \
                                      jch_warn("failed transaction: %s (%s)", (comment), knot_strerror(txn->ret)); \
                                      if (fatal) return txn->ret; } } while (0)
 
-int journal_check(journal_t *j, journal_check_level warn_level)
+int journal_check(journal_t *j, journal_check_level_t warn_level)
 {
 	int ret, allok = 1;
 	changeset_t *ch = NULL;
@@ -1716,6 +1921,34 @@ int journal_check(journal_t *j, journal_check_level warn_level)
 		goto check_merged;
 	}
 
+	if (md_flag(txn, FIRST_SERIAL_INVALID)) {
+		jch_info("there is just the bootstrap changeset in journal");
+		ret = load_bootstrap_changeset(j, txn, &ch);
+		if (ret != KNOT_EOK) {
+			jch_warn("can't read bootstrap changeset (%s)", knot_strerror(ret));
+		}
+		else {
+			changeset_free(ch);
+		}
+		goto check_merged;
+	}
+	else {
+		ret = load_bootstrap_changeset(j, txn, &ch);
+		switch (ret) {
+		case KNOT_EOK:
+			sto = knot_soa_serial(&ch->soa_to->rrs);
+			jch_info("bootstrap changeset loaded, sto %u", sto);
+			changeset_free(ch);
+			break;
+		case KNOT_ENOENT:
+			txn->ret = KNOT_EOK;
+			break;
+		default:
+			jch_info("failed to read bootstrap changeset (%s)", knot_strerror(ret));
+			break;
+		}
+	}
+
 	ret = load_one(j, txn, txn->shadow_md.first_serial, &ch);
 	if (ret != KNOT_EOK) {
 		jch_warn("can't read first changeset %u (%s)",
@@ -1789,7 +2022,7 @@ int journal_check(journal_t *j, journal_check_level warn_level)
 	changesets_free(&l);
 
 check_merged:
-	if (txn->ret != KNOT_ESEMCHECK) txn_abort(txn);
+	if (txn->opened) txn_abort(txn);
 	txn_begin(txn, false);
 	jch_txn("begin2", true);
 	if (md_flag(txn, MERGED_SERIAL_VALID)) {
