@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,39 +14,34 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <sys/stat.h>
+#include <netinet/in.h>
 
 #include "contrib/dnstap/dnstap.h"
 #include "contrib/dnstap/dnstap.pb-c.h"
 #include "contrib/dnstap/message.h"
 #include "contrib/dnstap/writer.h"
-#include "contrib/mempattern.h"
 #include "contrib/time.h"
-#include "knot/modules/dnstap/dnstap.h"
+#include "knot/include/module.h"
 
-/* Module configuration scheme. */
 #define MOD_SINK	"\x04""sink"
 #define MOD_IDENTITY	"\x08""identity"
 #define MOD_VERSION	"\x07""version"
 #define MOD_QUERIES	"\x0B""log-queries"
 #define MOD_RESPONSES	"\x0D""log-responses"
 
-const yp_item_t scheme_mod_dnstap[] = {
-	{ C_ID,          YP_TSTR,  YP_VNONE },
+const yp_item_t dnstap_conf[] = {
 	{ MOD_SINK,      YP_TSTR,  YP_VNONE },
 	{ MOD_IDENTITY,  YP_TSTR,  YP_VNONE },
-	{ MOD_VERSION,   YP_TSTR,  YP_VSTR = { "Knot DNS " PACKAGE_VERSION } },
+	{ MOD_VERSION,   YP_TSTR,  YP_VNONE },
 	{ MOD_QUERIES,   YP_TBOOL, YP_VBOOL = { true } },
 	{ MOD_RESPONSES, YP_TBOOL, YP_VBOOL = { true } },
-	{ C_COMMENT,     YP_TSTR,  YP_VNONE },
 	{ NULL }
 };
 
-int check_mod_dnstap(conf_check_t *args)
+int dnstap_conf_check(knotd_conf_check_args_t *args)
 {
-	conf_val_t sink = conf_rawid_get_txn(args->conf, args->txn, C_MOD_DNSTAP,
-	                                     MOD_SINK, args->id, args->id_len);
-	if (sink.code != KNOT_EOK) {
+	knotd_conf_t sink = knotd_conf_check_item(args, MOD_SINK);
+	if (sink.count == 0 || sink.single.string[0] == '\0') {
 		args->err_str = "no sink specified";
 		return KNOT_EINVAL;
 	}
@@ -62,18 +57,20 @@ typedef struct {
 	size_t version_len;
 } dnstap_ctx_t;
 
-static int log_message(int state, const knot_pkt_t *pkt, struct query_data *qdata,
-                       dnstap_ctx_t *ctx)
+static knotd_state_t log_message(knotd_state_t state, const knot_pkt_t *pkt,
+                                 knotd_qdata_t *qdata, knotd_mod_t *mod)
 {
-	assert(pkt && qdata && ctx);
+	assert(pkt && qdata && mod);
 
 	/* Skip empty packet. */
 	if (pkt->size == 0) {
 		return state;
 	}
 
+	dnstap_ctx_t *ctx = knotd_mod_ctx(mod);
+
 	struct fstrm_iothr_queue *ioq =
-		fstrm_iothr_get_input_queue_idx(ctx->iothread, qdata->param->thread_id);
+		fstrm_iothr_get_input_queue_idx(ctx->iothread, qdata->params->thread_id);
 
 	/* Unless we want to measure the time it takes to process each query,
 	 * we can treat Q/R times the same. */
@@ -87,14 +84,14 @@ static int log_message(int state, const knot_pkt_t *pkt, struct query_data *qdat
 
 	/* Determine whether we run on UDP/TCP. */
 	int protocol = IPPROTO_TCP;
-	if (qdata->param->proc_flags & NS_QUERY_LIMIT_SIZE) {
+	if (qdata->params->flags & KNOTD_QUERY_FLAG_LIMIT_SIZE) {
 		protocol = IPPROTO_UDP;
 	}
 
 	/* Create a dnstap message. */
 	Dnstap__Message msg;
 	int ret = dt_message_fill(&msg, msgtype,
-	                          (const struct sockaddr *)qdata->param->remote,
+	                          (const struct sockaddr *)qdata->params->remote,
 	                          NULL, /* todo: fill me! */
 	                          protocol, pkt->wire, pkt->size, &tv, &tv);
 	if (ret != KNOT_EOK) {
@@ -137,19 +134,19 @@ static int log_message(int state, const knot_pkt_t *pkt, struct query_data *qdat
 }
 
 /*! \brief Submit message - query. */
-static int dnstap_message_log_query(int state, knot_pkt_t *pkt, struct query_data *qdata,
-                                    void *ctx)
+static knotd_state_t dnstap_message_log_query(knotd_state_t state, knot_pkt_t *pkt,
+                                              knotd_qdata_t *qdata, knotd_mod_t *mod)
 {
 	assert(qdata);
 
-	return log_message(state, qdata->query, qdata, ctx);
+	return log_message(state, qdata->query, qdata, mod);
 }
 
 /*! \brief Submit message - response. */
-static int dnstap_message_log_response(int state, knot_pkt_t *pkt, struct query_data *qdata,
-                                       void *ctx)
+static knotd_state_t dnstap_message_log_response(knotd_state_t state, knot_pkt_t *pkt,
+                                                 knotd_qdata_t *qdata, knotd_mod_t *mod)
 {
-	return log_message(state, pkt, qdata, ctx);
+	return log_message(state, pkt, qdata, mod);
 }
 
 /*! \brief Create a UNIX socket sink. */
@@ -220,42 +217,47 @@ static struct fstrm_writer* dnstap_writer(const char *path)
 	return dnstap_file_writer(path);
 }
 
-int dnstap_load(struct query_module *self)
+int dnstap_load(knotd_mod_t *mod)
 {
-	assert(self);
-
 	/* Create dnstap context. */
-	dnstap_ctx_t *ctx = mm_alloc(self->mm, sizeof(*ctx));
+	dnstap_ctx_t *ctx = calloc(1, sizeof(*ctx));
 	if (ctx == NULL) {
 		return KNOT_ENOMEM;
 	}
 
-	conf_val_t val;
-
 	/* Set identity. */
-	val = conf_mod_get(self->config, MOD_IDENTITY, self->id);
-	if (val.code == KNOT_EOK) {
-		ctx->identity = strdup(conf_str(&val));
+	knotd_conf_t conf = knotd_conf_mod(mod, MOD_IDENTITY);
+	if (conf.count == 1) {
+		ctx->identity = (conf.single.string != NULL) ?
+		                strdup(conf.single.string) : NULL;
 	} else {
-		ctx->identity = sockaddr_hostname();
+		knotd_conf_t host = knotd_conf_env(mod, KNOTD_CONF_ENV_HOSTNAME);
+		ctx->identity = strdup(host.single.string);
 	}
 	ctx->identity_len = (ctx->identity != NULL) ? strlen(ctx->identity) : 0;
 
 	/* Set version. */
-	val = conf_mod_get(self->config, MOD_VERSION, self->id);
-	ctx->version = strdup(conf_str(&val));
-	ctx->version_len = strlen(ctx->version);
+	conf = knotd_conf_mod(mod, MOD_VERSION);
+	if (conf.count == 1) {
+		ctx->version = (conf.single.string != NULL) ?
+		               strdup(conf.single.string) : NULL;
+	} else {
+		knotd_conf_t version = knotd_conf_env(mod, KNOTD_CONF_ENV_VERSION);
+		ctx->version = strdup(version.single.string);
+	}
+	ctx->version_len = (ctx->version != NULL) ? strlen(ctx->version) : 0;
 
-	val = conf_mod_get(self->config, MOD_SINK, self->id);
-	const char *sink = conf_str(&val);
+	/* Set sink. */
+	conf = knotd_conf_mod(mod, MOD_SINK);
+	const char *sink = conf.single.string;
 
 	/* Set log_queries. */
-	val = conf_mod_get(self->config, MOD_QUERIES, self->id);
-	const bool log_queries = conf_bool(&val);
+	conf = knotd_conf_mod(mod, MOD_QUERIES);
+	const bool log_queries = conf.single.boolean;
 
 	/* Set log_responses. */
-	val = conf_mod_get(self->config, MOD_RESPONSES, self->id);
-	const bool log_responses = conf_bool(&val);
+	conf = knotd_conf_mod(mod, MOD_RESPONSES);
+	const bool log_responses = conf.single.boolean;
 
 	/* Initialize the writer and the options. */
 	struct fstrm_writer *writer = dnstap_writer(sink);
@@ -270,8 +272,9 @@ int dnstap_load(struct query_module *self)
 	}
 
 	/* Initialize queues. */
-	size_t qcount = conf_udp_threads(self->config) +
-	                conf_tcp_threads(self->config);
+	knotd_conf_t udp = knotd_conf_env(mod, KNOTD_CONF_ENV_WORKERS_UDP);
+	knotd_conf_t tcp = knotd_conf_env(mod, KNOTD_CONF_ENV_WORKERS_TCP);
+	size_t qcount = udp.single.integer + tcp.single.integer;
 	fstrm_iothr_options_set_num_input_queues(opt, qcount);
 
 	/* Create the I/O thread. */
@@ -282,35 +285,36 @@ int dnstap_load(struct query_module *self)
 		goto fail;
 	}
 
-	self->ctx = ctx;
+	knotd_mod_ctx_set(mod, ctx);
 
 	/* Hook to the query plan. */
 	if (log_queries) {
-		query_module_step(self, QPLAN_BEGIN, dnstap_message_log_query);
+		knotd_mod_hook(mod, KNOTD_STAGE_BEGIN, dnstap_message_log_query);
 	}
 	if (log_responses) {
-		query_module_step(self, QPLAN_END, dnstap_message_log_response);
+		knotd_mod_hook(mod, KNOTD_STAGE_END, dnstap_message_log_response);
 	}
 
 	return KNOT_EOK;
 fail:
-	MODULE_ERR(C_MOD_DNSTAP, "failed to init sink '%s'", sink);
+	knotd_mod_log(mod, LOG_ERR, "failed to init sink '%s'", sink);
 
 	free(ctx->identity);
 	free(ctx->version);
-	mm_free(self->mm, ctx);
+	free(ctx);
 
 	return KNOT_ENOMEM;
 }
 
-void dnstap_unload(struct query_module *self)
+void dnstap_unload(knotd_mod_t *mod)
 {
-	assert(self);
-
-	dnstap_ctx_t *ctx = self->ctx;
+	dnstap_ctx_t *ctx = knotd_mod_ctx(mod);
 
 	fstrm_iothr_destroy(&ctx->iothread);
 	free(ctx->identity);
 	free(ctx->version);
-	mm_free(self->mm, ctx);
+	free(ctx);
 }
+
+KNOTD_MOD_API(dnstap, KNOTD_MOD_FLAG_SCOPE_ANY,
+              dnstap_load, dnstap_unload, dnstap_conf, dnstap_conf_check);
