@@ -16,19 +16,18 @@
 
 #include <urcu.h>
 
-#include "knot/nameserver/internet.h"
+#include "contrib/mempattern.h"
+#include "contrib/sockaddr.h"
 #include "knot/nameserver/axfr.h"
+#include "knot/nameserver/internet.h"
 #include "knot/nameserver/ixfr.h"
 #include "knot/nameserver/log.h"
 #include "knot/nameserver/xfr.h"
 #include "knot/zone/serial.h"
-#include "knot/zone/zonefile.h"
 #include "libknot/libknot.h"
-#include "contrib/mempattern.h"
-#include "contrib/sockaddr.h"
 
 #define ZONE_NAME(qdata) knot_pkt_qname((qdata)->query)
-#define REMOTE(qdata) (struct sockaddr *)(qdata)->param->remote
+#define REMOTE(qdata) (struct sockaddr *)(qdata)->params->remote
 
 #define IXFROUT_LOG(priority, qdata, fmt...) \
 	ns_log(priority, ZONE_NAME(qdata), LOG_OPERATION_IXFR, \
@@ -36,7 +35,7 @@
 
 /*! \brief Helper macro for putting RRs into packet. */
 #define IXFR_SAFE_PUT(pkt, rr) \
-	ret = knot_pkt_put((pkt), 0, (rr), KNOT_PF_NOTRUNC); \
+	int ret = knot_pkt_put((pkt), 0, (rr), KNOT_PF_NOTRUNC); \
 	if (ret != KNOT_EOK) { \
 		return ret; \
 	}
@@ -53,13 +52,12 @@ static int ixfr_put_chg_part(knot_pkt_t *pkt, struct ixfr_proc *ixfr,
 		ixfr->cur_rr = changeset_iter_next(itt);
 	}
 
-	int ret = KNOT_EOK; // Declaration for IXFR_SAFE_PUT macro
 	while (!knot_rrset_empty(&ixfr->cur_rr)) {
 		IXFR_SAFE_PUT(pkt, &ixfr->cur_rr);
 		ixfr->cur_rr = changeset_iter_next(itt);
 	}
 
-	return ret;
+	return KNOT_EOK;
 }
 
 /*! \brief Tests if iteration has started. */
@@ -136,7 +134,6 @@ static int ixfr_process_changeset(knot_pkt_t *pkt, const void *item,
 
 #undef IXFR_SAFE_PUT
 
-/*! \brief Loads IXFRs from journal. */
 static int ixfr_load_chsets(list_t *chgsets, zone_t *zone,
                             const knot_rrset_t *their_soa)
 {
@@ -159,8 +156,7 @@ static int ixfr_load_chsets(list_t *chgsets, zone_t *zone,
 	return ret;
 }
 
-/*! \brief Check IXFR query validity. */
-static int ixfr_query_check(struct query_data *qdata)
+static int ixfr_query_check(knotd_qdata_t *qdata)
 {
 	/* Check if zone exists. */
 	NS_NEED_ZONE(qdata, KNOT_RCODE_NOTAUTH);
@@ -178,33 +174,32 @@ static int ixfr_query_check(struct query_data *qdata)
 	NS_NEED_QNAME(qdata, their_soa->owner, KNOT_RCODE_FORMERR);
 
 	/* Check transcation security and zone contents. */
-	NS_NEED_AUTH(qdata, qdata->zone->name, ACL_ACTION_TRANSFER);
+	NS_NEED_AUTH(qdata, qdata->extra->zone->name, ACL_ACTION_TRANSFER);
 	NS_NEED_ZONE_CONTENTS(qdata, KNOT_RCODE_SERVFAIL); /* Check expiration. */
 
 	return KNOT_STATE_DONE;
 }
 
-/*! \brief Cleans up ixfr processing context. */
-static void ixfr_answer_cleanup(struct query_data *qdata)
+static void ixfr_answer_cleanup(knotd_qdata_t *qdata)
 {
-	struct ixfr_proc *ixfr = (struct ixfr_proc *)qdata->ext;
+	struct ixfr_proc *ixfr = (struct ixfr_proc *)qdata->extra->ext;
 	knot_mm_t *mm = qdata->mm;
 
 	ptrlist_free(&ixfr->proc.nodes, mm);
 	changeset_iter_clear(&ixfr->cur);
 	changesets_free(&ixfr->changesets);
-	mm_free(mm, qdata->ext);
+	mm_free(mm, qdata->extra->ext);
 
 	/* Allow zone changes (finished). */
 	rcu_read_unlock();
 }
 
-/*! \brief Inits ixfr processing context. */
-static int ixfr_answer_init(struct query_data *qdata)
+static int ixfr_answer_init(knotd_qdata_t *qdata)
 {
+	assert(qdata);
+
 	/* Check IXFR query validity. */
-	int state = ixfr_query_check(qdata);
-	if (state == KNOT_STATE_FAIL) {
+	if (ixfr_query_check(qdata) == KNOT_STATE_FAIL) {
 		if (qdata->rcode == KNOT_RCODE_FORMERR) {
 			return KNOT_EMALF;
 		} else {
@@ -217,7 +212,7 @@ static int ixfr_answer_init(struct query_data *qdata)
 	const knot_rrset_t *their_soa = knot_pkt_rr(authority, 0);
 	list_t chgsets;
 	init_list(&chgsets);
-	int ret = ixfr_load_chsets(&chgsets, (zone_t *)qdata->zone, their_soa);
+	int ret = ixfr_load_chsets(&chgsets, (zone_t *)qdata->extra->zone, their_soa);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -252,21 +247,19 @@ static int ixfr_answer_init(struct query_data *qdata)
 	xfer->soa_to = chs->soa_to;
 
 	/* Set up cleanup callback. */
-	qdata->ext = xfer;
-	qdata->ext_cleanup = &ixfr_answer_cleanup;
+	qdata->extra->ext = xfer;
+	qdata->extra->ext_cleanup = &ixfr_answer_cleanup;
 
-	/* No zone changes during multipacket answer (unlocked in axfr_answer_cleanup) */
+	/* No zone changes during multipacket answer (unlocked in ixfr_answer_cleanup) */
 	rcu_read_lock();
 
 	return KNOT_EOK;
 }
 
-/*! \brief Sends response to SOA query. */
-static int ixfr_answer_soa(knot_pkt_t *pkt, struct query_data *qdata)
+static int ixfr_answer_soa(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 {
-	if (pkt == NULL || qdata == NULL) {
-		return KNOT_STATE_FAIL;
-	}
+	assert(pkt);
+	assert(qdata);
 
 	/* Check query. */
 	int state = ixfr_query_check(qdata);
@@ -278,7 +271,7 @@ static int ixfr_answer_soa(knot_pkt_t *pkt, struct query_data *qdata)
 	knot_pkt_reserve(pkt, knot_tsig_wire_size(&qdata->sign.tsig_key));
 
 	/* Guaranteed to have zone contents. */
-	const zone_node_t *apex = qdata->zone->contents->apex;
+	const zone_node_t *apex = qdata->extra->zone->contents->apex;
 	knot_rrset_t soa_rr = node_rrset(apex, KNOT_RRTYPE_SOA);
 	if (knot_rrset_empty(&soa_rr)) {
 		return KNOT_STATE_FAIL;
@@ -292,26 +285,24 @@ static int ixfr_answer_soa(knot_pkt_t *pkt, struct query_data *qdata)
 	return KNOT_STATE_DONE;
 }
 
-int ixfr_process_query(knot_pkt_t *pkt, struct query_data *qdata)
+int ixfr_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 {
 	if (pkt == NULL || qdata == NULL) {
 		return KNOT_STATE_FAIL;
 	}
 
-	int ret = KNOT_EOK;
-	struct ixfr_proc *ixfr = (struct ixfr_proc *)qdata->ext;
-
 	/* If IXFR is disabled, respond with SOA. */
-	if (qdata->param->proc_flags & NS_QUERY_NO_IXFR) {
+	if (qdata->params->flags & KNOTD_QUERY_FLAG_NO_IXFR) {
 		return ixfr_answer_soa(pkt, qdata);
 	}
 
 	/* Initialize on first call. */
-	if (qdata->ext == NULL) {
-		ret = ixfr_answer_init(qdata);
+	struct ixfr_proc *ixfr = qdata->extra->ext;
+	if (ixfr == NULL) {
+		int ret = ixfr_answer_init(qdata);
+		ixfr = qdata->extra->ext;
 		switch (ret) {
 		case KNOT_EOK:      /* OK */
-			ixfr = (struct ixfr_proc *)qdata->ext;
 			IXFROUT_LOG(LOG_INFO, qdata, "started, serial %u -> %u",
 			            knot_soa_serial(&ixfr->soa_from->rrs),
 			            knot_soa_serial(&ixfr->soa_to->rrs));
@@ -322,10 +313,11 @@ int ixfr_process_query(knot_pkt_t *pkt, struct query_data *qdata)
 		case KNOT_ERANGE:   /* No history -> AXFR. */
 		case KNOT_ENOENT:
 			IXFROUT_LOG(LOG_INFO, qdata, "incomplete history, fallback to AXFR");
-			qdata->packet_type = KNOT_QUERY_AXFR; /* Solve as AXFR. */
+			qdata->type = KNOTD_QUERY_TYPE_AXFR; /* Solve as AXFR. */
 			return axfr_process_query(pkt, qdata);
 		default:            /* Server errors. */
-			IXFROUT_LOG(LOG_ERR, qdata, "failed to start (%s)", knot_strerror(ret));
+			IXFROUT_LOG(LOG_ERR, qdata, "failed to start (%s)",
+			            knot_strerror(ret));
 			return KNOT_STATE_FAIL;
 		}
 	}
@@ -334,7 +326,7 @@ int ixfr_process_query(knot_pkt_t *pkt, struct query_data *qdata)
 	knot_pkt_reserve(pkt, knot_tsig_wire_size(&qdata->sign.tsig_key));
 
 	/* Answer current packet (or continue). */
-	ret = xfr_process_list(pkt, &ixfr_process_changeset, qdata);
+	int ret = xfr_process_list(pkt, &ixfr_process_changeset, qdata);
 	switch (ret) {
 	case KNOT_ESPACE: /* Couldn't write more, send packet and continue. */
 		return KNOT_STATE_PRODUCE; /* Check for more. */
@@ -342,13 +334,9 @@ int ixfr_process_query(knot_pkt_t *pkt, struct query_data *qdata)
 		xfr_stats_end(&ixfr->proc.stats);
 		xfr_log_finished(ZONE_NAME(qdata), LOG_OPERATION_IXFR, LOG_DIRECTION_OUT,
 		                 REMOTE(qdata), &ixfr->proc.stats);
-		ret = KNOT_STATE_DONE;
-		break;
+		return KNOT_STATE_DONE;
 	default:          /* Generic error. */
 		IXFROUT_LOG(LOG_ERR, qdata, "failed (%s)", knot_strerror(ret));
-		ret = KNOT_STATE_FAIL;
-		break;
+		return KNOT_STATE_FAIL;
 	}
-
-	return ret;
 }
