@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,10 +21,12 @@
 #include <urcu.h>
 
 #include "libknot/errcode.h"
+#include "libknot/yparser/ypschema.h"
 #include "knot/common/log.h"
 #include "knot/common/stats.h"
 #include "knot/conf/confio.h"
 #include "knot/conf/migration.h"
+#include "knot/conf/module.h"
 #include "knot/server/server.h"
 #include "knot/server/udp-handler.h"
 #include "knot/server/tcp-handler.h"
@@ -182,7 +184,7 @@ static int server_init_iface(iface_t *new_if, struct sockaddr_storage *addr, int
 			udp_bind_flags |= NET_BIND_NONLOCAL;
 			sock = net_bound_socket(SOCK_DGRAM, (struct sockaddr *)addr, udp_bind_flags);
 			if (sock >= 0 && !warn_bind) {
-				log_warning("address '%s' is not available", addr_str);
+				log_warning("address '%s' bound, but required nonlocal bind", addr_str);
 				warn_bind = true;
 			}
 		}
@@ -209,7 +211,16 @@ static int server_init_iface(iface_t *new_if, struct sockaddr_storage *addr, int
 	}
 
 	/* Create bound TCP socket. */
-	int sock = net_bound_socket(SOCK_STREAM, (struct sockaddr *)addr, 0);
+	int tcp_bind_flags = 0;
+	int sock = net_bound_socket(SOCK_STREAM, (struct sockaddr *)addr, tcp_bind_flags);
+	if (sock == KNOT_EADDRNOTAVAIL) {
+		tcp_bind_flags |= NET_BIND_NONLOCAL;
+		sock = net_bound_socket(SOCK_STREAM, (struct sockaddr *)addr, tcp_bind_flags);
+		if (sock >= 0) {
+			log_warning("address '%s' bound, but required nonlocal bind", addr_str);
+		}
+	}
+
 	if (sock < 0) {
 		log_error("cannot bind address '%s' (%s)", addr_str,
 		          knot_strerror(sock));
@@ -381,8 +392,9 @@ int server_init(server_t *server, int bg_workers)
 
 	char *journal_dir = conf_journalfile(conf());
 	conf_val_t journal_size = conf_default_get(conf(), C_MAX_JOURNAL_DB_SIZE);
+	conf_val_t journal_mode = conf_default_get(conf(), C_JOURNAL_DB_MODE);
 	int ret = journal_db_init(&server->journal_db, journal_dir,
-	                          conf_int(&journal_size));
+	                          conf_int(&journal_size), conf_opt(&journal_mode));
 	free(journal_dir);
 	if (ret != KNOT_EOK) {
 		worker_pool_destroy(server->workers);
@@ -390,6 +402,17 @@ int server_init(server_t *server, int bg_workers)
 		return ret;
 	}
 
+	char *kasp_dir = conf_kaspdir(conf());
+	conf_val_t kasp_size = conf_default_get(conf(), C_MAX_KASP_DB_SIZE);
+	ret = kasp_db_init(kaspdb(), kasp_dir, conf_int(&kasp_size));
+	free(kasp_dir);
+	if (ret != KNOT_EOK) {
+		journal_db_close(&server->journal_db);
+		worker_pool_destroy(server->workers);
+		evsched_deinit(&server->sched);
+		return ret;
+	}
+
 	return KNOT_EOK;
 }
 
@@ -417,6 +440,9 @@ void server_deinit(server_t *server)
 	/* Free remaining events. */
 	evsched_deinit(&server->sched);
 
+	/* Close kasp_db. */
+	kasp_db_close(kaspdb());
+
 	/* Close journal database if open. */
 	journal_db_close(&server->journal_db);
 
@@ -523,25 +549,46 @@ void server_wait(server_t *server)
 
 static int reload_conf(conf_t *new_conf)
 {
+	yp_schema_purge_dynamic(new_conf->schema);
+
+	/* Re-load common modules. */
+	int ret = conf_mod_load_common(new_conf);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
 	/* Re-import zonefile if specified. */
 	const char *filename = conf()->filename;
 	if (filename != NULL) {
 		log_info("reloading configuration file '%s'", filename);
 
 		/* Import the configuration file. */
-		int ret = conf_import(new_conf, filename, true);
+		ret = conf_import(new_conf, filename, true);
 		if (ret != KNOT_EOK) {
 			log_error("failed to load configuration file (%s)",
 			          knot_strerror(ret));
-			conf_free(new_conf);
 			return ret;
 		}
 	} else {
 		log_info("reloading configuration database");
+
+		/* Re-load extra modules. */
+		for (conf_iter_t iter = conf_iter(new_conf, C_MODULE);
+		     iter.code == KNOT_EOK; conf_iter_next(new_conf, &iter)) {
+			conf_val_t id = conf_iter_id(new_conf, &iter);
+			conf_val_t file = conf_id_get(new_conf, C_MODULE, C_FILE, &id);
+			ret = conf_mod_load_extra(new_conf, conf_str(&id), conf_str(&file), false);
+			if (ret != KNOT_EOK) {
+				conf_iter_finish(new_conf, &iter);
+				return ret;
+			}
+		}
 	}
 
+	conf_mod_load_purge(new_conf, false);
+
 	// Migrate from old schema.
-	int ret = conf_migrate(new_conf);
+	ret = conf_migrate(new_conf);
 	if (ret != KNOT_EOK) {
 		log_error("failed to migrate configuration (%s)", knot_strerror(ret));
 	}
@@ -576,27 +623,28 @@ int server_reload(server_t *server)
 	bool full = !(flags & CONF_IO_FACTIVE);
 	bool reuse_modules = !full && !(flags & CONF_IO_FRLD_MOD);
 
-	/* Reload configuration if full reload. */
-	if (full) {
+	/* Reload configuration and modules if full reload or a module change. */
+	if (full || !reuse_modules) {
 		ret = reload_conf(new_conf);
 		if (ret != KNOT_EOK) {
+			conf_free(new_conf);
 			return ret;
 		}
-	}
 
-	/* Load global modules if full reload or required. */
-	if (full || !reuse_modules) {
 		conf_activate_modules(new_conf, NULL, &new_conf->query_modules,
 		                      &new_conf->query_plan);
 	}
 
-	conf_update_flag_t upd_flags = full ? CONF_UPD_FNONE : CONF_UPD_FCONFIO;
+	conf_update_flag_t upd_flags = CONF_UPD_FNOFREE;
+	if (full) {
+		upd_flags |= CONF_UPD_FCONFIO;
+	}
 	if (reuse_modules) {
 		upd_flags |= CONF_UPD_FMODULES;
 	}
 
 	/* Update to the new config. */
-	conf_update(new_conf, upd_flags);
+	conf_t *old_conf = conf_update(new_conf, upd_flags);
 
 	/* Reload each component if full reload or a specific one if required. */
 	if (full || (flags & CONF_IO_FRLD_LOG)) {
@@ -610,6 +658,9 @@ int server_reload(server_t *server)
 		server_update_zones(conf(), server);
 	}
 
+	/* Free old config needed for module unload in zone reload. */
+	conf_free(old_conf);
+
 	if (full) {
 		log_info("configuration reloaded");
 	} else {
@@ -678,20 +729,26 @@ static int reconfigure_journal_db(conf_t *conf, server_t *server)
 {
 	char *journal_dir = conf_journalfile(conf);
 	conf_val_t journal_size = conf_default_get(conf, C_MAX_JOURNAL_DB_SIZE);
+	conf_val_t journal_mode = conf_default_get(conf, C_JOURNAL_DB_MODE);
 	bool changed_path = (strcmp(journal_dir, server->journal_db->path) != 0);
 	bool changed_size = (conf_int(&journal_size) != server->journal_db->fslimit);
+	bool changed_mode = (conf_opt(&journal_mode) != server->journal_db->mode);
 	int ret = KNOT_EOK;
 
 	if (server->journal_db->db != NULL) {
 		if (changed_path) {
-			log_warning("journal, ignored reconfiguration of journal DB path (already open)");
+			log_warning("ignored reconfiguration of journal DB path (already open)");
 		}
 		if (changed_size) {
-			log_warning("journal, ignored reconfiguration of journal DB max size (already open)");
+			log_warning("ignored reconfiguration of journal DB max size (already open)");
 		}
-	} else if (changed_path || changed_size) {
+		if (changed_mode) {
+			log_warning("ignored reconfiguration of journal DB mode (already open)");
+		}
+	} else if (changed_path || changed_size || changed_mode) {
 		journal_db_t *newjdb = NULL;
-		ret = journal_db_init(&newjdb, journal_dir, conf_int(&journal_size));
+		ret = journal_db_init(&newjdb, journal_dir, conf_int(&journal_size),
+		                      conf_opt(&journal_mode));
 		if (ret == KNOT_EOK) {
 			journal_db_close(&server->journal_db);
 			server->journal_db = newjdb;
@@ -702,6 +759,31 @@ static int reconfigure_journal_db(conf_t *conf, server_t *server)
 	return ret;
 }
 
+static int reconfigure_kasp_db(conf_t *conf, server_t *server)
+{
+	char *kasp_dir = conf_kaspdir(conf);
+	conf_val_t kasp_size = conf_default_get(conf, C_MAX_KASP_DB_SIZE);
+	int ret = kasp_db_reconfigure(kaspdb(), kasp_dir, conf_int(&kasp_size));
+	switch (ret) {
+	case KNOT_EBUSY:
+		log_warning("ignored reconfiguration of KASP DB path (already open)");
+		break;
+	case KNOT_EEXIST:
+		ret = KNOT_EBUSY;
+		log_warning("ignored reconfiguration of KASP DB max size (already open)");
+		break;
+	case KNOT_ENODIFF:
+	case KNOT_EOK:
+		ret = KNOT_EOK;
+		break;
+	default:
+		break;
+	}
+	free(kasp_dir);
+
+	return ret;
+}
+
 void server_reconfigure(conf_t *conf, server_t *server)
 {
 	if (conf == NULL || server == NULL) {
@@ -726,6 +808,12 @@ void server_reconfigure(conf_t *conf, server_t *server)
 		          knot_strerror(ret));
 	}
 
+	/* Reconfigure KASP DB. */
+	if ((ret = reconfigure_kasp_db(conf, server)) < 0) {
+		log_error("failed to reconfigure KASP DB (%s)",
+		          knot_strerror(ret));
+	}
+
 	/* Update bound sockets. */
 	if ((ret = reconfigure_sockets(conf, server)) < 0) {
 		log_error("failed to reconfigure server sockets (%s)",
@@ -743,10 +831,12 @@ static void reopen_timers_database(conf_t *conf, server_t *server)
 	val = conf_default_get(conf, C_TIMER_DB);
 	char *timer_db = conf_abs_path(&val, storage);
 	free(storage);
+	val = conf_default_get(conf, C_MAX_TIMER_DB_SIZE);
+	size_t mapsize = conf_int(&val);
 
-	int ret = zone_timers_open(timer_db, &server->timers_db);
+	int ret = zone_timers_open(timer_db, &server->timers_db, mapsize);
 	if (ret != KNOT_EOK) {
-		log_warning("cannot open persistent timers DB '%s' (%s)",
+		log_warning("cannot open persistent timer DB '%s' (%s)",
 		            timer_db, knot_strerror(ret));
 	}
 
