@@ -20,6 +20,7 @@
 #include "knot/zone/zone-diff.h"
 #include "knot/zone/zone-load.h"
 #include "knot/zone/zonefile.h"
+#include "knot/dnssec/key-events.h"
 #include "knot/dnssec/zone-events.h"
 #include "knot/updates/apply.h"
 #include "libknot/libknot.h"
@@ -173,8 +174,42 @@ int zone_load_journal(conf_t *conf, zone_t *zone, zone_contents_t *contents)
 	return ret;
 }
 
+int zone_load_from_journal(conf_t *conf, zone_t *zone, zone_contents_t **contents)
+{
+	if (conf == NULL || zone == NULL || contents == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	list_t chgs;
+	init_list(&chgs);
+	int ret = zone_in_journal_load(conf, zone, &chgs);
+	if (ret != KNOT_EOK) {
+		changesets_free(&chgs);
+		return ret; // include ENOENT, which is normal operation
+	}
+
+	changeset_t *boo_ch = (changeset_t *)HEAD(chgs);
+	rem_node(&boo_ch->n);
+	*contents = changeset_to_contents(boo_ch);
+
+	apply_ctx_t a_ctx = { 0 };
+	apply_init_ctx(&a_ctx, *contents, 0);
+	ret = apply_changesets_directly(&a_ctx, &chgs);
+	if (ret == KNOT_EOK) {
+		log_zone_info(zone->name, "zone loaded from journal, serial %u",
+		              zone_contents_serial(*contents));
+	} else {
+		log_zone_error(zone->name, "failed to load zone from journal (%s)",
+		               knot_strerror(ret));
+	}
+	update_cleanup(&a_ctx);
+	changesets_free(&chgs);
+
+	return ret;
+}
+
 int zone_load_post(conf_t *conf, zone_t *zone, zone_contents_t *contents,
-                   uint32_t *dnssec_refresh)
+		   zone_sign_reschedule_t *dnssec_refresh)
 {
 	if (conf == NULL || zone == NULL || contents == NULL) {
 		return KNOT_EINVAL;
@@ -192,6 +227,28 @@ int zone_load_post(conf_t *conf, zone_t *zone, zone_contents_t *contents,
 	val = conf_zone_get(conf, C_IXFR_DIFF, zone->name);
 	bool build_diffs = conf_bool(&val);
 	if (dnssec_enable) {
+		/* Perform NSEC3 resalt and ZSK rollover if needed. */
+		kdnssec_ctx_t kctx = { 0 };
+		ret = kdnssec_ctx_init(conf, &kctx, zone->name, NULL);
+		if (ret != KNOT_EOK) {
+			changeset_clear(&change);
+			return ret;
+		}
+
+		bool ignore1 = false; knot_time_t ignore2 = 0;
+		ret = knot_dnssec_nsec3resalt(&kctx, &ignore1, &ignore2);
+		if (ret != KNOT_EOK) {
+			kdnssec_ctx_deinit(&kctx);
+			changeset_clear(&change);
+			return ret;
+		}
+
+		if (zone_has_key_sbm(&kctx)) {
+			zone_events_schedule_now(zone, ZONE_EVENT_PARENT_DS_Q);
+		}
+
+		kdnssec_ctx_deinit(&kctx);
+
 		ret = knot_dnssec_zone_sign(contents, &change, 0, dnssec_refresh);
 		if (ret != KNOT_EOK) {
 			changeset_clear(&change);
