@@ -132,6 +132,8 @@ void zone_control_clear(zone_t *zone);
 int zone_change_store(conf_t *conf, zone_t *zone, changeset_t *change);
 int zone_changes_store(conf_t *conf, zone_t *zone, list_t *chgs);
 int zone_changes_load(conf_t *conf, zone_t *zone, list_t *dst, uint32_t from);
+int zone_in_journal_load(conf_t *conf, zone_t *zone, list_t *dst);
+int zone_in_journal_store(conf_t *conf, zone_t *zone, zone_contents_t *new_contents);
 
 /*! \brief Synchronize zone file with journal. */
 int zone_flush_journal(conf_t *conf, zone_t *zone);
@@ -173,9 +175,12 @@ int zone_master_try(conf_t *conf, zone_t *zone, zone_master_cb callback,
 
 
 /*! \brief Enqueue UPDATE request for processing. */
-int zone_update_enqueue(zone_t *zone, knot_pkt_t *pkt, struct process_query_param *param);
+int zone_update_enqueue(zone_t *zone, knot_pkt_t *pkt, knotd_qdata_params_t *params);
 
 /*! \brief Dequeue UPDATE request. Returns number of queued updates. */
 size_t zone_update_dequeue(zone_t *zone, list_t *updates);
 
+/*! \brief Write zone contents to zonefile, but into different directory. */
+int zone_dump_to_dir(conf_t *conf, zone_t *zone, const char *dir);
+
 /*! @} */
