@@ -146,6 +146,16 @@ enum knot_error {
 	/* Processing error. */
 	KNOT_LAYER_ERROR,
 
+	/* DNSSEC errors. */
+	KNOT_INVALID_PUBLIC_KEY,
+	KNOT_INVALID_PRIVATE_KEY,
+	KNOT_INVALID_KEY_ALGORITHM,
+	KNOT_INVALID_KEY_SIZE,
+	KNOT_INVALID_KEY_ID,
+	KNOT_INVALID_KEY_NAME,
+	KNOT_NO_PUBLIC_KEY,
+	KNOT_NO_PRIVATE_KEY,
+
 	KNOT_ERROR_MAX = -501
 };
 
