@@ -520,14 +520,8 @@ static int check_digest(const knot_rrset_t *tsig_rr,
 		return KNOT_TSIG_EBADKEY;
 	}
 
-	/* Check time signed. */
-	int ret = check_time_signed(tsig_rr, prev_time_signed);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
 	/* Check that libknot knows the algorithm. */
-	ret = check_algorithm(tsig_rr);
+	int ret = check_algorithm(tsig_rr);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -546,6 +540,9 @@ static int check_digest(const knot_rrset_t *tsig_rr,
 	memset(wire_to_sign, 0, size);
 	memcpy(wire_to_sign, wire, size);
 
+	// restore message ID to which the signature had been created with
+	knot_wire_set_id(wire_to_sign, knot_tsig_rdata_orig_id(tsig_rr));
+
 	uint8_t digest_tmp[KNOT_TSIG_MAX_DIGEST_SIZE];
 	size_t digest_tmp_len = 0;
 	assert(tsig_rr->rrs.rr_count > 0);
@@ -588,6 +585,12 @@ static int check_digest(const knot_rrset_t *tsig_rr,
 		return KNOT_TSIG_EBADSIG;
 	}
 
+	/* Check TSIG validity period, must be after the signature check! */
+	ret = check_time_signed(tsig_rr, prev_time_signed);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
 	return KNOT_EOK;
 }
 
