@@ -1,4 +1,4 @@
-/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <stdlib.h>
 
 #include "libknot/yparser/yptrafo.h"
+#include "libknot/attribute.h"
 #include "libknot/consts.h"
 #include "libknot/dname.h"
 #include "libknot/lookup.h"
@@ -66,6 +67,7 @@ static wire_ctx_t copy_in(
 	return ctx;
 }
 
+_public_
 int yp_str_to_bin(
 	YP_TXT_BIN_PARAMS)
 {
@@ -79,6 +81,7 @@ int yp_str_to_bin(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_str_to_txt(
 	YP_BIN_TXT_PARAMS)
 {
@@ -94,6 +97,7 @@ int yp_str_to_txt(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_bool_to_bin(
 	YP_TXT_BIN_PARAMS)
 {
@@ -114,6 +118,7 @@ int yp_bool_to_bin(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_bool_to_txt(
 	YP_BIN_TXT_PARAMS)
 {
@@ -198,6 +203,7 @@ static int remove_unit(
 	return KNOT_EOK;
 }
 
+_public_
 int yp_int_to_bin(
 	YP_TXT_BIN_PARAMS,
 	int64_t min,
@@ -304,6 +310,7 @@ static void add_unit(
 	}
 }
 
+_public_
 int yp_int_to_txt(
 	YP_BIN_TXT_PARAMS,
 	yp_style_t style)
@@ -349,6 +356,7 @@ static uint8_t sock_type_guess(
 	}
 }
 
+_public_
 int yp_addr_noport_to_bin(
 	YP_TXT_BIN_PARAMS,
 	bool allow_unix)
@@ -391,6 +399,7 @@ int yp_addr_noport_to_bin(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_addr_noport_to_txt(
 	YP_BIN_TXT_PARAMS)
 {
@@ -431,6 +440,7 @@ int yp_addr_noport_to_txt(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_addr_to_bin(
 	YP_TXT_BIN_PARAMS)
 {
@@ -468,6 +478,7 @@ int yp_addr_to_bin(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_addr_to_txt(
 	YP_BIN_TXT_PARAMS)
 {
@@ -502,6 +513,125 @@ int yp_addr_to_txt(
 	YP_CHECK_RET;
 }
 
+_public_
+int yp_addr_range_to_bin(
+	YP_TXT_BIN_PARAMS)
+{
+	YP_CHECK_PARAMS_BIN;
+
+	// Format: 0 - single address, 1 - address prefix, 2 - address range.
+	uint8_t format = 0;
+
+	// Check for the "addr/mask" format.
+	const uint8_t *pos = (uint8_t *)strchr((char *)in->position, '/');
+	if (pos >= stop) {
+		pos = NULL;
+	}
+
+	if (pos != NULL) {
+		format = 1;
+	} else {
+		// Check for the "addr1-addr2" format.
+		pos = (uint8_t *)strchr((char *)in->position, '-');
+		if (pos >= stop) {
+			pos = NULL;
+		}
+		if (pos != NULL) {
+			format = 2;
+		}
+	}
+
+	// Store address1 type position.
+	uint8_t *type1 = out->position;
+
+	// Write the first address.
+	int ret = yp_addr_noport_to_bin(in, out, pos, false);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	wire_ctx_write_u8(out, format);
+
+	switch (format) {
+	case 1:
+		// Skip the separator.
+		wire_ctx_skip(in, sizeof(uint8_t));
+
+		// Write the prefix length.
+		ret = yp_int_to_bin(in, out, stop, 0, (*type1 == 4) ? 32 : 128,
+		                    YP_SNONE);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+		break;
+	case 2:
+		// Skip the separator.
+		wire_ctx_skip(in, sizeof(uint8_t));
+
+		// Store address2 type position.
+		uint8_t *type2 = out->position;
+
+		// Write the second address.
+		ret = yp_addr_noport_to_bin(in, out, stop, false);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+
+		// Check for address mismatch.
+		if (*type1 != *type2) {
+			return KNOT_EINVAL;
+		}
+		break;
+	default:
+		break;
+	}
+
+	YP_CHECK_RET;
+}
+
+_public_
+int yp_addr_range_to_txt(
+	YP_BIN_TXT_PARAMS)
+{
+	YP_CHECK_PARAMS_TXT;
+
+	// Write the first address.
+	int ret = yp_addr_noport_to_txt(in, out);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	uint8_t format = wire_ctx_read_u8(in);
+
+	switch (format) {
+	case 1:
+		// Write the separator.
+		wire_ctx_write_u8(out, '/');
+
+		// Write the prefix length.
+		ret = yp_int_to_txt(in, out, YP_SNONE);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+		break;
+	case 2:
+		// Write the separator.
+		wire_ctx_write_u8(out, '-');
+
+		// Write the second address.
+		ret = yp_addr_noport_to_txt(in, out);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+		break;
+	default:
+		break;
+	}
+
+	YP_CHECK_RET;
+}
+
+_public_
 int yp_option_to_bin(
 	YP_TXT_BIN_PARAMS,
 	const knot_lookup_t *opts)
@@ -520,6 +650,7 @@ int yp_option_to_bin(
 	return KNOT_EINVAL;
 }
 
+_public_
 int yp_option_to_txt(
 	YP_BIN_TXT_PARAMS,
 	const knot_lookup_t *opts)
@@ -543,6 +674,7 @@ int yp_option_to_txt(
 	return KNOT_EINVAL;
 }
 
+_public_
 int yp_dname_to_bin(
 	YP_TXT_BIN_PARAMS)
 {
@@ -580,6 +712,7 @@ int yp_dname_to_bin(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_dname_to_txt(
 	YP_BIN_TXT_PARAMS)
 {
@@ -603,6 +736,7 @@ static int hex_to_num(char hex) {
 	return -1;
 }
 
+_public_
 int yp_hex_to_bin(
 	YP_TXT_BIN_PARAMS)
 {
@@ -645,6 +779,7 @@ int yp_hex_to_bin(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_hex_to_txt(
 	YP_BIN_TXT_PARAMS)
 {
@@ -687,6 +822,7 @@ int yp_hex_to_txt(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_base64_to_bin(
 	YP_TXT_BIN_PARAMS)
 {
@@ -710,6 +846,7 @@ int yp_base64_to_bin(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_base64_to_txt(
 	YP_BIN_TXT_PARAMS)
 {
@@ -732,6 +869,7 @@ int yp_base64_to_txt(
 	YP_CHECK_RET;
 }
 
+_public_
 int yp_item_to_bin(
 	const yp_item_t *item,
 	const char *txt,
@@ -766,6 +904,9 @@ int yp_item_to_bin(
 	case YP_TADDR:
 		ret = yp_addr_to_bin(&in, &out, NULL);
 		break;
+	case YP_TNET:
+		ret = yp_addr_range_to_bin(&in, &out, NULL);
+		break;
 	case YP_TDNAME:
 		ret = yp_dname_to_bin(&in, &out, NULL);
 		break;
@@ -802,6 +943,7 @@ int yp_item_to_bin(
 	return KNOT_EOK;
 }
 
+_public_
 int yp_item_to_txt(
 	const yp_item_t *item,
 	const uint8_t *bin,
@@ -841,6 +983,9 @@ int yp_item_to_txt(
 	case YP_TADDR:
 		ret = yp_addr_to_txt(&in, &out);
 		break;
+	case YP_TNET:
+		ret = yp_addr_range_to_txt(&in, &out);
+		break;
 	case YP_TDNAME:
 		ret = yp_dname_to_txt(&in, &out);
 		break;
@@ -887,6 +1032,7 @@ int yp_item_to_txt(
 	return KNOT_EOK;
 }
 
+_public_
 struct sockaddr_storage yp_addr_noport(
 	const uint8_t *data)
 {
@@ -914,12 +1060,14 @@ struct sockaddr_storage yp_addr_noport(
 	return ss;
 }
 
+_public_
 int64_t yp_int(
 	const uint8_t *data)
 {
 	return (int64_t)wire_read_u64(data);
 }
 
+_public_
 struct sockaddr_storage yp_addr(
 	const uint8_t *data,
 	bool *no_port)
@@ -954,6 +1102,7 @@ struct sockaddr_storage yp_addr(
 	return ss;
 }
 
+_public_
 const size_t yp_bin_len(
 	const uint8_t *data)
 {
