@@ -15,7 +15,6 @@
 */
 
 #include <arpa/inet.h>
-#include <getopt.h>
 #include <locale.h>
 #include <stdio.h>
 #include <string.h>
@@ -531,6 +530,24 @@ static int opt_noignore(const char *arg, void *query)
 	return KNOT_EOK;
 }
 
+static int opt_crypto(const char *arg, void *query)
+{
+	query_t *q = query;
+
+	q->style.style.hide_crypto = false;
+
+	return KNOT_EOK;
+}
+
+static int opt_nocrypto(const char *arg, void *query)
+{
+	query_t *q = query;
+
+	q->style.style.hide_crypto = true;
+
+	return KNOT_EOK;
+}
+
 static int opt_tcp(const char *arg, void *query)
 {
 	query_t *q = query;
@@ -680,7 +697,7 @@ static int opt_bufsize(const char *arg, void *query)
 {
 	query_t *q = query;
 
-	uint16_t num;
+	uint16_t num = 0;
 	if (str_to_u16(arg, &num) != KNOT_EOK) {
 		ERR("invalid +bufsize=%s\n", arg);
 		return KNOT_EINVAL;
@@ -711,22 +728,26 @@ static int opt_padding(const char *arg, void *query)
 {
 	query_t *q = query;
 
-	uint16_t num;
-	if (str_to_u16(arg, &num) != KNOT_EOK) {
-		ERR("invalid +padding=%s\n", arg);
-		return KNOT_EINVAL;
+	if (arg == NULL) {
+		q->padding = -2;
+		return KNOT_EOK;
+	} else {
+		uint16_t num = 0;
+		if (str_to_u16(arg, &num) != KNOT_EOK) {
+			ERR("invalid +padding=%s\n", arg);
+			return KNOT_EINVAL;
+		}
+
+		q->padding = num;
+		return KNOT_EOK;
 	}
-
-	q->padding = num;
-
-	return KNOT_EOK;
 }
 
 static int opt_nopadding(const char *arg, void *query)
 {
 	query_t *q = query;
 
-	q->padding = -1;
+	q->padding = -3;
 
 	return KNOT_EOK;
 }
@@ -739,7 +760,7 @@ static int opt_alignment(const char *arg, void *query)
 		q->alignment = DEFAULT_ALIGNMENT_SIZE;
 		return KNOT_EOK;
 	} else {
-		uint16_t num;
+		uint16_t num = 0;
 		if (str_to_u16(arg, &num) != KNOT_EOK || num < 2) {
 			ERR("invalid +alignment=%s\n", arg);
 			return KNOT_EINVAL;
@@ -774,7 +795,7 @@ static int opt_subnet(const char *arg, void *query)
 	}
 
 	// Separate address and network mask.
-	if ((sep = index(arg, '/')) != NULL) {
+	if ((sep = strchr(arg, '/')) != NULL) {
 		addr_len = sep - arg;
 	} else {
 		addr_len = arg_len;
@@ -841,7 +862,7 @@ static int opt_edns(const char *arg, void *query)
 		q->edns = 0;
 		return KNOT_EOK;
 	} else {
-		uint8_t num;
+		uint8_t num = 0;
 		if (str_to_u8(arg, &num) != KNOT_EOK) {
 			ERR("invalid +edns=%s\n", arg);
 			return KNOT_EINVAL;
@@ -992,6 +1013,9 @@ static const param_t kdig_opts2[] = {
 	{ "ttl",            ARG_NONE,     opt_ttl },
 	{ "nottl",          ARG_NONE,     opt_nottl },
 
+	{ "crypto",         ARG_NONE,     opt_crypto },
+	{ "nocrypto",       ARG_NONE,     opt_nocrypto },
+
 	{ "tcp",            ARG_NONE,     opt_tcp },
 	{ "notcp",          ARG_NONE,     opt_notcp },
 
@@ -1016,7 +1040,7 @@ static const param_t kdig_opts2[] = {
 	{ "bufsize",        ARG_REQUIRED, opt_bufsize },
 	{ "nobufsize",      ARG_NONE,     opt_nobufsize },
 
-	{ "padding",        ARG_REQUIRED, opt_padding },
+	{ "padding",        ARG_OPTIONAL, opt_padding },
 	{ "nopadding",      ARG_NONE,     opt_nopadding },
 
 	{ "alignment",      ARG_OPTIONAL, opt_alignment },
@@ -1607,6 +1631,7 @@ static void print_help(void)
 	       "       +[no]stats            Show trailing packet statistics.\n"
 	       "       +[no]class            Show DNS class.\n"
 	       "       +[no]ttl              Show TTL value.\n"
+	       "       +[no]crypto           Show binary parts of RRSIGs and DNSKEYs.\n"
 	       "       +[no]tcp              Use TCP protocol.\n"
 	       "       +[no]ignore           Don't use TCP automatically if truncated.\n"
 	       "       +[no]tls              Use TLS with Opportunistic privacy profile.\n"
@@ -1615,8 +1640,8 @@ static void print_help(void)
 	       "       +[no]tls-hostname=STR Use TLS with remote server hostname.\n"
 	       "       +[no]nsid             Request NSID.\n"
 	       "       +[no]bufsize=B        Set EDNS buffer size.\n"
-	       "       +[no]padding=N        Padding block size EDNS(0) padding.\n"
-	       "       +[no]alignment[=N]    Set packet alignment with EDNS(0) padding.\n"
+	       "       +[no]padding[=N]      Pad with EDNS(0) (default or specify size).\n"
+	       "       +[no]alignment[=N]    Pad with EDNS(0) to blocksize (%u or specify size).\n"
 	       "       +[no]subnet=SUBN      Set EDNS(0) client subnet addr/prefix.\n"
 	       "       +[no]edns[=N]         Use EDNS(=version).\n"
 	       "       +[no]time=T           Set wait for reply interval in seconds.\n"
@@ -1625,7 +1650,7 @@ static void print_help(void)
 	       "\n"
 	       "       -h, --help            Print the program help.\n"
 	       "       -V, --version         Print the program version.\n",
-	       PROGRAM_NAME);
+	       PROGRAM_NAME, DEFAULT_ALIGNMENT_SIZE);
 }
 
 static int parse_opt1(const char *opt, const char *value, kdig_params_t *params,
@@ -1922,15 +1947,18 @@ static int parse_token(const char *value, kdig_params_t *params)
 	}
 
 	// Try to guess the meaning of the token.
-	if (parse_type(value, query) == KNOT_EOK) {
+	if (strlen(value) == 0) {
+		ERR("invalid empty parameter\n");
+	} else if (parse_type(value, query) == KNOT_EOK) {
 		return KNOT_EOK;
 	} else if (parse_class(value, query) == KNOT_EOK) {
 		return KNOT_EOK;
 	} else if (parse_name(value, &params->queries, params->config) == KNOT_EOK) {
 		return KNOT_EOK;
+	} else {
+		ERR("invalid parameter: %s\n", value);
 	}
 
-	ERR("invalid parameter: %s\n", value);
 	return KNOT_EINVAL;
 }
 
