@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,12 +16,15 @@
 
 #include <stdlib.h>
 #include <getopt.h>
+#include <sys/stat.h>
 
 #include "libknot/libknot.h"
 #include "knot/journal/journal.h"
 #include "knot/zone/zone-dump.h"
 #include "utils/common/exec.h"
+#include "contrib/dynarray.h"
 #include "contrib/strtonum.h"
+#include "contrib/string.h"
 
 #define PROGRAM_NAME	"kjournalprint"
 
@@ -38,7 +41,10 @@ static void print_help(void)
 	       " -l, --limit <num>  Read only <num> newest changes.\n"
 	       " -n, --no-color     Get output without terminal coloring.\n"
 	       " -z, --zone-list    Instead of reading jurnal, display the list\n"
-	       "                    of zones in the DB (<zone_name> not needed).\n",
+	       "                    of zones in the DB (<zone_name> not needed).\n"
+	       " -d, --debug        Debug mode output.\n"
+	       " -h, --help         Print the program help.\n"
+	       " -V, --version      Print the program version.\n",
 	       PROGRAM_NAME);
 }
 
@@ -48,25 +54,136 @@ static inline char *get_rrset(knot_rrset_t *rrset, char **buff, size_t *len)
 	return (ret > 0) ? *buff : "Corrupted or missing!\n";
 }
 
-int print_journal(char *path, knot_dname_t *name, uint32_t limit, bool color)
+// workaround: LMDB fails to detect proper mapsize
+static int reconfigure_mapsize(const char *journal_path, size_t *mapsize)
 {
-	list_t db;
-	init_list(&db);
-
-	size_t buflen = 8192;
-	char *buff = malloc(buflen);
-	if (buff == NULL) {
+	char *data_mdb = sprintf_alloc("%s/data.mdb", journal_path);
+	if (data_mdb == NULL) {
 		return KNOT_ENOMEM;
 	}
 
-	journal_db_t *jdb = NULL;
-	journal_t *j = journal_new();
-	int ret;
+	struct stat st;
+	if (stat(data_mdb, &st) != 0 || st.st_size == 0) {
+		fprintf(stderr, "Journal does not exist: %s\n", journal_path);
+		free(data_mdb);
+		return KNOT_ENOENT;
+	}
 
-	ret = journal_db_init(&jdb, path, 1);
+	*mapsize = st.st_size + st.st_size / 8; // 112.5% to allow opening when growing
+	free(data_mdb);
+	return KNOT_EOK;
+}
+
+static void print_changeset(const changeset_t *chs, bool color, char **buff, size_t *buflen)
+{
+	printf(color ? YLW : "");
+	if (chs->soa_from == NULL) {
+		printf(";; Zone-in-journal, serial: %u\n",
+		       knot_soa_serial(&chs->soa_to->rrs));
+
+		printf(color ? GRN : "");
+		printf("%s", get_rrset(chs->soa_to, buff, buflen));
+		zone_dump_text(chs->add, stdout, false);
+		printf(color ? RESET : "");
+	} else {
+		printf(";; Changes between zone versions: %u -> %u\n",
+		       knot_soa_serial(&chs->soa_from->rrs),
+		       knot_soa_serial(&chs->soa_to->rrs));
+
+		printf(color ? RED : "");
+		printf(";; Removed\n");
+		printf("%s", get_rrset(chs->soa_from, buff, buflen));
+		zone_dump_text(chs->remove, stdout, false);
+
+		printf(color ? GRN : "");
+		printf(";; Added\n");
+		printf("%s", get_rrset(chs->soa_to, buff, buflen));
+		zone_dump_text(chs->add, stdout, false);
+		printf(color ? RESET : "");
+	}
+}
+
+dynarray_declare(rrtype, uint16_t, DYNARRAY_VISIBILITY_STATIC, 100)
+dynarray_define(rrtype, uint16_t, DYNARRAY_VISIBILITY_STATIC)
+
+typedef struct {
+	rrtype_dynarray_t *arr;
+	size_t *counter;
+} rrtypelist_ctx_t;
+
+static void rrtypelist_add(rrtype_dynarray_t *arr, uint16_t add_type)
+{
+	bool already_present = false;
+	dynarray_foreach(rrtype, uint16_t, i, *arr) {
+		if (*i == add_type) {
+			already_present = true;
+			break;
+		}
+	}
+	if (!already_present) {
+		rrtype_dynarray_add(arr, &add_type);
+	}
+}
+
+static int rrtypelist_callback(zone_node_t *node, void *data)
+{
+	rrtypelist_ctx_t *ctx = data;
+	for (int i = 0; i < node->rrset_count; i++) {
+		knot_rrset_t rrset = node_rrset_at(node, i);
+		rrtypelist_add(ctx->arr, rrset.type);
+		*ctx->counter += rrset.rrs.rr_count;
+	}
+	return KNOT_EOK;
+}
+
+static void print_changeset_debugmode(const changeset_t *chs)
+{
+	// detect all types
+	rrtype_dynarray_t types = { 0 };
+	size_t count_minus = 1, count_plus = 1; // 1 for SOA which is always present but not iterated
+	rrtypelist_ctx_t ctx_minus = { &types, &count_minus }, ctx_plus = { &types, &count_plus };
+	(void)zone_contents_apply(chs->remove, rrtypelist_callback, &ctx_minus);
+	(void)zone_contents_nsec3_apply(chs->remove, rrtypelist_callback, &ctx_minus);
+	(void)zone_contents_apply(chs->add, rrtypelist_callback, &ctx_plus);
+	(void)zone_contents_nsec3_apply(chs->add, rrtypelist_callback, &ctx_plus);
+
+	if (chs->soa_from == NULL) {
+		printf("Zone-in-journal %u  +++: %zu\t size: %zu\t", knot_soa_serial(&chs->soa_to->rrs),
+		       count_plus, changeset_serialized_size(chs));
+	} else {
+		printf("%u -> %u  ---: %zu\t  +++: %zu\t size: %zu\t", knot_soa_serial(&chs->soa_from->rrs),
+		       knot_soa_serial(&chs->soa_to->rrs), count_minus, count_plus, changeset_serialized_size(chs));
+	}
+
+	char temp[100];
+	dynarray_foreach(rrtype, uint16_t, i, types) {
+		(void)knot_rrtype_to_string(*i, temp, sizeof(temp));
+		printf(" %s", temp);
+	}
+	printf("\n");
+}
+
+int print_journal(char *path, knot_dname_t *name, uint32_t limit, bool color, bool debugmode)
+{
+	list_t db;
+	init_list(&db);
+	int ret;
+	journal_db_t *jdb = NULL;
+
+	ret = journal_db_init(&jdb, path, 1, JOURNAL_MODE_ROBUST);
 	if (ret != KNOT_EOK) {
-		journal_free(&j);
-		free(buff);
+		return ret;
+	}
+
+	ret = reconfigure_mapsize(jdb->path, &jdb->fslimit);
+	if (ret != KNOT_EOK) {
+		journal_db_close(&jdb);
+		return ret;
+	}
+
+	ret = journal_open_db(&jdb);
+	if (ret != KNOT_EOK) {
+		journal_db_close(&jdb);
 		return ret;
 	}
 
@@ -75,6 +192,14 @@ int print_journal(char *path, knot_dname_t *name, uint32_t limit, bool color)
 		ret = KNOT_ENOENT;
 	}
 
+	size_t buflen = 8192;
+	char *buff = malloc(buflen);
+	if (buff == NULL) {
+		journal_db_close(&jdb);
+		return KNOT_ENOMEM;
+	}
+	journal_t *j = journal_new();
+
 	if (ret == KNOT_EOK) {
 		ret = journal_open(j, &jdb, name);
 	}
@@ -93,7 +218,10 @@ int print_journal(char *path, knot_dname_t *name, uint32_t limit, bool color)
 		goto pj_finally;
 	}
 
-	ret = journal_load_changesets(j, &db, serial_from);
+	ret = journal_load_bootstrap(j, &db);
+	if (ret == KNOT_ENOENT) {
+		ret = journal_load_changesets(j, &db, serial_from);
+	}
 	if (ret != KNOT_EOK) {
 		goto pj_finally;
 	}
@@ -106,24 +234,11 @@ int print_journal(char *path, knot_dname_t *name, uint32_t limit, bool color)
 		if (--db_remains >= limit && limit > 0) {
 			continue;
 		}
-
-		printf(color ? YLW : "");
-		printf(";; Changes between zone versions: %u -> %u\n",
-		       knot_soa_serial(&chs->soa_from->rrs),
-		       knot_soa_serial(&chs->soa_to->rrs));
-
-		// Removed.
-		printf(color ? RED : "");
-		printf(";; Removed\n");
-		printf("%s", get_rrset(chs->soa_from, &buff, &buflen));
-		zone_dump_text(chs->remove, stdout, false);
-
-		// Added.
-		printf(color ? GRN : "");
-		printf(";; Added\n");
-		printf("%s", get_rrset(chs->soa_to, &buff, &buflen));
-		zone_dump_text(chs->add, stdout, false);
-		printf(color ? RESET : "");
+		if (debugmode) {
+			print_changeset_debugmode(chs);
+		} else {
+			print_changeset(chs, color, &buff, &buflen);
+		}
 	}
 
 	changesets_free(&db);
@@ -140,11 +255,23 @@ pj_finally:
 int list_zones(char *path)
 {
 	journal_db_t *jdb = NULL;
-	int ret = journal_db_init(&jdb, path, 1);
+	int ret = journal_db_init(&jdb, path, 1, JOURNAL_MODE_ROBUST);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
+	ret = reconfigure_mapsize(jdb->path, &jdb->fslimit);
+	if (ret != KNOT_EOK) {
+		journal_db_close(&jdb);
+		return ret;
+	}
+
+	ret = journal_open_db(&jdb);
+	if (ret != KNOT_EOK) {
+		journal_db_close(&jdb);
+		return ret;
+	}
+
 	list_t zones;
 	init_list(&zones);
 	ret = journal_db_list_zones(&jdb, &zones);
@@ -164,19 +291,20 @@ int list_zones(char *path)
 int main(int argc, char *argv[])
 {
 	uint32_t limit = 0;
-	bool color = true, justlist = false;
+	bool color = true, justlist = false, debugmode = false;
 
 	struct option opts[] = {
 		{ "limit",     required_argument, NULL, 'l' },
 		{ "no-color",  no_argument,       NULL, 'n' },
 		{ "zone-list", no_argument,       NULL, 'z' },
+		{ "debug",     no_argument,       NULL, 'd' },
 		{ "help",      no_argument,       NULL, 'h' },
 		{ "version",   no_argument,       NULL, 'V' },
 		{ NULL }
 	};
 
 	int opt = 0;
-	while ((opt = getopt_long(argc, argv, "l:nzhV", opts, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "l:nzdhV", opts, NULL)) != -1) {
 		switch (opt) {
 		case 'l':
 			if (str_to_u32(optarg, &limit) != KNOT_EOK) {
@@ -190,6 +318,9 @@ int main(int argc, char *argv[])
 		case 'z':
 			justlist = true;
 			break;
+		case 'd':
+			debugmode = true;
+			break;
 		case 'h':
 			print_help();
 			return EXIT_SUCCESS;
@@ -244,7 +375,7 @@ int main(int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
-	int ret = print_journal(db, name, limit, color);
+	int ret = print_journal(db, name, limit, color, debugmode);
 	free(name);
 
 	switch (ret) {
