@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -372,6 +372,16 @@ static void test_conf_io_set(void)
 	ok(conf_io_set("include", NULL, NULL, "invalid") ==
 	   KNOT_EFILE, "set invalid callback value");
 
+	// Single group, no item, no value.
+	ok(conf_io_set("server", NULL, NULL, NULL) ==
+	   KNOT_ENOTSUP, "set group no value");
+	// Single group, no item, value.
+	ok(conf_io_set("server", NULL, NULL, "text") ==
+	   KNOT_YP_ENOTSUP_DATA, "set group value");
+	// Single group, item, no value.
+	ok(conf_io_set("server", "version", NULL, NULL) ==
+	   KNOT_YP_ENODATA, "set group item no value");
+
 	// Single group, single value.
 	ok(conf_io_set("server", "version", NULL, "text") ==
 	   KNOT_EOK, "set single value");
@@ -396,6 +406,10 @@ static void test_conf_io_set(void)
 	knot_dname_t *zone3 = knot_dname_from_str_alloc(ZONE3);
 	ok(zone3 != NULL, "create dname "ZONE3);
 
+	// Multi group no id.
+	ok(conf_io_set("zone", "domain", NULL, NULL) ==
+	   KNOT_YP_ENOID, "set zone empty domain");
+
 	// Multi group ids.
 	ok(conf_io_set("zone", "domain", NULL, ZONE1) ==
 	   KNOT_EOK, "set zone domain "ZONE1);
@@ -941,7 +955,7 @@ static const yp_item_t desc_zone[] = {
 	{ NULL }
 };
 
-const yp_item_t test_scheme[] = {
+const yp_item_t test_schema[] = {
 	{ C_SRV,  YP_TGRP, YP_VGRP = { desc_server } },
 	{ C_CTL,  YP_TGRP, YP_VGRP = { desc_control } },
 	{ C_RMT,  YP_TGRP, YP_VGRP = { desc_remote }, YP_FMULTI, { check_remote } },
@@ -955,7 +969,7 @@ int main(int argc, char *argv[])
 {
 	plan_lazy();
 
-	ok(test_conf("", test_scheme) == KNOT_EOK, "Prepare configuration");
+	ok(test_conf("", test_schema) == KNOT_EOK, "Prepare configuration");
 
 	diag("conf_io_begin");
 	test_conf_io_begin();
